// Task.cpp : Implementation of CTask
#include "stdafx.h"
#include "dtspkg.h"
#include "dtstask.h"
#include "Task.h"

/////////////////////////////////////////////////////////////////////////////
// CTask

STDMETHODIMP CTask::get_Properties( 
            /* [retval][out] */ IDispatch  **pRetVal)
{
	//You can implement  a properties collection if you want or just return NULL and DTS will implement a 'PropertiesProvider' utility object
	//which will suck the properties out of your IDispatch typeinfo

	*pRetVal = NULL;
	return NOERROR;
}
        
STDMETHODIMP CTask::get_Name( 
            /* [retval][out] */ BSTR  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = SysAllocString(m_bstrName);
	if (!*pRetVal)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_Name( 
            /* [in] */ BSTR NewValue)
{
	if (m_bstrName)
		SysFreeString(m_bstrName);
	m_bstrName = SysAllocString(NewValue);
	if (!m_bstrName)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
     
STDMETHODIMP CTask::get_Description( 
            /* [retval][out] */ BSTR  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = SysAllocString(m_bstrDescription);
	if (!*pRetVal)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_Description( 
            /* [in] */ BSTR NewValue)
{
	if (m_bstrDescription)
		SysFreeString(m_bstrDescription);
	m_bstrDescription = SysAllocString(NewValue);
	if (!m_bstrDescription)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::Execute( 
            /* [in] */ IDispatch  *pPackage,
            /* [in] */ IDispatch  *pPackageEvents,
            /* [in] */ IDispatch  *pPackageLog,
            /* [out][in] */ LONG  *pTaskResult)
{
	//********************************************************************************************
	//NOTE: This sample does not properly SetErrorInfo. You should implement ISupportErrorInfo
	//and properly call SetErrorInfo on failures.
	//********************************************************************************************
	USES_CONVERSION;

	HRESULT hr = NOERROR;

	// Validate and initialize return value and output parameter pointers.
	*pTaskResult = DTSTaskExecResult_Failure; //Assume failure

	//*** BEGIN_METHOD_CODE ***

	PROCESS_INFORMATION procInfo;
	STARTUPINFO startupInfo;
	DWORD	dwTimeout;
	LPTSTR	szCommandLine;

	memset(&startupInfo, 0, sizeof(startupInfo));
	memset(&procInfo, 0, sizeof(procInfo));
	startupInfo.cb = sizeof(STARTUPINFO);

	szCommandLine = W2A(m_bstrCommandLine);

	//create process from command line
	if (!CreateProcess(NULL, szCommandLine, NULL, NULL, FALSE, CREATE_DEFAULT_ERROR_MODE, NULL, NULL, &startupInfo, &procInfo))
		return E_UNEXPECTED;

	if (m_lTimeout == 0)	//no timeout
		dwTimeout = INFINITE;
	else
		dwTimeout = 1000 * m_lTimeout;

	if (WAIT_TIMEOUT != WaitForSingleObject(procInfo.hProcess, dwTimeout))
	{
		//Process terminated
		DWORD dwExitCode;
		if (!GetExitCodeProcess(procInfo.hProcess, &dwExitCode))
			return E_UNEXPECTED;

		if (dwExitCode == (DWORD)m_lSuccessReturnCode)	//if exit code matches the desired value it is a success
			*pTaskResult = DTSTaskExecResult_Success; //Assume failure
	}
	else
	{
		if (m_bTerminateProcessAfterTimeout)	//blow the process if that is want the user wants. *Not* the default
			TerminateProcess(procInfo.hProcess, (UINT)-1);
		if (m_bFailPackageOnTimeout)	//We error this ::Execute if timeout instead of just failing task if that is what the											
			return HRESULT_FROM_WIN32(ERROR_TIMEOUT);	//This is like a cancel.
	}

	return hr;
}
        
        
STDMETHODIMP CTask::get_ProcessCommandLine( 
            /* [retval][out] */ BSTR  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = SysAllocString(m_bstrCommandLine);
	if (!*pRetVal)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_ProcessCommandLine( 
            /* [in] */ BSTR NewValue)
{
	if (m_bstrCommandLine)
		SysFreeString(m_bstrCommandLine);
	m_bstrCommandLine = SysAllocString(NewValue);
	if (!m_bstrCommandLine)
		return E_OUTOFMEMORY;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::get_SuccessReturnCode( 
            /* [retval][out] */ LONG  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = m_lSuccessReturnCode;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_SuccessReturnCode( 
            /* [in] */ LONG NewValue)
{
	m_lSuccessReturnCode = NewValue;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::get_Timeout( 
            /* [retval][out] */ LONG  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = m_lTimeout;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_Timeout( 
            /* [in] */ LONG NewValue)
{
	m_lTimeout = NewValue;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::get_TerminateProcessAfterTimeout( 
            /* [retval][out] */ VARIANT_BOOL  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = m_bTerminateProcessAfterTimeout;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::put_TerminateProcessAfterTimeout( 
            /* [in] */ VARIANT_BOOL NewValue)
{
	m_bTerminateProcessAfterTimeout = NewValue;
	return NOERROR;
}
        
        
STDMETHODIMP CTask::get_FailPackageOnTimeout( 
            /* [retval][out] */ VARIANT_BOOL  *pRetVal)
{
	if (!pRetVal)
		return E_POINTER;
	*pRetVal = m_bFailPackageOnTimeout;
	return NOERROR;
}
        

STDMETHODIMP CTask::put_FailPackageOnTimeout( 
            /* [in] */ VARIANT_BOOL NewValue)
{
	m_bFailPackageOnTimeout = NewValue;
	return NOERROR;
}
