// BCPDlg.cpp : implementation file
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#include "stdafx.h"
#include "BCP.h"
#include "BCPDlg.h"
#include "Script.h"
#include "GetFile.h"
#include "ODBCErr.h"

#include <odbcss.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg dialog
CBCPDlg::CBCPDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBCPDlg::IDD, pParent)
    {
	//{{AFX_DATA_INIT(CBCPDlg)
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
    }

void CBCPDlg::DoDataExchange(CDataExchange* pDX)
    {
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBCPDlg)
	DDX_Control(pDX, IDC_STATIC_FILE, m_staticPath);
	DDX_Control(pDX, IDC_STATIC_PROGRESS, m_staticProgress);
	DDX_Control(pDX, IDC_STATIC_FILES, m_grpFiles);
	DDX_Control(pDX, IDC_STATIC_DSN, m_staticDSN);
	DDX_Control(pDX, IDC_STATIC_CONNECTION, m_grpConnect);
	DDX_Control(pDX, IDC_PROGRESS, m_progress);
	DDX_Control(pDX, IDC_EDIT_PATH, m_editPath);
	DDX_Control(pDX, IDC_COMBO_DSN, m_comboDSN);
	DDX_Control(pDX, IDB_GO, m_btnGo);
	DDX_Control(pDX, IDB_DISCONNECT, m_btnDisconnect);
	DDX_Control(pDX, IDB_CONNECT, m_btnConnect);
	DDX_Control(pDX, IDB_BROWSE, m_btnBrowse);
	//}}AFX_DATA_MAP
    }

BEGIN_MESSAGE_MAP(CBCPDlg, CDialog)
	//{{AFX_MSG_MAP(CBCPDlg)
	ON_WM_SYSCOMMAND()
	ON_BN_CLICKED(IDB_BROWSE, OnBrowsePath)
	ON_BN_CLICKED(IDB_CLOSE, OnClose)
	ON_BN_CLICKED(IDB_CONNECT, OnConnect)
	ON_BN_CLICKED(IDB_DISCONNECT, OnDisconnect)
	ON_BN_CLICKED(IDB_GO, OnGo)
	ON_EN_CHANGE(IDC_EDIT_PATH, OnChangeEditPath)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handlers
BOOL CBCPDlg::OnInitDialog()
    {
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
    // Set our connected and file exists flags
    m_bConnected = FALSE;
    m_bCmdFileExists = FALSE;

	// Load the data source combo with available SQL Server datasources.
    TCHAR       acDSN[SQL_MAX_DSN_LENGTH + 1];
    SQLSMALLINT cbDSN;
    TCHAR       acDesc[SQL_MAX_OPTION_STRING_LENGTH + 1];
    SQLSMALLINT cbDesc;
    SQLRETURN   sRet;

    sRet = SQLDataSources(m_henv, SQL_FETCH_FIRST_USER, 
        (SQLTCHAR*) acDSN, SQL_MAX_DSN_LENGTH + 1, &cbDSN,
        (SQLTCHAR*) acDesc, SQL_MAX_OPTION_STRING_LENGTH + 1, &cbDesc);
    while (SQL_SUCCEEDED(sRet))
        {
        if (_tcsstr(acDesc, _T("SQL Server")) != NULL)
            {
            m_comboDSN.AddString(acDSN);
            }
        sRet = SQLDataSources(m_henv, SQL_FETCH_NEXT,
            (SQLTCHAR*) acDSN, SQL_MAX_DSN_LENGTH + 1, &cbDSN,
            (SQLTCHAR*) acDesc, SQL_MAX_OPTION_STRING_LENGTH + 1, &cbDesc);
        }

    // Set the selection for the data source combobox.
    m_comboDSN.SetCurSel(0);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- browse button clicked.
//
// Launch a Win32 file open dialog to collect a filename.
void CBCPDlg::OnBrowsePath() 
    {
    static TCHAR BASED_CODE szFilter[]
        = _T("Command Files (*.cmd) | *.cmd ||");
    
    CString     strFileName;
    m_editPath.GetWindowText(strFileName);

    if (!strFileName.GetLength())
        {
        strFileName = _T("*.cmd");
        }

	CFileDialog fd(TRUE, _T("cmd"), (LPCTSTR) strFileName,
                    OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY,
                    szFilter);

    if (fd.DoModal() == IDOK)
        {
        m_editPath.SetWindowText(fd.GetPathName());
        }
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- Close button clicked.
//
// If connected, disconnect before forwarding message to class handler.
void CBCPDlg::OnClose() 
    {
    // If connected, disconnect, then CDialog::OnCancel processing.
    if (m_bConnected)
        {
	    OnDisconnect();
        }

	CDialog::OnCancel();
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- SYSCOMMAND (messages from the system
//  menu)
//
// If the message is SC_CLOSE (user pressed the X, chose close from
//  the system menu, or pressed [Alt-F4]) and we're connected, then
//  disconnect. Let the dialog's handler see all messages.
void CBCPDlg::OnSysCommand(UINT nID, LPARAM lParam)
    {
	if (nID == SC_CLOSE && m_bConnected)
	    {
	    OnDisconnect();
	    }

    CDialog::OnSysCommand(nID, lParam);
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- Connect button generated BN_CLICK
//
// Use SQLDriverConnect to attempt a connection to the indicated datasource.
void CBCPDlg::OnConnect() 
    {
    CString     strDatasource;
    CString     strConnect = _T("DSN=");

    // Get the datasource selected by the user.
    m_comboDSN.GetLBText(m_comboDSN.GetCurSel(), strDatasource);
    strConnect += strDatasource;
    strConnect += _T(";");

    // If this fails, there's not a lot we can do.
    if (SQLAllocHandle(SQL_HANDLE_DBC, m_henv, &m_hdbc) != SQL_SUCCESS)
        {
        AfxMessageBox(IDS_ERR_ALLOCHDBC);
        return;
        }

    CWaitCursor     wait;

    // Need to set this prior to connection.
    SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_BCP, (void*) SQL_BCP_ON,
        SQL_IS_INTEGER);

    switch (SQLDriverConnect(m_hdbc, AfxGetMainWnd()->m_hWnd,
        (SQLTCHAR*) ((LPCTSTR) strConnect), SQL_NTS, NULL, 0, NULL,
        SQL_DRIVER_COMPLETE))
        {
        case SQL_SUCCESS:
        case SQL_SUCCESS_WITH_INFO:
            {
            m_bConnected = TRUE;

            // Set buttons.
            SetConnectedButtons();
            break;
            }

        case SQL_NO_DATA:
            {
            // User canceled or other event for which we have no
            //  information.
            break;
            }

        default:
            {
            DisplayErrors(SQL_HANDLE_DBC, m_hdbc);

            SQLFreeHandle(SQL_HANDLE_DBC, m_hdbc);
            m_hdbc = NULL;
            }
        }
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- Disconnect button generated BN_CLICK
//
// Disconnect and free the ODBC connection handle. Set buttons.
void CBCPDlg::OnDisconnect() 
    {
	SQLDisconnect(m_hdbc);
    SQLFreeHandle(SQL_HANDLE_DBC, m_hdbc);
    m_hdbc = NULL;
    m_bConnected = FALSE;

    // Set buttons
    SetConnectedButtons();
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- Go button generated BN_CLICK
//
// Attempt to open the command file and process what's within.
void CBCPDlg::OnGo() 
    {
    int         cmdToke;
    CString     strCmdFile;
    int         idxBackSlash;
    PSTR        szAllCommands = NULL;
    PSTR        szCurCommand;
    SQLRETURN   sRet = SQL_SUCCESS;

    m_editPath.GetWindowText(strCmdFile);

    idxBackSlash = strCmdFile.ReverseFind(_T('\\'));

    if (idxBackSlash)
        {
        // Add one for the slash...
        idxBackSlash++;
        // ...one for the terminator
        m_szPath = new TCHAR[idxBackSlash + 1];

        _tcsnccpy(m_szPath, (LPCTSTR) strCmdFile, idxBackSlash);
        m_szPath[idxBackSlash] = (TCHAR) NULL;
        }
    else
        {
        m_szPath = new TCHAR[1];
        m_szPath[0] = (TCHAR) NULL;
        }
    
    SetGoButtons(TRUE);
    m_szArgs = NULL;

    // Get the command string...
    if (GetOSFile((LPCTSTR) strCmdFile, (PBYTE*) &szAllCommands) == TRUE)
        {
        m_progress.SetStep(11);

        szCurCommand = szAllCommands;
        while (SQL_SUCCEEDED(sRet) && 
            (cmdToke = GetNextCmd(&szCurCommand, &m_szArgs)) != EOF)
            {
            switch (cmdToke)
                {
                case PROGRESSTEXT:
                    {
                    m_staticProgress.SetWindowText(m_szArgs);
                    break;
                    }
            
                case SCRIPTRUN:
                    {
                    if (SQLAllocHandle(SQL_HANDLE_STMT, m_hdbc, &m_hstmt) ==
                            SQL_SUCCESS)
                        {
                        sRet = ScriptRun();
                        SQLFreeHandle(SQL_HANDLE_STMT, m_hstmt);
                        }

                    m_progress.StepIt();
                    break;
                    }

                case BCPTOKE:
                    {
                    if (!SQL_SUCCEEDED(sRet = BCPData()))
                        {
                        DisplayErrors(SQL_HANDLE_DBC, m_hdbc);
                        }

                    m_progress.StepIt();
                    break;
                    }

                case BADCOMMAND:
                    {
                    AfxMessageBox(IDS_ERR_UNKNOWN_CMD);

                    goto SafeExit;
                    }
                }
            
            delete [] m_szArgs;
            m_szArgs = NULL;
            }
        }

SafeExit:
    if (szAllCommands)
        delete [] szAllCommands;

    if (m_szArgs)
        {
        delete [] m_szArgs;
        m_szArgs = NULL;
        }

    if (m_szPath)
        {
        delete [] m_szPath;
        m_szPath = NULL;
        }

    m_progress.SetPos(0);
    SetGoButtons(FALSE);
    }

/////////////////////////////////////////////////////////////////////////////
// CBCPDlg message handler -- text in the path/filename edit has changed.
void CBCPDlg::OnChangeEditPath() 
    {
    SetReadyButtons();
    }

/////////////////////////////////////////////////////////////////////////////
// SetConnectedButtons -- When disconnected, the Connect, Browse and Close
//  buttons are enabled. Go and Disconnect are disabled. Connecting
//  reverses the enabled state of the Disconnect and connect buttons.
void CBCPDlg::SetConnectedButtons()
    {
    m_btnDisconnect.EnableWindow(m_bConnected);

    m_btnConnect.EnableWindow(!m_bConnected);
    m_staticDSN.EnableWindow(!m_bConnected);
    m_comboDSN.EnableWindow(!m_bConnected);

    SetReadyButtons();
    }

/////////////////////////////////////////////////////////////////////////////
// SetReadyButtons -- If connected and there's text in the edit, then
//  enable the "Go" button.
void CBCPDlg::SetReadyButtons()
    {
    if (m_editPath.GetWindowTextLength())
        {
        if (m_bCmdFileExists == FALSE)
            {
            m_bCmdFileExists = TRUE;
            }
        }
    else
        {
        if (m_bCmdFileExists == TRUE)
            {
            m_bCmdFileExists = FALSE;
            }
        }

    m_btnGo.EnableWindow(m_bConnected && m_bCmdFileExists);
    }

/////////////////////////////////////////////////////////////////////////////
// SetGoButtons -- When Go is pressed, any changes are disallowed until
//  completion of command processing.
void CBCPDlg::SetGoButtons(BOOL bEnabled)
    {
    m_grpConnect.EnableWindow(!bEnabled);
    m_btnDisconnect.EnableWindow(!bEnabled);
    m_grpFiles.EnableWindow(!bEnabled);
    m_staticPath.EnableWindow(!bEnabled);
    m_editPath.EnableWindow(!bEnabled);
    m_btnBrowse.EnableWindow(!bEnabled);

    m_btnGo.EnableWindow(!bEnabled);
    }

/////////////////////////////////////////////////////////////////////////////
// DisplayErrors -- Build a message box string from the error list and
//  display it.
void CBCPDlg::DisplayErrors
    (
    SQLSMALLINT eHandleType,
    SQLHANDLE hodbc
    )
    {
    CODBCErrList*   plistErrors;
    CString         strAllErrors;
    CString         strError;
    UINT            nErr;
    UINT            nErrs = 4;          // Number of errors limit.

    plistErrors = new CODBCErrList(eHandleType, hodbc);
    if (plistErrors->GetReturnCode() == INVALID_OBJECT)
        {
        AfxMessageBox(IDS_INVALID_HANDLE_AT_ERROR);
        delete plistErrors;
        return;
        }

    plistErrors->FormatHeader(strAllErrors);
    nErrs = min(nErrs, plistErrors->GetCount());
    for (nErr = 0; nErr < nErrs; nErr++)
        {
        plistErrors->FormatErr(strError, nErr + 1);
        strAllErrors += _T("\n\n");
        strAllErrors += strError;
        }

    if (nErrs < plistErrors->GetCount())
        {
        strError.LoadString(IDS_ERR_TOOMANYERRORS);
        strAllErrors += _T("\n\n");
        strAllErrors += strError;
        }
    
    AfxMessageBox(strAllErrors);

    delete plistErrors;
    }

/////////////////////////////////////////////////////////////////////////////
// BCPData -- Use bcp_init, bcp_exec to process the BCP components of the
//  command script.
SQLRETURN CBCPDlg::BCPData()
    {
    BOOL        bIdentityInsert = FALSE;
    PTSTR       szTableName;
    UINT        cbTableName;
    PTSTR       szDataFile;
    UINT        cbDataFile;
    PTSTR       szErrFile;
    UINT        cbErrFile;
    PTSTR       szDirection;
    INT         iDirection;
    DBINT       nRowsCopied;
    
    szTableName = _tcstok(m_szArgs, _T(","));
    szDataFile  = _tcstok(NULL, _T(","));
    szErrFile   = _tcstok(NULL, _T(","));
    szDirection = _tcstok(NULL, _T(","));
    
    cbTableName = TrimQuotes(szTableName, szTableName);
    cbDataFile  = TrimQuotes(szDataFile,  szDataFile);
    cbErrFile   = TrimQuotes(szErrFile,   szErrFile);

    // Check for the basics...
    if (!cbTableName || !cbDataFile)
        {
        return (SQL_ERROR);
        }

    if (_tcsicmp(szDirection, _T("IN")) == 0)
        {
        iDirection = DB_IN;
        }
    else if (_tcsicmp(szDirection, _T("OUT")) == 0)
        {
        iDirection = DB_OUT;
        }
    else
        {
        return (SQL_ERROR);
        }

    // ...we can continue without an error file, logging's not performed.
    if (!cbErrFile)
        {
        szErrFile = NULL;
        }

    // Check for an identity column in the table and turn it off if
    //  our column list includes the identity column.
    bIdentityInsert = bTableHasIdentity(szTableName);

    if (bcp_init(m_hdbc, szTableName, szDataFile, szErrFile, iDirection)
        == SUCCEED)
        {
        bcp_control(m_hdbc, BCP6xFILEFMT, (void *)TRUE);

        if (bIdentityInsert == TRUE)
            {
            bcp_control(m_hdbc, BCPKEEPIDENTITY, (void *)TRUE);
            }

        // If we succeed, BCPing the data may have made an empty error file,
        //  be nice and delete it.
        if (bcp_exec(m_hdbc, &nRowsCopied) == SUCCEED)
            {
            if (szErrFile != NULL)
                {
                DeleteFile(szErrFile);
                }
            }
        else 
            {
            return (SQL_ERROR);
            }
        }
    else 
        {
        return (SQL_ERROR);
        }

    return (SQL_SUCCESS);
    }

/////////////////////////////////////////////////////////////////////////////
// bTableHasIdentity -- determine whether a table has an identity column or
//  not. We assume that the presence of the identity in the table indicates
//  that the data to be BCP'd to the table also contains data for the
//  identity column.
BOOL CBCPDlg::bTableHasIdentity
    (
    LPCTSTR szTableName
    )
    {
    SQLUSMALLINT nCols;
    SQLUSMALLINT nCol;
    SQLINTEGER   nValue;
    
    CString     strCommand;

    BOOL        bIdentity = FALSE;

    if (SQLAllocHandle(SQL_HANDLE_STMT, m_hdbc, &m_hstmt) != SQL_SUCCESS)
        {
        return (bIdentity);
        }

    strCommand.Format(IDS_SELECT_ALL, szTableName);
    
    if (!SQL_SUCCEEDED(SQLPrepare(m_hstmt, (SQLTCHAR*) ((LPCTSTR) strCommand),
        SQL_NTS)))
        {
        DisplayErrors(SQL_HANDLE_STMT, m_hstmt);
        goto SAFERETURN;
        }

    if (!SQL_SUCCEEDED(SQLNumResultCols(m_hstmt, (SQLSMALLINT*) &nCols)))
        {
        DisplayErrors(SQL_HANDLE_STMT, m_hstmt);
        goto SAFERETURN;
        }

    for (nCol = 0; nCol < nCols && bIdentity == FALSE; nCol++)
        {
        // Determine identity property of column.
        if (SQL_SUCCEEDED(SQLColAttribute(m_hstmt, nCol + 1,
                (SQLUSMALLINT) SQL_DESC_AUTO_UNIQUE_VALUE, NULL, 0, 
                NULL, (SQLPOINTER) &nValue)))
            {
            bIdentity = (nValue ? TRUE : FALSE);
            }
        }

SAFERETURN:
    SQLFreeHandle(SQL_HANDLE_STMT, m_hstmt);
    return (bIdentity);
    }

/////////////////////////////////////////////////////////////////////////////
// ScriptRun -- executes a batch of Transact SQL statements one at a time.
//  The function assumes that the statements are DDL or other statements
//  that do not return results and it does not include any data retrieval
//  mechanism.
// 
// Returns SQL_SUCCESS if all statements execute correctly. SQL_ERROR on 
//  execution error or failure to find or read the command batch file.
SQLRETURN CBCPDlg::ScriptRun()
    {
    PSTR        szScript;
    PSTR        szAllLines;
    PSTR        szLine;
    CString     strPathAndFile = m_szPath;
    CString     strCmd = _T("");
    SQLRETURN   sRet = SQL_SUCCESS;
    
    if (TrimQuotes(m_szArgs, m_szArgs) == 0)
        {
        return (SQL_ERROR);
        }

    strPathAndFile += m_szArgs;
    if (GetOSFile((LPCTSTR) strPathAndFile, (PBYTE*) &szScript) == FALSE)
        {
        return (SQL_ERROR);
        }

    szAllLines = szScript;
    while (SQL_SUCCEEDED(sRet) && 
        GetNextLine(&szAllLines,  &szLine) != EOF)
        {
        if (stricmp(szLine, "GO") == 0)
            {
            if (strCmd.GetLength())
                {
                if (!SQL_SUCCEEDED(sRet = SQLExecDirect(m_hstmt,
                    (SQLTCHAR*) (LPCTSTR) strCmd, strCmd.GetLength())))
                    {
                    DisplayErrors(SQL_HANDLE_STMT, m_hstmt);
                    }

                while (SQL_SUCCEEDED(sRet = SQLMoreResults(m_hstmt)))
                    {
                    if (sRet == SQL_SUCCESS_WITH_INFO)
                        {
                        DisplayErrors(SQL_HANDLE_STMT, m_hstmt);
                        }
                    }

                if (sRet == SQL_NO_DATA)
                    {
                    sRet = SQL_SUCCESS;
                    }

                strCmd = _T("");
                }
            }
        else
            {
            if (strCmd.GetLength())
                {
                strCmd += _T(" ");
                }
            strCmd += szLine;
            }

        delete [] szLine;
        }

    delete [] szScript;

    return (SQL_SUCCEEDED(sRet) ? SQL_SUCCESS : sRet);
    }

