if exists (select * from sysobjects where id = object_id('dbo.[Order Details]'))
    drop table dbo.[Order Details]
go
if exists (select * from sysobjects where id = object_id('dbo.Orders'))
    drop table dbo.Orders
go
if exists (select * from sysobjects where id = object_id('dbo.Shippers'))
    drop table dbo.Shippers
go
if exists (select * from sysobjects where id = object_id('dbo.Products'))
    drop table dbo.Products
go
if exists (select * from sysobjects where id = object_id('dbo.Suppliers'))
    drop table dbo.Suppliers
go
if exists (select * from sysobjects where id = object_id('dbo.Employees'))
    drop table dbo.Employees
go
if exists (select * from sysobjects where id = object_id('dbo.Customers'))
    drop table dbo.Customers
go
if exists (select * from sysobjects where id = object_id('dbo.Categories'))
    drop table dbo.Categories
go

CREATE TABLE Categories
(
    CategoryID          int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    CategoryName        varchar(15)     NOT NULL,
    Description         text            NULL,
    Picture             image           NULL
)
go
GRANT ALL ON Categories TO public
go

CREATE TABLE Customers
(
    CustomerID          char(5)
        PRIMARY KEY CLUSTERED,
    CompanyName         nvarchar(40)    NOT NULL,
    ContactName         nvarchar(30)    NULL,
    ContactTitle        nvarchar(30)    NULL,
    Address             nvarchar(60)    NULL,
    City                nvarchar(15)    NULL,
    Region              nvarchar(15)    NULL,
    PostalCode          varchar(10)     NULL,
    Country             nvarchar(15)    NULL,
    Phone               varchar(24)     NULL,
    Fax                 varchar(24)     NULL
)
go
GRANT ALL ON Customers TO public
go

CREATE TABLE Employees
(
    EmployeeID          int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    LastName            nvarchar(20)    NOT NULL,
    FirstName           nvarchar(20)    NOT NULL,
    Title               nvarchar(30)    NULL,
    TitleOfCourtesy     nvarchar(25)    NULL,
    BirthDate           datetime        NULL,
    HireDate            datetime        NULL,
    Address             nvarchar(60)    NULL,
    City                nvarchar(15)    NULL,
    Region              nvarchar(15)    NULL,
    PostalCode          varchar(10)     NULL,
    Country             nvarchar(15)    NULL,
    HomePhone           varchar(24)     NULL,
    Extension           varchar(4)      NULL,
    Photo               image           NULL,
    Notes               ntext           NULL,
    ReportsTo           int             NULL
)
go
GRANT ALL ON Employees TO public
go

CREATE TABLE Suppliers
(
    SupplierID          int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    CompanyName         nvarchar(40)    NOT NULL,
    ContactName         nvarchar(30)    NULL,
    ContactTitle        nvarchar(30)    NULL,
    Address             nvarchar(60)    NULL,
    City                nvarchar(15)    NULL,
    Region              nvarchar(15)    NULL,
    PostalCode          varchar(10)     NULL,
    Country             nvarchar(15)    NULL,
    Phone               varchar(24)     NULL,
    Fax                 varchar(24)     NULL,
    HomePage            text
)
go
GRANT ALL ON Suppliers TO public
go

CREATE TABLE Products
(
    ProductID           int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    ProductName         nvarchar(40)    NOT NULL,
    SupplierID          int             NULL
        REFERENCES Suppliers(SupplierID),
    CategoryID          int             NULL
        REFERENCES Categories(CategoryID),
    QuantityPerUnit     varchar(20)     NULL,
    UnitPrice           money           NULL,
    UnitsInStock        int             NULL,
    UnitsOnOrder        int             NULL,
    ReorderLevel        int             NULL,
    Discontinued        bit             NOT NULL
        DEFAULT	0
)
go
GRANT ALL ON Products TO public
go

CREATE TABLE Shippers
(
    ShipperID           int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    CompanyName         nvarchar(40)    NOT NULL,
    Phone               varchar(24)     NULL
)
go
GRANT ALL ON Shippers TO public
go

CREATE TABLE Orders
(
    OrderID             int
        IDENTITY(1,1)
        PRIMARY KEY CLUSTERED,
    CustomerID          char(5)         NOT NULL
        REFERENCES Customers(CustomerID),
    EmployeeID          int             NOT NULL
        REFERENCES Employees(EmployeeID),
    OrderDate           datetime        NOT NULL
        DEFAULT (getdate()),
    RequiredDate        datetime        NULL,
    ShippedDate         datetime        NULL,
    ShipVia             int             NULL
        REFERENCES Shippers(ShipperID),
    Freight             money           NULL,
    ShipName            nvarchar(40)    NULL,
    ShipAddress         nvarchar(60)    NULL,
    ShipCity            nvarchar(15)    NULL,
    ShipRegion          nvarchar(15)    NULL,
    ShipPostalCode      varchar(10)     NULL,
    ShipCountry         nvarchar(15)    NULL
)
go
GRANT ALL ON Orders TO public
go

CREATE TABLE [Order Details]
(
    OrderID             int             NOT NULL
        REFERENCES Orders(OrderID),
    ProductID           int             NOT NULL
        REFERENCES Products(ProductID)
        CONSTRAINT PK_OrderDetails PRIMARY KEY CLUSTERED (OrderID, ProductID),
    UnitPrice           money           NOT NULL,
    Quantity            int             NOT NULL,
    Discount            decimal(2,2)    NOT NULL
        DEFAULT 0
        CHECK (Discount >= 0 AND Discount <= .25)
)
go
GRANT ALL ON [Order Details] TO public
go

if exists (select * from sysobjects where id = object_id('dbo.CustOrdersDetail'))
    drop procedure dbo.CustOrdersDetail
GO

CREATE PROCEDURE CustOrdersDetail @OrderID int
AS
SELECT ProductName,
    UnitPrice=ROUND(Od.UnitPrice, 2),
    Quantity,
    Discount=CONVERT(int, Discount * 100), 
    ExtendedPrice=ROUND(CONVERT(money, Quantity * (1 - Discount) * Od.UnitPrice), 2)
FROM Products P, [Order Details] Od
WHERE Od.ProductID = P.ProductID and Od.OrderID = @OrderID
go

GRANT  EXECUTE  ON dbo.CustOrdersDetail  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.CustOrdersOrders'))
	drop procedure dbo.CustOrdersOrders
GO

CREATE PROCEDURE CustOrdersOrders @CustomerID char(5)
AS
SELECT OrderID, 
	OrderDate,
	RequiredDate,
	ShippedDate
FROM Orders
WHERE CustomerID = @CustomerID
ORDER BY OrderID
GO

GRANT EXECUTE ON dbo.CustOrdersOrders TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.CustOrderHist') and sysstat & 0xf = 4)
	drop procedure dbo.CustOrderHist
GO
CREATE PROCEDURE CustOrderHist @CustomerID char(5)
AS
SELECT ProductName, Total=SUM(Quantity)
FROM Products P, [Order Details] OD, Orders O, Customers C
WHERE C.CustomerID = @CustomerID
AND C.CustomerID = O.CustomerID AND O.OrderID = OD.OrderID AND OD.ProductID = P.ProductID
GROUP BY ProductName
GO
GRANT EXECUTE ON dbo.CustOrderHist TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.SalesByCategory') and sysstat & 0xf = 4)
	drop procedure dbo.SalesByCategory
GO
CREATE PROCEDURE SalesByCategory
    @CategoryName varchar(15), @OrdYear varchar(4) = '1996'
AS
IF @OrdYear != '1994' AND @OrdYear != '1995' AND @OrdYear != '1996' 
BEGIN
	SELECT @OrdYear = '1996'
END

SELECT ProductName,
	TotalPurchase=ROUND(SUM(CONVERT(decimal(14,2), OD.Quantity * (1-OD.Discount) * OD.UnitPrice)), 0)
FROM [Order Details] OD, Orders O, Products P, Categories C
WHERE OD.OrderID = O.OrderID 
	AND OD.ProductID = P.ProductID 
	AND P.CategoryID = C.CategoryID
	AND C.CategoryName = @CategoryName
	AND SUBSTRING(CONVERT(varchar(22), O.OrderDate, 111), 1, 4) = @OrdYear
GROUP BY ProductName
ORDER BY ProductName
GO
GRANT  EXECUTE  ON dbo.SalesByCategory  TO public
GO

