if exists (select * from sysobjects where id = object_id('dbo.OrderDetails'))
	drop table dbo.OrderDetails
go
if exists (select * from sysobjects where id = object_id('dbo.Orders'))
	drop table dbo.Orders
go
if exists (select * from sysobjects where id = object_id('dbo.Shippers'))
	drop table dbo.Shippers
go
if exists (select * from sysobjects where id = object_id('dbo.Products'))
	drop table dbo.Products
go
if exists (select * from sysobjects where id = object_id('dbo.Suppliers'))
	drop table dbo.Suppliers
go
if exists (select * from sysobjects where id = object_id('dbo.Employees'))
	drop table dbo.Employees
go
if exists (select * from sysobjects where id = object_id('dbo.Customers'))
	drop table dbo.Customers
go
if exists (select * from sysobjects where id = object_id('dbo.Categories'))
	drop table dbo.Categories
go

CREATE TABLE Categories
(
	CategoryID	int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	CategoryName	varchar(15)	NOT NULL,
	Description	text		NULL,
	Picture		image		NULL
)
go
GRANT ALL ON Categories TO PUBLIC
go

CREATE TABLE Customers
(
	CustomerID	char(5)
		PRIMARY KEY CLUSTERED,
	CompanyName	varchar(40)	NOT NULL,
	ContactName	varchar(30)	NULL,
	ContactTitle	varchar(30)	NULL,
	Address		varchar(60)	NULL,
	City		varchar(15)	NULL,
	Region		varchar(15)	NULL,
	PostalCode	varchar(10)	NULL,
	Country		varchar(15)	NULL,
	Phone		varchar(24)	NULL,
	Fax		varchar(24)	NULL
)
go
GRANT ALL ON Customers TO PUBLIC
go

CREATE TABLE Employees
(
	EmployeeID	int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	LastName	varchar(20)	NOT NULL,
	FirstName	varchar(20)	NOT NULL,
	Title		varchar(30)	NULL,
	TitleOfCourtesy	varchar(25)	NULL,
	BirthDate	datetime	NULL,
	HireDate	datetime	NULL,
	Address		varchar(60)	NULL,
	City		varchar(15)	NULL,
	Region		varchar(15)	NULL,
	PostalCode	varchar(10)	NULL,
	Country		varchar(15)	NULL,
	HomePhone	varchar(24)	NULL,
	Extension	varchar(4)	NULL,
	Photo		image		NULL,
	Notes		text		NULL,
	ReportsTo	int		NULL
)
go
GRANT ALL ON Employees TO PUBLIC
go

CREATE TABLE Suppliers
(
	SupplierID	int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	CompanyName	varchar(40)	NOT NULL,
	ContactName	varchar(30)	NULL,
	ContactTitle	varchar(30)	NULL,
	Address		varchar(60)	NULL,
	City		varchar(15)	NULL,
	Region		varchar(15)	NULL,
	PostalCode	varchar(10)	NULL,
	Country		varchar(15)	NULL,
	Phone		varchar(24)	NULL,
	Fax		varchar(24)	NULL,
	HomePage	text
)
go
GRANT ALL ON Suppliers TO PUBLIC
go

CREATE TABLE Products
(
	ProductID	int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	ProductName	varchar(40)	NOT NULL,
	SupplierID	int		NULL
		REFERENCES Suppliers(SupplierID),
	CategoryID	int		NULL
		REFERENCES Categories(CategoryID),
	QuantityPerUnit	varchar(20)	NULL,
	UnitPrice	money		NULL,
	UnitsInStock	int		NULL,
	UnitsOnOrder	int		NULL,
	ReorderLevel	int		NULL,
	Discontinued	bit
		DEFAULT	0
)
go
GRANT ALL ON Products TO PUBLIC
go

CREATE TABLE Shippers
(
	ShipperID	int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	CompanyName	varchar(40)	NOT NULL,
	Phone		varchar(24)	NULL
)
go
GRANT ALL ON Shippers TO PUBLIC
go

CREATE TABLE Orders
(
	OrderID		int
		IDENTITY(1,1)
		PRIMARY KEY CLUSTERED,
	CustomerID	char(5)		NOT NULL
		REFERENCES Customers(CustomerID),
	EmployeeID	int		NOT NULL
		REFERENCES Employees(EmployeeID),
	OrderDate	datetime	NOT NULL
		DEFAULT (getdate()),
	RequiredDate	datetime	NULL,
	ShippedDate	datetime	NULL,
	ShipVia		int		NULL
		REFERENCES Shippers(ShipperID),
	Freight		money		NULL,
	ShipName	varchar(40)	NULL,
	ShipAddress	varchar(60)	NULL,
	ShipCity	varchar(15)	NULL,
	ShipRegion	varchar(15)	NULL,
	ShipPostalCode	varchar(10)	NULL,
	ShipCountry	varchar(15)	NULL
)
go
GRANT ALL ON Orders TO PUBLIC
go

CREATE TABLE OrderDetails
(
	OrderID		int		NOT NULL
		REFERENCES Orders(OrderID),
	ProductID	int		NOT NULL
		REFERENCES Products(ProductID)
		CONSTRAINT PK_OrderDetails PRIMARY KEY CLUSTERED (OrderID, ProductID),
	UnitPrice	money		NOT NULL,
	Quantity	int		NOT NULL,
	Discount	decimal(2,2)	NOT NULL
		DEFAULT 0
		CHECK (Discount >= 0 AND Discount <= .25)
)
go
GRANT ALL ON OrderDetails TO PUBLIC
go

if exists (select * from sysobjects where id = object_id('dbo.CustOrdersDetail'))
	drop procedure dbo.CustOrdersDetail
GO

CREATE PROCEDURE CustOrdersDetail @order_id int
AS
select ProductName,
	UnitPrice=Round(Od.UnitPrice, 2),
	Quantity,
	Discount=Convert(int, Discount * 100), 
	ExtendedPrice=Round(Convert(money, Quantity * (1 - Discount) * Od.UnitPrice), 2)
from Products P, OrderDetails Od
where Od.ProductID = P.ProductID and Od.OrderID = @order_id
GO

GRANT  EXECUTE  ON dbo.CustOrdersDetail  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.CustOrdersOrders'))
	drop procedure dbo.CustOrdersOrders
GO

CREATE PROCEDURE CustOrdersOrders @cust_id char(5)
AS
SELECT OrderID, 
	OrderDate,
	RequiredDate,
	ShippedDate
FROM Orders
WHERE CustomerID = @cust_id
ORDER BY OrderID
GO

GRANT  EXECUTE  ON dbo.CustOrdersOrders  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.InsertOrder'))
	drop procedure dbo.InsertOrder
GO

CREATE PROCEDURE InsertOrder
	@OrderID int, @CustomerID char(5), @EmployeeID int, @OrderDate datetime,
	@RequiredDate datetime, @ShippedDate datetime = NULL, @ShipVia int, @Freight money,
	@ShipName varchar(40), @ShipAddress varchar(60), @ShipCity varchar(15),
	@ShipRegion varchar(15), @ShipPostalCode varchar(10), @ShipCountry varchar(15)
AS
INSERT INTO Orders (OrderID, CustomerID, EmployeeID, OrderDate, RequiredDate, ShippedDate,
			ShipVia, Freight, ShipName, ShipAddress, ShipCity, ShipRegion,
			ShipPostalCode, ShipCountry)
VALUES (@OrderID, @CustomerID,@EmployeeID, @OrderDate, @RequiredDate, @ShippedDate,
			@ShipVia, @Freight, @ShipName, @ShipAddress, @ShipCity, @ShipRegion,
			@ShipPostalCode, @ShipCountry)

GO

GRANT  EXECUTE  ON dbo.InsertOrder  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.InsertOrderDetail'))
	drop procedure dbo.InsertOrderDetail
GO
CREATE PROCEDURE InsertOrderDetail 
	@OrderID int, @ProductID int, @UnitPrice money, @Quantity int, @Discount float = 0
AS
INSERT INTO OrderDetails (OrderID, ProductID, UnitPrice, Quantity, Discount)
VALUES (@OrderID, @ProductID, @UnitPrice, @Quantity, @Discount)
GO
GRANT  EXECUTE  ON dbo.InsertOrderDetail  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.CustOrderHist') and sysstat & 0xf = 4)
	drop procedure dbo.CustOrderHist
GO
CREATE PROCEDURE CustOrderHist
	@CustomerID char(5)
AS
SELECT ProductName, Sum(Quantity)
FROM Products P, OrderDetails OD, Orders O, Customers C
WHERE C.CustomerID = @CustomerID
AND C.CustomerID = O.CustomerID AND O.OrderID = OD.OrderID AND OD.ProductID = P.ProductID
GROUP BY ProductName
GO
GRANT  EXECUTE  ON dbo.CustOrderHist  TO public
GO

if exists (select * from sysobjects where id = object_id('dbo.SalesByCategory') and sysstat & 0xf = 4)
	drop procedure dbo.SalesByCategory
GO
CREATE PROCEDURE SalesByCategory @CategoryName varchar(15), @OrdYear varchar(4) = '1996' AS
IF @OrdYear != '1994' AND @OrdYear != '1995' AND @OrdYear != '1996' 
BEGIN
	SELECT @OrdYear = '1996'
END

SELECT ProductName,
	TotalPurchase=ROUND(SUM(CONVERT(decimal(14,2), OD.Quantity * (1-OD.Discount) * OD.UnitPrice)), 0)
FROM OrderDetails OD, Orders O, Products P, Categories C
WHERE OD.OrderID = O.OrderID 
	AND OD.ProductID = P.ProductID 
	AND P.CategoryID = C.CategoryID
	AND C.CategoryName = @CategoryName
	AND SUBSTRING(CONVERT(varchar(22), O.OrderDate, 111), 1, 4) = @OrdYear
GROUP BY ProductName
ORDER BY ProductName
GO
GRANT  EXECUTE  ON dbo.SalesByCategory  TO public
GO

