// ColAttr.h -- Column attributes class implementation. The class provides
//  an interface to column data retrieved from an executed SQL Server ODBC
//  statement.
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#ifndef __COLATTR_H__
#define __COLATTR_H__

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "sql.h"
#include "sqlext.h"
#include "odbcss.h"

// Macros
#define MAX_COLUMN_NAME     128

/////////////////////////////////////////////////////////////////////////////
// CColAttr -- Aattributes of an ODBC returned data column.
class CColAttr : public CObject
{
DECLARE_DYNAMIC(CColAttr)

// Construction
public:
    CColAttr(SQLHSTMT hStmt, SQLSMALLINT nCol);

private:
protected:

// Implementation
public:
    SQLRETURN   GetAttr();
    UINT        GetBoundBufferWidth();
    UINT        GetCharBufferWidth();
    SQLSMALLINT GetDecimalDigits();
    SQLSMALLINT GetDefaultBindType();
    SQLUINTEGER GetDisplaySize();
    BOOL        GetIsIdentity();
    LPCTSTR     GetName();
    UINT        GetNameLen();
    SQLSMALLINT GetODBCType();
    SQLUINTEGER GetPrecision();

    virtual ~CColAttr();

private:
    SQLHSTMT    m_hstmt;
    UINT        m_nCol;
    CString*    m_psColName;
    SQLSMALLINT m_eSQLType;
    SQLSMALLINT m_eDefaultBindType;
    UINT        m_cbDefBuffer;
    UINT        m_cbCharBuffer;
    SQLUINTEGER m_cbDisplay;
    SQLSMALLINT m_nDecimalDigits;
    SQLSMALLINT m_bNullable;
    SQLINTEGER  m_bIsIdentity;

private:
    UINT AdjustLen(UINT cbColData);

public:
#ifdef _DEBUG
    virtual void Dump(CDumpContext& dc) const;
    virtual void AssertValid() const;
#endif
};

#ifndef _DEBUG // inline versions of data member access functions
_inline UINT CColAttr::GetBoundBufferWidth()
    {
    return (m_cbDefBuffer);
    };

_inline UINT CColAttr::GetCharBufferWidth()
    {
    return (m_cbCharBuffer);
    };

_inline SQLSMALLINT CColAttr::GetDecimalDigits()
    {
    return (m_nDecimalDigits);
    };

_inline SQLSMALLINT CColAttr::GetDefaultBindType()
    {
    return (m_eDefaultBindType);
    };

_inline SQLUINTEGER CColAttr::GetDisplaySize()
    {
    return (m_cbDisplay);
    };

_inline BOOL CColAttr::GetIsIdentity()
    {
    return ((BOOL) m_bIsIdentity);
    };

_inline LPCTSTR CColAttr::GetName()
    {
    return ((LPCTSTR) *m_psColName);
    };

_inline UINT CColAttr::GetNameLen()
    {
    return ((UINT) m_psColName->GetLength());
    };

_inline SQLSMALLINT CColAttr::GetODBCType()
    {
    return (m_eSQLType);
    };

_inline SQLUINTEGER CColAttr::GetPrecision()
    {
    return (m_cbDisplay);
    };
#endif // ifndef _DEBUG

// Inline function supports binding on four-byte boundaries.
_inline UINT CColAttr::AdjustLen(UINT cb)
    {
    return ((cb + 3) & ~3);
    };

#endif // __COLATTR_H__
