// This file is a part of the Microsoft SQL Server on-line documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft SQL Server on-line references and related electronic
// documentation provided with SQL Server.
#ifndef __CURSOR_H__
#define __CURSOR_H__

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "afxtempl.h"
#include "SQL.h"
#include "SQLExt.h"

// Macros
#define TIMEOUT_MAX             15
#define CURSORNAME_MAX          64
#define ROWARRAY_MAX          4096

// Forward declarations
class CColAttr;
class CParam;

class CCursor
    {
public:

    CCursor(SQLHDBC hdbc,
        LPCTSTR strOpenCursor,
        SQLINTEGER eCursorType = SQL_CURSOR_KEYSET_DRIVEN,
        SQLINTEGER eConcur = SQL_CONCUR_ROWVER);

    virtual ~CCursor();

    // Cursor control operations (position and fetch)
    BOOL AddParam(CParam* pParam);
    SQLRETURN SetRowArraySize(UINT nRows);
    SQLRETURN Query();
    SQLRETURN Fetch();
    SQLRETURN Position(UINT nRow);

    // Non-positioned data manipulation support
    SQLRETURN Update();
    SQLRETURN Delete();

    // Positioned data manipulation support. strPositionedUpdate
    //  is tested before submission to the server. Format is 
    //  "[DELETE|UPDATE] ... WHERE CURRENT OF cursorname ....
    LPCTSTR   GetCursorName();
    SQLRETURN Update(LPCTSTR strPositionedUpdate);

    // Support for clients. Client classes maintain memory for
    //  bind and indicator offsets, and fetched rows.
    UINT      GetColCount()
        { return (m_nCols); }
    UINT      GetRowWidth()
        { return (m_cbRow); }
    BOOL      SetDataPtrs(PUINT pBindOffsets, PUINT pIndOffsets,
        PBYTE pData);
    UINT      GetRowsFetched()
        { return (m_nRows); }

private:
    // Cursor properties.
    SQLHSTMT    m_hstmt;
    SQLHDBC     m_hdbc;
    SQLINTEGER  m_eConcur;
    SQLINTEGER  m_eCursorType;
    SQLINTEGER  m_nRowsetSize;
    SQLINTEGER  m_nTimeout;
    CString     m_strCursorName;
    CString     m_strOpen;
    CString     m_strUpdate;
    CString     m_strDelete;

    // Result set data (rows fetched from our cursor).
    SQLSMALLINT m_nCols;
    UINT        m_nRows;
    UINT        m_nCurrentRow;
    UINT        m_cbRow;
    BOOL        m_bparamsBound;
    BOOL        m_bDataBound;
    PUINT       m_pBindOffsets;
    PUINT       m_pIndOffsets;

    // Owned by the client, use this memory.
    PBYTE       m_pRowData;
    PUINT       m_pClientBindOffsets;
    PUINT       m_pClientIndOffsets;
    
    CList<CColAttr*, CColAttr*> m_listColAttr;
    CList<CParam*, CParam*> m_listParams;

private:
    // Help for retrieving data.
    void GetBindOffsets();
    BOOL BindColumns();
    };

#endif // __CURSOR_H__
