// LstCList.h
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#ifndef __LSTCLIST_H__
#define __LSTCLIST_H__
#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "afxtempl.h"
#include "SQL.h"
#include "SQLExt.h"

// Macros
#define MIN_BUFFERED_ROWS       20
#define MIN_GRID_ROWS           (MIN_BUFFERED_ROWS + 1)

// Forward declarations
class CMSFlexGrid;
class CColAttr;
class CParam;

class CListCtlList
{
public:
    // Construction requires 1) a database connection; 2) the identifier of a 
    //  string resource containing an ODBC or Transact-SQL SQL statement; and
    //  3) a pointer to an Microsoft FlexGrid OLE control.
    CListCtlList(SQLHDBC hdbc, int idString, CListCtrl* pGrid);

    // SetBufferSize adjusts the size of the rows fetched buffer. Must be
    //  called prior to initial execution (first call to member function
    //  Query()).
    BOOL SetBufferSize(UINT nRowsBuffered);

    // SetExecutionMode enables nonblocking (server cursor) execution for
    //  the list.
    BOOL SetExecutionMode(BOOL bBlocking);

    // AddParam adds the passed parameter to a list of parameters available
    //  to statement execution. Parameters are owned by the caller. The
    //  caller must ensure that the parameter does not go out of scope before
    //  the LstCList does and must delete the parameter at an appropriate
    //  time.
    BOOL AddParam(CParam* pParam);

    // Query executes the SQL statement associated with the list and fills
    //  the FlexGrid control with the first fetched rows.
    BOOL Query(BOOL bReconstruct = FALSE);

    // Empty clears any displayed items from the grid control.
    void Empty();

    virtual ~CListCtlList();

private:
    // The command, ODBC statement, and SQL Server execution mode.
    SQLHDBC     m_hdbc;
    SQLHSTMT    m_hstmt;
    CString     m_strCommand;
    BOOL        m_bBlocking;

    // Grid data. The grid will be used to display SQL Server rows.
    CListCtrl*  m_pList;
    UINT        m_nGridRows;
    UINT        m_nRowsInFirstPage;

    // A list of parameters for the ODBC statement
    CList<CParam*, CParam*>     m_listParams;
    BOOL        m_bParamsBound;

    // A list of ColumnAttributes from the executed statement
    CList<CColAttr*, CColAttr*> m_listColAttr;

    // Data and offsets to bound values
    UINT        m_nRowsBuffered;
    PBYTE       m_pRecords;
    PUINT       m_pBindOffsets;
    PUINT       m_pIndOffsets;
    UINT        m_cbRow;

    // Number of records and columns in result set
    UINT        m_nRowsFetched;
    UINT        m_nCols;

private:
    BOOL ExecuteStatement();
    BOOL GetColumnAttributes();
    void GetBindOffsets();
    BOOL BindColumns();
    BOOL FetchRecords();
};

#endif  // __LstCList_H__
