// CParam.h -- definition of an ODBC parameter handling class.
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#ifndef __PARAM_H__
#define __PARAM_H__

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "SQL.h"
#include "SQLExt.h"

// macros
#define MAX_SHORT_STR       255

/////////////////////////////////////////////////////////////////////////////
// CParam -- An ODBC parameter class
class CParam : public CObject
{
DECLARE_DYNAMIC(CParam)

// Construction
public:
    // Construct from an MFC string class object; default SQL data type
    //  is VARCHAR. Supports types SQL_CHAR, SQL_VARCHAR, SQL_LONGVARCHAR,
    //  and numeric string types.
    CParam(CString* pCString, SQLSMALLINT eSQLType = SQL_VARCHAR,
        UINT cbSQLCol = 0, UINT nSQLDigits = 0);

    // Construct from a standard C character ptr. Supports same types as 
    //  MFC string class constructor.
    CParam(PTSTR pStr, SQLSMALLINT eSQLType = SQL_VARCHAR,
        UINT cbSQLCol = 0, UINT nSQLDigits = 0);

    // Construct from a pointer to an unsigned char (BYTE) string. Supports
    //  data types like VARBINARY and LONGVARBINARY.
    CParam(PBYTE pByte, UINT cbStr = 0,
        SQLSMALLINT eSQLType = SQL_VARBINARY);

    // Construct from integers (signed or unsigned). Either parameter supports
    //  SQL data types of SQL_INTEGER, SQL_SMALLINT, and SQL_TINYINT.
    CParam(int* pInt, SQLSMALLINT eSQLType = SQL_INTEGER);
    CParam(UINT* pUint, SQLSMALLINT eSQLType = SQL_INTEGER);

    // Construct from a pointer to float or double. Supports SQL_REAL (SQL
    //  Server type 'real'), or SQL_DOUBLE (SQL Server type 'float(15)').
    CParam(float* pFloat);
    CParam(double* pDouble);

private:
protected:

// Implementation
public:
    BOOL Bind(SQLHSTMT hStmt, UINT nPos);
    
    void SetNull();
    void SetType(int eParamType);

    UINT GetDataLength();
    BOOL GetData(CString* sValue);
    BOOL GetData(PBYTE pValue, UINT cbValue, UINT* cbValueOut);

    BOOL PutData(SQLHSTMT hStmt);
    
    virtual ~CParam();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

private:
    SQLSMALLINT m_eCType;           // The C type of the data (ODBC enumerated).
    SQLSMALLINT m_eSQLType;         // The ODBC type of the data.
    SQLSMALLINT m_eParamType;       // Parameter is INPUT, OUTPUT or I/O type.

    BOOL        m_bIsCString;       // Indicates that the user is binding to
                                    //  an MFC string class object (CString).
    BOOL        m_bIsNull;          // Indicates that parameter is bound to a 
                                    //  NULL data value.
    UINT        m_cbValue;          // Length of data bound to.
    UINT        m_cbSQLCol;         // Count of bytes in SQL (server) column.
    UINT        m_nSQLDigits;       // Number of decimal digits for exact
                                    //  numeric data types.
    SQLINTEGER  m_Indicator;        // Indicator for parameter binding.
    void*       m_pValue;           // Pointer to the bound value.

protected:
};

#ifndef _DEBUG // Nondebug. Inline version of PutData
_inline BOOL CParam::PutData(SQLHSTMT hStmt)
    {
    // Everything set, just call SQLPutData.
    return (SQLPutData(hStmt, m_pValue, m_cbValue) == SQL_SUCCESS);
    }

_inline void CParam::SetNull()
    {
    m_bIsNull = TRUE;
    }

_inline UINT CParam::GetDataLength()
    {
    return (m_eParamType == SQL_PARAM_INPUT ? (UINT) -1 : (UINT) m_Indicator);
    }
#endif

#endif // __PARAM_H__
