// Transactions.cpp : Defines the class behaviors for the application.
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#include "stdafx.h"
#include "Tran.h"

#ifdef _DTCBUILD
#define INITGUID
#include "\mssql\msdtc\include\txdtc.h"
#include "\mssql\msdtc\include\xolehlp.h"
#undef  INITGUID
#endif

#include "MainFrm.h"
#include "ChildFrm.h"
#include "TranDoc.h"
#include "TranView.h"
#include "TranType.h"
#include "ODBCErr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTransactionsApp
BEGIN_MESSAGE_MAP(CTransactionsApp, CWinApp)
	//{{AFX_MSG_MAP(CTransactionsApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(IDD_TRANSTYPE, OnTranstype)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTransactionsApp construction
CTransactionsApp::CTransactionsApp()
    {
    }

/////////////////////////////////////////////////////////////////////////////
// The one and only CTransactionsApp object
CTransactionsApp theApp;

/////////////////////////////////////////////////////////////////////////////
// Globals
SQLHENV     g_hEnv = NULL;
BOOL        g_bTransType = TRANS_TYPE_OPTIMISTIC;
CString     g_strDepositText;
CString     g_strWithdrawalText;
CString     g_strSavingsText;
CString     g_strCheckingText;
#ifdef _DTCBUILD
ITransactionDispenser* g_pITransactionDispenser = NULL;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTransactionsApp initialization
BOOL CTransactionsApp::InitInstance()
    {
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_SSTRANTYPE,
		RUNTIME_CLASS(CTransactionsDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTransactionsView));
	AddDocTemplate(pDocTemplate);

    // If we get a transaction dispenser we'll allow the end-user to
    //  activate distributed transaction control.
#ifdef _DTCBUILD
    if (SUCCEEDED(DtcGetTransactionManager(NULL, NULL,
        IID_ITransactionDispenser, 0, 0, NULL,
        (void**) &g_pITransactionDispenser)))
        {
        ITransactionOptions* pITransactionOptions;
        if (SUCCEEDED(g_pITransactionDispenser->GetOptionsObject(
            &pITransactionOptions)))
            {
            XACTOPT     xactopt;

            xactopt.ulTimeout = 15000;
            strcpy((char*) xactopt.szDescription, "Transactions sample app");
            pITransactionOptions->SetOptions(&xactopt);
            pITransactionOptions->Release();
            }
        }
#endif

    // Alloc our global environment handle prior to showing the window.
    if (SQLAllocHandle(SQL_HANDLE_ENV, NULL, &g_hEnv) != SQL_SUCCESS)
        {
        return FALSE;
        }
    SQLSetEnvAttr(g_hEnv, SQL_ATTR_ODBC_VERSION, (void*) SQL_OV_ODBC3,
        SQL_IS_INTEGER);

    // Load global strings for dialog boxes and update processing.
    if (!g_strDepositText.LoadString(IDS_DEPOSITTEXT))
        {
        g_strDepositText = _T("Deposit");
        }
    if (!g_strWithdrawalText.LoadString(IDS_WITHDRAWALTEXT))
        {
        g_strWithdrawalText = _T("Withdrawal");
        }
    if (!g_strCheckingText.LoadString(IDS_CHECKINGTYPE))
        {
        g_strCheckingText = _T("Checking");
        }
    if (!g_strSavingsText.LoadString(IDS_SAVINGSTYPE))
        {
        g_strSavingsText = _T("Savings");
        }

	// Create main MDI Frame window.
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// User must create a new document, get rid of default behavior for DDE
    //  and file open.
	pMainFrame->ShowWindow(m_nCmdShow = SW_SHOWNORMAL);
	pMainFrame->UpdateWindow();

	return TRUE;
    }

/////////////////////////////////////////////////////////////////////////////
// CTransactionsApp commands

int CTransactionsApp::ExitInstance() 
    {
	if (g_hEnv != NULL)
        {
        SQLFreeHandle(SQL_HANDLE_ENV, g_hEnv);
        }

#ifdef _DTCBUILD	
    if (g_pITransactionDispenser != NULL)
        {
        g_pITransactionDispenser->Release();
        }
#endif

	return CWinApp::ExitInstance();
    }

void CTransactionsApp::OnTranstype() 
    {
	CTransTypeDlg   dlg;

    dlg.m_intCheck = (int) g_bTransType;
    if (dlg.DoModal() == IDOK)
        {
        g_bTransType = (BOOL) dlg.m_intCheck;
        }
    }

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CTransactionsApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

