// TransactionsDoc.cpp : implementation of the CTransactionsDoc class
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#include "stdafx.h"
#include "Tran.h"

#include "TranDoc.h"
#include "LoginDlg.h"
#include "Globals.h"

#include "odbcss.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// Globals
extern SQLHENV  g_hEnv;
extern BOOL     g_bTransType;
extern CTransactionsApp theApp;

#ifdef _DTCBUILD
extern ITransactionDispenser* g_pITransactionDispenser;
#endif

/////////////////////////////////////////////////////////////////////////////
// Macros
#ifdef _DTCBUILD
#define ENDTRAN(c)      { (g_bTransType == TRANS_TYPE_DISTRIBUTED ?         \
                            ((c) == SQL_COMMIT ?                            \
                                m_pITransaction->Commit(FALSE, 0, 0) :      \
                                m_pITransaction->Abort(NULL, 0, 0))  :      \
                                SQLEndTran(SQL_HANDLE_DBC, m_hdbc, (c))) ; }
#else
#define ENDTRAN(c)      { SQLEndTran(SQL_HANDLE_DBC, m_hdbc, (c)) ; }
#endif

/////////////////////////////////////////////////////////////////////////////
// CTransactionsDoc
IMPLEMENT_DYNCREATE(CTransactionsDoc, CDocument)

BEGIN_MESSAGE_MAP(CTransactionsDoc, CDocument)
	//{{AFX_MSG_MAP(CTransactionsDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTransactionsDoc construction/destruction
CTransactionsDoc::CTransactionsDoc()
    {
    m_hdbc          = NULL;
    m_idChecking = m_idSavings = m_idCustomer = -1;
    }

CTransactionsDoc::~CTransactionsDoc()
    {
    if (m_hdbc != NULL)
        {
        ENDTRAN(SQL_ROLLBACK);

#ifdef _DTCBUILD
        if (g_bTransType == TRANS_TYPE_DISTRIBUTED)
            {
            // "Un-enlist" the connection handle, then release the interface
            //  pointer.
            SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_ENLIST_IN_DTC, NULL,
                SQL_IS_INTEGER);
            m_pITransaction->Release();
            }
#endif

        if (SQL_SUCCEEDED(SQLDisconnect(m_hdbc)))
            {
            SQLFreeHandle(SQL_HANDLE_DBC, m_hdbc);
        
            m_hdbc = NULL;
            }
        else
            {
            DisplayErrors(SQL_HANDLE_DBC, m_hdbc);
            }
        }
    }

BOOL CTransactionsDoc::OnNewDocument()
    {
	if (!CDocument::OnNewDocument())
		return (FALSE);

    CLoginDlg   dlg;
    if (dlg.DoModal() == IDOK)
        {
        CString     strConnect(_T("DSN="));

        if (SQLAllocHandle(SQL_HANDLE_DBC, g_hEnv, &m_hdbc) != SQL_SUCCESS)
            {
            return (FALSE);
            }

        CWaitCursor wait;

        strConnect += dlg.m_strDSN;
        strConnect += _T(";");
        if (dlg.m_bIntegrated == TRUE)
            {
            strConnect += _T("Trusted_Connection=yes;");
            }
        else
            {
            strConnect += _T("UID=");
            strConnect += dlg.m_strUID;
            strConnect += _T(";PWD=");
            strConnect += dlg.m_strPWD;
            strConnect += _T(";");
            }

        // Set connection timeout to 15 seconds.
        SQLSetConnectAttr(m_hdbc, SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER) 15,
            SQL_IS_UINTEGER);

        // Hold cursors open until we say it's ok for them to close.
        SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_PRESERVE_CURSORS,
            (SQLPOINTER) SQL_PC_ON, SQL_IS_INTEGER);

        switch (SQLDriverConnect(m_hdbc, theApp.m_pMainWnd->m_hWnd,
            (SQLTCHAR*) ((LPCTSTR) strConnect), SQL_NTS, NULL, 0, NULL,
            SQL_DRIVER_COMPLETE))
            {
            case SQL_ERROR:
                {
                // Display the error, then fall through to SQL_NO_DATA
                //  (Cancel press on password collection).
                DisplayErrors(SQL_HANDLE_DBC, m_hdbc);
                }

            case SQL_NO_DATA:
                {
                SQLFreeHandle(SQL_HANDLE_DBC, m_hdbc);
                m_hdbc = NULL;
                return (FALSE);
                }
            }

#ifdef _DTCBUILD
        // If we're attempting distributed transaction control, get a 
        //  transaction object for this connection. Each connection
        //  maintains it's own transaction space under this model, though
        //  MS DTC maintains transaction space across multiple connections.
        if (g_bTransType == TRANS_TYPE_DISTRIBUTED)
            {
            if (FAILED(g_pITransactionDispenser->BeginTransaction(
                NULL, ISOLATIONLEVEL_ISOLATED, ISOFLAG_RETAIN_DONTCARE,
                NULL, &m_pITransaction)))
                {
                AfxMessageBox(IDS_TRANSACTION_DISPENSER_ERR);

                SQLFreeHandle(SQL_HANDLE_DBC, m_hdbc);
                m_hdbc = NULL;
                return (FALSE);
                }

            SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_ENLIST_IN_DTC,
                (SQLPOINTER) m_pITransaction, SQL_IS_INTEGER);
            }
        else
            {
#endif
            // Set AutoCommit behavior off. We will control our transactions.
            SQLSetConnectAttr(m_hdbc, SQL_ATTR_AUTOCOMMIT,
            (SQLPOINTER) SQL_AUTOCOMMIT_OFF, SQL_IS_INTEGER);
#ifdef _DTCBUILD
            }
#endif
        }
    else
        {
        return (FALSE);
        }

	return (TRUE);
    }

/////////////////////////////////////////////////////////////////////////////
// CTransactionsDoc diagnostics
#ifdef _DEBUG
void CTransactionsDoc::AssertValid() const
    {
	CDocument::AssertValid();
    }

void CTransactionsDoc::Dump(CDumpContext& dc) const
    {
	CDocument::Dump(dc);
    }
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTransactionsDoc public members
void CTransactionsDoc::DistributedCommit(BOOL bCommit)
    {
    ENDTRAN(bCommit == TRUE ? SQL_COMMIT : SQL_ROLLBACK);

#ifdef _DTCBUILD
    if (g_bTransType == TRANS_TYPE_DISTRIBUTED)
        {
        SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_ENLIST_IN_DTC, NULL,
            SQL_IS_INTEGER);
        m_pITransaction->Release();

        if (g_bTransType == TRANS_TYPE_DISTRIBUTED)
            {
            if (FAILED(g_pITransactionDispenser->BeginTransaction(
                NULL, ISOLATIONLEVEL_ISOLATED, ISOFLAG_RETAIN_DONTCARE,
                NULL, &m_pITransaction)))
                {
                AfxMessageBox(IDS_TRANSACTION_DISPENSER_ERR);
                return;
                }

            SQLSetConnectAttr(m_hdbc, SQL_COPT_SS_ENLIST_IN_DTC,
                (SQLPOINTER) m_pITransaction, SQL_IS_INTEGER);
            }
        }
#endif
    }
