// TransactionsView.cpp : implementation of the CTransactionsView class
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.
#include "stdafx.h"
#include "Tran.h"

#include "TranDoc.h"
#include "TranView.h"

#include "Pages.h"
#include "CmboList.h"
#include "LstCList.h"
#include "Param.h"
#include "ColAttr.h"
#include "Cursor.h"
#include "Globals.h"
#include "ActDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BALANCE_COL             2
#define ACCTYPEID_COL           1

extern CString  g_strDepositText;
extern CString  g_strWithdrawalText;
extern CString  g_strSavingsText;
extern CString  g_strCheckingText;
extern BOOL     g_bTransType;

/////////////////////////////////////////////////////////////////////////////
// CTransactionsView
IMPLEMENT_DYNCREATE(CTransactionsView, CView)

BEGIN_MESSAGE_MAP(CTransactionsView, CView)
	//{{AFX_MSG_MAP(CTransactionsView)
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTransactionsView construction/destruction
CTransactionsView::CTransactionsView()
    {
    m_bSizedBefore    = FALSE;
    m_pPropSheet      = NULL;
    m_pPageCustomer   = NULL;
    m_pPageChecking   = NULL;
    m_pPageSavings    = NULL;
    m_hwndFocus       = NULL;

    m_strCustCursor   = _T("");
    m_pCustCursor     = NULL;
    m_pAcctCursorRows = NULL;
    m_pAcctCursorBind = NULL;
    m_pAcctCursorInd  = NULL;

    m_pglCustHistory  = NULL;
    m_pglCheckHistory = NULL;
    m_pglSaveHistory  = NULL;
    m_bReconCust      = TRUE;
    m_bReconCheck     = TRUE;
    m_bReconSave      = TRUE;

    m_pparamCustID    = NULL;
    m_pclCustomers    = NULL;
    }

CTransactionsView::~CTransactionsView()
    {
    if (m_pPageChecking)
        delete m_pPageChecking;
    if (m_pPageSavings)
        delete m_pPageSavings;
    if (m_pPageCustomer)
        delete m_pPageCustomer;
    if (m_pPropSheet)
        delete m_pPropSheet;
    if (m_pCustCursor)
        delete m_pCustCursor;
    if (m_pglCustHistory)
        delete m_pglCustHistory;
    if (m_pglCheckHistory)
        delete m_pglCheckHistory;
    if (m_pglSaveHistory)
        delete m_pglSaveHistory;
    if (m_pparamCustID)
        delete m_pparamCustID;
    if (m_pclCustomers)
        delete m_pclCustomers;
    if (m_pAcctCursorRows != NULL)
        delete [] m_pAcctCursorRows;
    if (m_pAcctCursorBind != NULL)
        delete [] m_pAcctCursorBind;
    if (m_pAcctCursorInd  != NULL)
        delete [] m_pAcctCursorInd;
    }

/////////////////////////////////////////////////////////////////////////////
// CTransactionsView drawing
void CTransactionsView::OnDraw(CDC* pDC)
    {
    }

/////////////////////////////////////////////////////////////////////////////
// CTransactionsView diagnostics
#ifdef _DEBUG
void CTransactionsView::AssertValid() const
    {
	CView::AssertValid();
    }

void CTransactionsView::Dump(CDumpContext& dc) const
    {
	CView::Dump(dc);
    }

CTransactionsDoc* CTransactionsView::GetDocument() // non-debug version is inline
    {
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTransactionsDoc)));
	return (CTransactionsDoc*)m_pDocument;
    }
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTransactionsView message handlers
BOOL CTransactionsView::Create
    (
    LPCTSTR lpszClassName, 
    LPCTSTR lpszWindowName, 
    DWORD dwStyle, 
    const RECT& rect, 
    CWnd* pParentWnd, 
    UINT nID, 
    CCreateContext* pContext
    ) 
    {
	ASSERT(pParentWnd != NULL);
	ASSERT_KINDOF(CFrameWnd, pParentWnd);

    CWaitCursor wait;

	if (!CWnd::Create(lpszClassName, lpszWindowName, dwStyle | WS_CLIPCHILDREN,
		rect, pParentWnd, nID, pContext))
	    {
		return FALSE;
	    }

	// Add pages here.
	m_pPageCustomer = new CCustomerPage;
	m_pPageChecking = new CCheckingPage;
	m_pPageSavings  = new CSavingsPage;

    m_pPageCustomer->m_pview = (this);
    m_pPageChecking->m_pview = (this);
    m_pPageSavings->m_pview  = (this);

	// Create the window object.
	m_pPropSheet = new CPropertySheet;
	m_pPropSheet->AddPage(m_pPageCustomer);
	m_pPropSheet->AddPage(m_pPageChecking);
	m_pPropSheet->AddPage(m_pPageSavings);

	// Create a modeless property page.
	if (!m_pPropSheet->Create(this,
			DS_CONTEXTHELP | DS_SETFONT | WS_CHILD | WS_VISIBLE))
	    {
		DestroyWindow();
		return FALSE;
	    }

	m_pPropSheet->SetWindowPos(NULL, 0, 0, 0, 0,
			SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOSIZE);

	// Wse the style from the template, but make sure that the
	//  WS_BORDER bit is correct.
	m_pPropSheet->ModifyStyle(WS_BORDER|WS_CAPTION,
		dwStyle & (WS_BORDER|WS_CAPTION));

	// Force the size requested.
	// Fake a call to OnSize()--it would have been called automatically
	//  if this were using the base class implementation of Create().
	CFrameWnd* pParentFrame = GetParentFrame();
	CRect rectSize;
	m_pPropSheet->GetWindowRect(rectSize);
	pParentFrame->CalcWindowRect(rectSize);
	OnSize(SIZE_RESTORED, rectSize.Width(), rectSize.Height());

	return TRUE;
    }

void CTransactionsView::OnSize(UINT nType, int cx, int cy) 
    {
	if (nType != SIZE_MINIMIZED && cx != 0 && cy != 0 && m_pPropSheet != NULL)
	    {
		if (m_bSizedBefore == FALSE)
		    {
			m_bSizedBefore = TRUE;

			// Get the size of the property sheet.
			CRect rectSized;
			m_pPropSheet->GetWindowRect(rectSized);

			// Calculate the size of the frame.
			CFrameWnd* pFrame = GetParentFrame();
			if (pFrame != NULL)
			    {
				pFrame->CalcWindowRect(rectSized);
				CWnd* pParent = pFrame->GetParent();

				if (pParent != NULL)
					pParent->ScreenToClient(rectSized);

				// Resize and reposition the frame.
				pFrame->MoveWindow(rectSized);
			    }
		    }
	    }
    }

void CTransactionsView::OnSetFocus(CWnd* pOldWnd) 
    {
	if (!::IsWindow(m_hwndFocus))
	    {
		// Invalid or unknown focus window. Let Windows handle it.
		m_hwndFocus = NULL;
		Default();
		return;
	    }

	// Otherwise set focus to the last known focus window.
	::SetFocus(m_hwndFocus);
    }

void CTransactionsView::OnActivateView
    (
    BOOL bActivate,
    CView* pActivateView,
    CView* pDeactiveView
    ) 
    {
	if (SaveFocusControl())
		return;     // Don't call base class when focus is already set.

	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
    }

BOOL CTransactionsView::SaveFocusControl()
    {
	// Save focus window if focus is on this window's controls.
	HWND hWndFocus = ::GetFocus();
	if (hWndFocus != NULL && ::IsChild(m_hWnd, hWndFocus))
	    {
		m_hwndFocus = hWndFocus;
		return TRUE;
	    }
	return FALSE;
    }

void CTransactionsView::OnActivateFrame(UINT nState, CFrameWnd* /*pFrameWnd*/)
    {
	if (nState == WA_INACTIVE)
		SaveFocusControl();     // Save focus when frame loses activation
    }

void CTransactionsView::OnInitialUpdate() 
    {
	CView::OnInitialUpdate();
	
    CTransactionsDoc* pDoc = GetDocument();

    m_pclCustomers = new CComboList(pDoc->m_hdbc, IDS_CUSTOMERLIST,
        &(m_pPageCustomer->m_comboCustomers), 1);
    if( m_pclCustomers->Query() == FALSE)
        {
        pDoc->DistributedCommit(FALSE);
        }
	}

void CTransactionsView::ChangeCustomer()
    {
    PINT        pidCustomer;
    BOOL        bIsNULL;

    CTransactionsDoc* pDoc = GetDocument();

    if (m_pclCustomers->GetValue(2, (void**) &pidCustomer, &bIsNULL) == FALSE)
        {
        return;
        }

    if (pDoc->m_idCustomer == *pidCustomer)
        {
        return;
        }

    pDoc->m_idCustomer = *pidCustomer;

    if (m_strCustCursor.GetLength() == 0)
        {
        if (!m_strCustCursor.LoadString(IDS_CUSTOMERCURSOR))
            {
            RemoveAccountPages();
            return;
            }
        }

    if (m_pCustCursor == NULL)
        {
        m_pCustCursor = new CCursor(pDoc->m_hdbc,
            (LPCTSTR) m_strCustCursor, SQL_CURSOR_KEYSET_DRIVEN,
            (g_bTransType == TRANS_TYPE_PESIMISTIC ?
            SQL_CONCUR_LOCK : SQL_CONCUR_ROWVER));

        // At most, our customers have two accounts. Set the cursor
        //  block size appropriately to ensure that locks are held.
        m_pCustCursor->SetRowArraySize(2);

        // Create the parameter that everybody will use, then add it to
        //  the parameterized cursor.
        m_pparamCustID = new CParam(&(pDoc->m_idCustomer));

        m_pCustCursor->AddParam(m_pparamCustID);
        }

    // If cursor query fails, then remove the checking and savings
    //  pages from the property sheet, and then clean out the display
    //  items on the customer page.
    if (!SQL_SUCCEEDED(m_pCustCursor->Query()))
        {
        RemoveAccountPages();
        return;
        }

    // Check for first-fetch processing.
    if (m_pAcctCursorRows == NULL)
        {
        m_pAcctCursorBind = new UINT[m_pCustCursor->GetColCount()];
        m_pAcctCursorInd  = new UINT[m_pCustCursor->GetColCount()];
        m_pAcctCursorRows = new BYTE[m_pCustCursor->GetRowWidth() * 2];
        m_pCustCursor->SetDataPtrs(m_pAcctCursorBind, m_pAcctCursorInd,
            m_pAcctCursorRows);
        }

    // If cursor fetch fails, remove the checking and savings pages and
    //  return.
    if (!SQL_SUCCEEDED(m_pCustCursor->Fetch()))
        {
        RemoveAccountPages();
        return;
        }
    
    UpdateBalances();
    m_pPageCustomer->m_editBalance.SetWindowText(
        Int64ToStr(m_curTotalBalance));

    if (m_pglCustHistory == NULL)
        {
        m_pglCustHistory = new CListCtlList(pDoc->m_hdbc, 
            IDS_CUSTOMERHISTORY, &(m_pPageCustomer->m_listctrlHistory));

        m_pglCustHistory->AddParam(m_pparamCustID);
        }

    // Update the customer history grid control.
    if (m_pglCustHistory->Query(m_bReconCust) == FALSE)
        {
        pDoc->DistributedCommit(FALSE);
        return;
        }
    m_bReconCust = FALSE;

    if (m_sCheckingBalance.GetLength() == 0)
        {
        if (m_pPropSheet->GetPageIndex(m_pPageChecking) != -1)
            {
            m_pPropSheet->RemovePage(m_pPageChecking);
            m_bReconCheck = TRUE;
            pDoc->m_idChecking = -1;
            }
        }
    else
        {
        if (m_pPropSheet->GetPageIndex(m_pPageChecking) == -1)
            {
            if (m_pPropSheet->GetPageIndex(m_pPageSavings) != -1)
                {
                m_pPropSheet->RemovePage(m_pPageSavings);
                m_bReconSave = TRUE;
                pDoc->m_idSavings = -1;
                }

            m_pPropSheet->AddPage(m_pPageChecking);
            }
        }

    if (m_sSavingsBalance.GetLength() == 0)
        {
        if (m_pPropSheet->GetPageIndex(m_pPageSavings) != -1)
            {
            m_pPropSheet->RemovePage(m_pPageSavings);
            m_bReconSave = TRUE;
            pDoc->m_idSavings = -1;
            }
        }
    else
        {
        if (m_pPropSheet->GetPageIndex(m_pPageSavings) == -1)
            {
            m_pPropSheet->AddPage(m_pPageSavings);
            }
        }
    }

void CTransactionsView::SavingsActive()
    {
    PINT        pidCustomer;
    BOOL        bIsNULL;
    CTransactionsDoc* pDoc = GetDocument();

    if (m_pCustCursor == NULL)
        {
        return;
        }

    if (m_pclCustomers->GetValue(2, (void**) &pidCustomer, &bIsNULL) == FALSE)
        {
        return;
        }

    if (pDoc->m_idSavings == *pidCustomer)
        {
        return;
        }

    pDoc->m_idSavings = *pidCustomer;
    if (m_pglSaveHistory == NULL)
        {
        m_pglSaveHistory = new CListCtlList(pDoc->m_hdbc, 
        IDS_SAVINGSHISTORY, &(m_pPageSavings->m_listctrlHistory));

        m_pglSaveHistory->AddParam(m_pparamCustID);
        }

    m_pPageSavings->m_editBalance.SetWindowText(
        Int64ToStr(m_curSavingsBalance));
    if (m_pglSaveHistory->Query(m_bReconSave) == FALSE)
        {
        pDoc->DistributedCommit(FALSE);
        }
    
    m_bReconSave = FALSE;
    }

void CTransactionsView::CheckingActive()
    {
    PINT        pidCustomer;
    BOOL        bIsNULL;
    CTransactionsDoc* pDoc = GetDocument();

    if (m_pCustCursor == NULL)
        {
        return;
        }

    if (m_pclCustomers->GetValue(2, (void**) &pidCustomer, &bIsNULL) == FALSE)
        {
        return;
        }

    if (pDoc->m_idChecking == *pidCustomer)
        {
        return;
        }

    pDoc->m_idChecking = *pidCustomer;
    if (m_pglCheckHistory == NULL)
        {
        m_pglCheckHistory = new CListCtlList(pDoc->m_hdbc, 
        IDS_CHECKINGHISTORY, &(m_pPageChecking->m_listctrlHistory));

        m_pglCheckHistory->AddParam(m_pparamCustID);
        }

    m_pPageChecking->m_editBalance.SetWindowText(
        Int64ToStr(m_curCheckingBalance));
    if (m_pglCheckHistory->Query(m_bReconCheck) == FALSE)
        {
        pDoc->DistributedCommit(FALSE);
        }
    m_bReconCheck = FALSE;
    }

void CTransactionsView::SavingsDeposit()
    {
    DoTransaction((LPCTSTR) g_strSavingsText, (LPCTSTR) g_strDepositText);
    }

void CTransactionsView::SavingsWithdrawal()
    {
    DoTransaction((LPCTSTR) g_strSavingsText, (LPCTSTR) g_strWithdrawalText);
    }

void CTransactionsView::CheckingDeposit()
    {
    DoTransaction((LPCTSTR) g_strCheckingText, (LPCTSTR) g_strDepositText);
    }

void CTransactionsView::CheckingWithdrawal()
    {
    DoTransaction((LPCTSTR) g_strCheckingText, (LPCTSTR) g_strWithdrawalText);
    }
    
/////////////////////////////////////////////////////////////////////////////
// private help
void CTransactionsView::DoTransaction
    (
    LPCTSTR pAccountType,
    LPCTSTR pActivityType
    )
    {
	CBankTransaction dlgTrans;
    PTSTR       pCustomerName = _T("");
    BOOL        bIsNULL;
    CTransactionsDoc* pDoc = GetDocument();

    dlgTrans.m_strTitle = pActivityType;
    dlgTrans.m_strAcctType = pAccountType;

    m_pclCustomers->GetValue(1, (void**) &pCustomerName, &bIsNULL);
    dlgTrans.m_strCustomer = pCustomerName;

    // The dialog box title is either "Deposit" or "Withdrawal" and tells
    //  us what kind of transaction is taking place. We retrieve the type
    //  of account from the dialog box as well.
    if (dlgTrans.DoModal() == IDOK)
        {
        CString     strActivity;
        CString     strBalanceUpdate;
        CString     pAmount = 
            dlgTrans.m_oleCurrency.Format(0, LANG_USER_DEFAULT);

        strActivity.Format(IDS_ACTIVITYINSERT, pActivityType);
        if (g_strDepositText == pActivityType)
            {
            strBalanceUpdate.Format(IDS_BALANCEADD, (LPCTSTR) pAmount,
                m_pCustCursor->GetCursorName());
            }
        else
            {
            strBalanceUpdate.Format(IDS_BALANCESUBTRACT, (LPCTSTR) pAmount,
                m_pCustCursor->GetCursorName());
            }

        if (!strBalanceUpdate.GetLength() || !strActivity.GetLength())
            {
            return;
            }

        // We can safely cast these pointers, the parameters are INPUT only.
        SQLHSTMT    hstmt;
        CParam      paramAccountType((PTSTR) pAccountType, SQL_VARCHAR, 15);
        CParam      paramTransAmount((PTSTR) ((LPCTSTR) pAmount), SQL_DECIMAL,
            15, 2);

        if (SQLAllocHandle(SQL_HANDLE_STMT, pDoc->m_hdbc, &hstmt) == SQL_ERROR)
            {
            DisplayErrors(SQL_HANDLE_DBC, pDoc->m_hdbc);
            return;
            }
    
        m_pparamCustID->Bind(hstmt, 1);
        paramAccountType.Bind(hstmt, 2);
        paramTransAmount.Bind(hstmt, 3);

        if (SQLExecDirect(hstmt,
            (SQLTCHAR*) ((LPCTSTR) strActivity), SQL_NTS) != SQL_SUCCESS)
            {
            DisplayErrors(SQL_HANDLE_STMT, hstmt);
            SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
            return;
            }
        SQLFreeHandle(SQL_HANDLE_STMT, hstmt);

        if (!SQL_SUCCEEDED(m_pCustCursor->Position((dlgTrans.m_strAcctType == 
            g_strCheckingText ? m_idxCheckingRow + 1 : m_idxSavingsRow + 1))))
            {
            pDoc->DistributedCommit(FALSE);
            }

        if (SQL_SUCCEEDED(m_pCustCursor->Update((LPCTSTR) strBalanceUpdate)))
            {
            pDoc->DistributedCommit();
            if (m_pPropSheet->GetActivePage() == m_pPageChecking)
                {
                UpdateBalances();
                m_pPageChecking->m_editBalance.SetWindowText(
                    Int64ToStr(m_curCheckingBalance));
                if (m_pglCheckHistory->Query() == FALSE)
                    {
                    GetDocument()->DistributedCommit(FALSE);
                    return;
                    }

                RefreshCustomer();
                }
            else
                {
                UpdateBalances();
                m_pPageSavings->m_editBalance.SetWindowText(
                    Int64ToStr(m_curSavingsBalance));
                if (m_pglSaveHistory->Query() == FALSE)
                    {
                    GetDocument()->DistributedCommit(FALSE);
                    return;
                    }

                RefreshCustomer();
                }
            }
        else
            {
            pDoc->DistributedCommit(FALSE);
            }
        }
    }

void CTransactionsView::RemoveAccountPages()
    {
    CTransactionsDoc* pDoc = GetDocument();

    if (m_pPropSheet->GetPageIndex(m_pPageChecking) != -1)
        {
        m_pPropSheet->RemovePage(m_pPageChecking);
        pDoc->m_idChecking = -1;
        }
    if (m_pPropSheet->GetPageIndex(m_pPageSavings) != -1)
        {
        m_pPropSheet->RemovePage(m_pPageSavings);
        pDoc->m_idSavings = -1;
        }
        
    if (m_pglCustHistory != NULL)
        {
        m_pglCustHistory->Empty();
        }
    m_pPageCustomer->m_editBalance.SetWindowText(_T("0.0000"));
    }

void CTransactionsView::UpdateBalances()
    {
    UINT        nRow;
    PTSTR       pBalance;

    int         iTotal = 0;

    m_idxSavingsRow  = (UINT) -1;
    m_idxCheckingRow = (UINT) -1;

    m_curCheckingBalance = 0;
    m_curSavingsBalance  = 0;
    m_curTotalBalance    = 0;
    m_sCheckingBalance = _T("");
    m_sSavingsBalance  = _T("");
    m_sTotalBalance    = _T("");

    for (nRow = 0; nRow < m_pCustCursor->GetRowsFetched(); nRow++)
        {
        pBalance = (PTSTR) (m_pAcctCursorRows + 
            (nRow * m_pCustCursor->GetRowWidth()) + 
            m_pAcctCursorBind[BALANCE_COL]);

        switch (*(int*) 
            (m_pAcctCursorRows + 
            (nRow * m_pCustCursor->GetRowWidth()) + 
            m_pAcctCursorBind[ACCTYPEID_COL]))
            {
            case 1:         // Checking account
                {
                m_idxCheckingRow     = nRow;
                m_sCheckingBalance   = pBalance;
                m_curCheckingBalance = StrToInt64(m_sCheckingBalance);
                
                m_curTotalBalance += m_curCheckingBalance;
                break;
                }

            case 2:         // Savings account
                {
                m_idxSavingsRow     = nRow;
                m_sSavingsBalance   = pBalance;
                m_curSavingsBalance = StrToInt64(m_sSavingsBalance);

                m_curTotalBalance += m_curSavingsBalance;
                break;
                }
            }
        }
    }

void CTransactionsView::RefreshCustomer()
    {
    m_pPageCustomer->m_editBalance.SetWindowText(
        Int64ToStr(m_curTotalBalance));
    if (m_pglCustHistory->Query() == FALSE)
        {
        GetDocument()->DistributedCommit(FALSE);
        }
    }

/////////////////////////////////////////////////////////////////////////////
// Int64ToStr -- Converts an int64 to a money value for display. The
//  conversion drops the last two digits of scale. For example, the
//  money value "123.4500" received from SQL Server is converted to 1234500
//  by the function StrToInt64 and is converted to "123.45" by Int64ToStr.
CString CTransactionsView::Int64ToStr
    (
    LONGLONG i64
    )
    {
    CString     strValue;
    PTSTR       pValue;
    UINT        nChars = 0;
    
    pValue = strValue.GetBufferSetLength(32);
    i64 = (LONGLONG) (i64 / 100);

    do
        {
        if (nChars == 2)
            {
            *pValue = _T('.');
            pValue++;
            }

        *pValue = (TCHAR) (i64 % 10) + _T('0');
        i64 = (i64 / 10);
        nChars++;
        pValue++;
        }
    while (i64);

    if (nChars < 3)
        {
        for (UINT nChar = nChars; nChar < 3; nChar++)
            {
            if (nChars == 2)
                {
                *pValue = _T('.');
                pValue++;
                }
            *pValue = _T('0');
            pValue++;
            }
        }

    *pValue = (TCHAR) NULL;
    strValue.ReleaseBuffer();

    strValue.MakeReverse();

    return (strValue);
    }

/////////////////////////////////////////////////////////////////////////////
// StrToInt64 -- Converts a SQL Server money value (fixed precision numeric
//  scaled by 4) to an int64.
LONGLONG CTransactionsView::StrToInt64
    (
    CString& str
    )
    {
    LPCTSTR     pValue = (LPCTSTR) str;
    TCHAR       c;
    LONGLONG    i64 = 0;

    while ((c = *pValue++) != NULL)
        {
        if (_istdigit(c))
            {
            i64 = (i64 * 10) + (c - '0');
            }
        else
            {
            if (c != _T('.'))
                {
                return (0);
                }
            }
        }

    return (i64);
    }
