// TransactionsView.h : interface of the CTransactionsView class
//
// This file is part of Microsoft SQL Server online documentation.
// Copyright (C) 1992-1997 Microsoft Corporation. All rights reserved.
//
// This source code is an intended supplement to the Microsoft SQL
// Server online references and related electronic documentation.

// Forward declarations
class CTransactionsDoc;
class CCustomerPage;
class CCheckingPage;
class CSavingsPage;
class CCursor;
class CComboList;
class CGridList;
class CListCtlList;
class CParam;
class CBankTransaction;

class CTransactionsView : public CView
{
protected: // create from serialization only
	CTransactionsView();
	DECLARE_DYNCREATE(CTransactionsView)

// Attributes
public:
	CTransactionsDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTransactionsView)
	public:
	virtual BOOL Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext = NULL);
	virtual void OnInitialUpdate();
	protected:
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	virtual void OnDraw(CDC* pDC);
	//}}AFX_VIRTUAL
	virtual void OnActivateFrame(UINT nState, CFrameWnd* /*pFrameWnd*/);
	BOOL SaveFocusControl();

// Implementation
public:
	virtual ~CTransactionsView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

public:
    // Handle notifications from the customer page.
    void ChangeCustomer();

    // Handle notifications from the checking account page.
    void CheckingActive();
    void CheckingDeposit();
    void CheckingWithdrawal();
    
    // Handle notifications from the savings account page.
    void SavingsActive();
    void SavingsDeposit();
    void SavingsWithdrawal();

protected:
	BOOL m_bSizedBefore;

	// Property sheet is wired to MDI child frame and is not displayed.
	CPropertySheet* m_pPropSheet;

	// One page for the customer data, checking and savings accounts.
	CCustomerPage*  m_pPageCustomer;
	CCheckingPage*  m_pPageChecking;
	CSavingsPage*   m_pPageSavings;

    // Current focus.
    HWND            m_hwndFocus;

private:
    CString         m_strCustCursor;
    CCursor*        m_pCustCursor;
    PBYTE           m_pAcctCursorRows;
    PUINT           m_pAcctCursorBind;
    PUINT           m_pAcctCursorInd;

    CListCtlList*   m_pglCustHistory;
    BOOL            m_bReconCust;
    CListCtlList*   m_pglCheckHistory;
    BOOL            m_bReconCheck;
    CListCtlList*   m_pglSaveHistory;
    BOOL            m_bReconSave;
    CParam*         m_pparamCustID;
    CComboList*     m_pclCustomers;

    UINT            m_idxSavingsRow;
    UINT            m_idxCheckingRow;

    LONGLONG        m_curCheckingBalance;
    LONGLONG        m_curSavingsBalance;
    LONGLONG        m_curTotalBalance;
    CString         m_sCheckingBalance;
    CString         m_sSavingsBalance;
    CString         m_sTotalBalance;

private:
    void DoTransaction(LPCTSTR pAccountType, LPCTSTR pActivityType);
    void RemoveAccountPages();
    void UpdateBalances();
    CString Int64ToStr(LONGLONG i64);
    LONGLONG StrToInt64(CString& str);

    void RefreshCustomer();

// Generated message map functions
protected:
	//{{AFX_MSG(CTransactionsView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in TransactionsView.cpp
inline CTransactionsDoc* CTransactionsView::GetDocument()
   { return (CTransactionsDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
