#include "distsamp.h"

HRESULT DisplayError(ISQLDistribution *pISQLDistribution);

main()
{
	HRESULT				hr = E_FAIL;
	ISQLDistribution	*pISQLDistribution = NULL;
	BSTR				bstr = NULL;
	WCHAR				wszComputerName[MAX_COMPUTERNAME_LENGTH + 1];
	ULONG				ulCharacters = MAX_COMPUTERNAME_LENGTH + 1;
	DWORD				dwCookie = 0;
	CImpISQLDistributionEvents *pSQLDistributionEvents = NULL;

	OleInitialize(NULL);
	CHECKERROR(CoCreateInstance(CLSID_SQLDistribution, NULL, CLSCTX_INPROC_SERVER, IID_ISQLDistribution, (LPVOID *) &pISQLDistribution));
	
	CHECKALLOC(pSQLDistributionEvents, CImpISQLDistributionEvents);
	CHECKERROR(SetUpStatusCallbacks((IUnknown *) pISQLDistribution, pSQLDistributionEvents, &dwCookie));

	if(!GetComputerName(wszComputerName, &ulCharacters))
	{
		cout << "GetComputerName failed" << endl;
		goto Exit;
	}
	bstr = SysAllocString(wszComputerName);
	if (!bstr)
	{
		cout << "Memory allocation failed" << endl;
		goto Exit;
	}
	pISQLDistribution->put_Publisher(bstr);
	pISQLDistribution->put_Subscriber(bstr);
	SysFreeString(bstr);
	bstr = NULL;

	pISQLDistribution->put_PublisherSecurityMode(NT_AUTHENTICATION);

	bstr = SysAllocString(L"SampleTransactionalPublication");
	pISQLDistribution->put_Publication(bstr);
	SysFreeString(bstr);
	bstr = NULL;

	bstr = SysAllocString(L"pubs");
	pISQLDistribution->put_PublisherDatabase(bstr);
	SysFreeString(bstr);
	bstr = NULL;
		
	bstr = SysAllocString(L"pubs_replica");
	pISQLDistribution->put_SubscriberDatabase(bstr);
	SysFreeString(bstr);
	bstr = NULL;

	pISQLDistribution->put_SubscriberSecurityMode(NT_AUTHENTICATION);
	pISQLDistribution->put_SubscriptionType(PULL);

	CHECKERROR(pISQLDistribution->Initialize());
	CHECKERROR(pISQLDistribution->Run());
	CHECKERROR(pISQLDistribution->Terminate());
	
Exit:
	if (FAILED(hr))
		{
		DisplayError(pISQLDistribution);
		}
	if (pISQLDistribution)
		pISQLDistribution->Release();

	OleUninitialize();

	return hr;
}


HRESULT SetUpStatusCallbacks(IUnknown *pSQLDistribution, ISQLDistributionEvents *pISQLDistributionEvents, DWORD *pdwCookie)
{
	HRESULT						hr = E_FAIL;
	IConnectionPointContainer	*pIConnectionPointContainer = NULL;
	IConnectionPoint			*pIConnectionPoint = NULL;

	CHECKERROR(pSQLDistribution->QueryInterface(IID_IConnectionPointContainer, (void **) &pIConnectionPointContainer));
	CHECKERROR(pIConnectionPointContainer->FindConnectionPoint(DIID__SQLDistributionEvents, &pIConnectionPoint));
	CHECKERROR(pIConnectionPoint->Advise(pISQLDistributionEvents, pdwCookie));

Exit:
	
	if (pIConnectionPointContainer)
		pIConnectionPointContainer->Release();
	if (pIConnectionPoint)
		pIConnectionPoint->Release();

	return hr;
}

CImpISQLDistributionEvents::CImpISQLDistributionEvents()
{
	m_cRef = 0;
}

HRESULT CImpISQLDistributionEvents::QueryInterface(REFIID riid, LPVOID *	ppv)
{
	HRESULT hr = NOERROR;

	if(NULL == ppv)
	{
		hr = E_INVALIDARG;
		goto Exit;
	}

	*ppv = NULL;

	if(DIID__SQLDistributionEvents == riid || IID_ISQLDistributionEvents == riid || IID_IUnknown == riid)
		*ppv = this;

	if(NULL != *ppv)
	{
		((LPUNKNOWN)*ppv)->AddRef();
	}
	else
	{
		hr = E_NOINTERFACE;
	}

Exit:
	return hr;
}

ULONG CImpISQLDistributionEvents::AddRef(void)
{
	m_cRef++;
	return m_cRef;
}

ULONG CImpISQLDistributionEvents::Release(void)
{
	if(0 == --m_cRef)
	{
		delete this;
		return 0;
	}
	
	return m_cRef;
}


HRESULT CImpISQLDistributionEvents::GetTypeInfoCount(unsigned int FAR* pctinfo)
{
	return E_NOTIMPL;
}
HRESULT CImpISQLDistributionEvents::GetTypeInfo(unsigned int iTInfo, LCID lcid, ITypeInfo FAR* FAR* ppTInfo)
{
	return E_NOTIMPL;
}
HRESULT CImpISQLDistributionEvents::GetIDsOfNames(REFIID riid, OLECHAR FAR* FAR* rgszNames, unsigned int cNames, LCID lcid, DISPID FAR* rgDispId)
{
	return E_NOTIMPL;
}
HRESULT CImpISQLDistributionEvents::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS FAR* pDispParams, VARIANT FAR* pVarResult, EXCEPINFO FAR* pExcepInfo, unsigned int FAR* puArgErr)
{
	// This is simplified to demonstrate how you can get status callbacks. Normally you 
	// would want to implement ITypeInfo.

	WCHAR *pwszMessage = NULL;
	LONG percent = 0;

	VARIANT *pvars = pDispParams->rgvarg;
	pwszMessage = (WCHAR *) pvars[1].bstrVal;
	percent = pvars[0].lVal;

	return Status (pwszMessage, percent);
}


HRESULT CImpISQLDistributionEvents::Status(WCHAR *wszMsg, LONG percent)
{
	if (wszMsg)
		wprintf(L"%s\n", wszMsg);

	if (percent)
		cout << percent << "% complete" << endl;

	return S_OK;
}


HRESULT DisplayError(ISQLDistribution *pISQLDistribution)
{
	BSTR 				bstrSource, bstrDescription;
	ISQLReplErrors * 	pSQLReplErrors;
	LONG				lErrorCount = 0;
	ISQLReplError * 	pSQLReplError = NULL;
	IEnumVARIANT * 		pEnum = NULL;
	HRESULT				hr = E_UNEXPECTED;
	CHECKERROR(pISQLDistribution->get_ErrorRecords(&pSQLReplErrors));
	
	/* Show error only there is extended error info */
	if (pSQLReplErrors && SUCCEEDED(pSQLReplErrors->get_Count(&lErrorCount)) && lErrorCount > 0)
	{
		if SUCCEEDED(hr = pSQLReplErrors->get_NewEnum((IUnknown **)&pEnum))
		{
			VARIANT		var;
			ULONG 		celtFetched = 0;
			VariantInit(&var);
			while (SUCCEEDED(hr = pEnum->Next(1, &var, &celtFetched)) && (celtFetched == 1))
			{
				if (var.vt == VT_DISPATCH)
				{
					pSQLReplError = (ISQLReplError * )var.pdispVal;
					if (SUCCEEDED(hr = pSQLReplError->get_Source(&bstrSource))
						&& SUCCEEDED(hr = pSQLReplError->get_Description(&bstrDescription)))
					{
						wprintf(L"%s:%s\n", bstrSource, bstrDescription);
						SysFreeString(bstrSource);					
						SysFreeString(bstrDescription);					
					}
					if (pSQLReplError)
						{
						pSQLReplError->Release();
						pSQLReplError = NULL;
						}
				}
			}
		}
		
	}

Exit:
	/* Release interface pointers */
	if (pEnum)
		pEnum->Release();

	if (pSQLReplError)
		pSQLReplError->Release();

	if (pSQLReplErrors)
		pSQLReplErrors->Release();
		
	return hr;		
}
