use master
go
exec sp_replicationdboption pubs, 'publish', false
go
exec sp_replicationdboption pubs, 'merge publish', false
go

drop database pubs_replica
go
drop database pubs_replica_html
go

-- Check and see if the distributor is already installed or else install it
declare @distributor sysname
exec dbo.sp_helpdistributor @rpcsrvname = @distributor OUTPUT
IF @distributor is null
BEGIN
	-- Get local install path
	declare @install_path varchar (255)
	exec master.dbo.xp_regread 'HKEY_LOCAL_MACHINE',
		'SOFTWARE\Microsoft\MSSQLServer\Setup',
		'SQLPath',
		@param = @install_path OUTPUT

	-- Adding the distributor
	exec sp_adddistributor  @distributor = @@servername

	-- Adding the distributor database
	declare @data_folder varchar(255)
	set @data_folder = @install_path + '\DATA'
	exec sp_adddistributiondb  @database = 'distribution', @data_folder = @data_folder, @data_file = 'distribution_Data', @data_file_size = 5, @log_folder = @data_folder, @log_file = 'distribution_Log', @log_file_size = 2, @security_mode = 1

	-- Adding the distribution publisher
	declare @working_folder varchar(255)
	set @working_folder = @install_path + '\REPLDATA'
	exec sp_adddistpublisher  @publisher = @@servername, @distribution_db = 'distribution', @security_mode = 1, @working_directory = @working_folder, @encrypted_password = 1
END


-- Adding the local server as a registered subscriber
exec sp_addsubscriber @subscriber = @@servername, @type = 0, @login = 'sa', @security_mode = 0, @frequency_type = 64, @frequency_interval = 1, @frequency_relative_interval = 2, @frequency_recurrence_factor = 0, @frequency_subday = 8, @frequency_subday_interval = 1, @active_start_date = 0, @active_end_date = 99991231, @active_start_time_of_day = 0, @active_end_time_of_day = 235900, @status_batch_size = 100, @commit_batch_size = 100, @description = '', @encrypted_password = 1
exec sp_changesubscriber_schedule @subscriber = @@servername, @agent_type = '1'

-- Enabling the replication database
exec sp_replicationdboption pubs, 'publish', true
exec sp_replicationdboption pubs, 'merge publish', true

use pubs
GO

-- Adding the transactional publication
exec sp_addpublication @publication = 'SampleTransactionalPublication', @restricted = 'false', @sync_method = 'native', @repl_freq = 'continuous', @description = 'Sample Transactional publication.', @status = 'active', @allow_push = 'true', @allow_pull = 'true', @allow_anonymous = 'true', @enabled_for_internet = 'true', @independent_agent = 'true', @immediate_sync = 'true', @allow_sync_tran = 'false', @retention = 60
exec sp_addpublication_snapshot @publication = 'SampleTransactionalPublication',@frequency_type = 4, @frequency_interval = 1, @frequency_relative_interval = 1, @frequency_recurrence_factor = 0, @frequency_subday = 8, @frequency_subday_interval = 1, @active_start_date = 0, @active_end_date = 99991231, @active_start_time_of_day = 0, @active_end_time_of_day = 235959

-- Adding the transactional articles
exec sp_addarticle @publication = 'SampleTransactionalPublication', @article = 'employee', @source_table = 'dbo.employee', @destination_table = 'employee', @type = 'logbased', @creation_script = null, @description = null, @pre_creation_cmd = 'drop', @vertical_partition = 'false', @ins_cmd = null, @del_cmd = null, @upd_cmd = null, @filter = null, @sync_object = null
exec sp_addarticle @publication = 'SampleTransactionalPublication', @article = 'jobs', @source_table = 'dbo.jobs', @destination_table = 'jobs', @type = 'logbased', @creation_script = null, @description = null, @pre_creation_cmd = 'drop', @vertical_partition = 'false', @ins_cmd = null, @del_cmd = null, @upd_cmd = null, @filter = null, @sync_object = null
exec sp_addarticle @publication = 'SampleTransactionalPublication', @article = 'pub_info', @source_table = 'dbo.pub_info', @destination_table = 'pub_info', @type = 'logbased', @creation_script = null, @description = null, @pre_creation_cmd = 'drop', @vertical_partition = 'false', @ins_cmd = null, @del_cmd = null, @upd_cmd = null, @filter = null, @sync_object = null
exec sp_addarticle @publication = 'SampleTransactionalPublication', @article = 'publishers', @source_table = 'dbo.publishers', @destination_table = 'publishers', @type = 'logbased', @creation_script = null, @description = null, @pre_creation_cmd = 'drop', @vertical_partition = 'false', @ins_cmd = null, @del_cmd = null, @upd_cmd = null, @filter = null, @sync_object = null
exec sp_addarticle @publication = 'SampleTransactionalPublication', @article = 'stores', @source_table = 'dbo.stores', @destination_table = 'stores', @type = 'logbased', @creation_script = null, @description = null, @pre_creation_cmd = 'drop', @vertical_partition = 'false', @ins_cmd = null, @del_cmd = null, @upd_cmd = null, @filter = null, @sync_object = null

-- Adding the merge publication
exec sp_addmergepublication @publication = 'SampleMergePublication', @description = 'Sample Merge publication .', @sync_mode = 'native', @allow_push = 'true', @allow_pull = 'true', @allow_anonymous = 'true', @enabled_for_internet = 'true', @centralized_conflicts = 'true'
exec sp_addpublication_snapshot @publication = 'SampleMergePublication',@frequency_type = 4, @frequency_interval = 1, @frequency_relative_interval = 1, @frequency_recurrence_factor = 0, @frequency_subday = 1, @frequency_subday_interval = 5, @active_start_date = 0, @active_end_date = 99991231, @active_start_time_of_day = 500, @active_end_time_of_day = 235959

-- Adding the merge articles
exec sp_addmergearticle @publication = 'SampleMergePublication', @article = 'titles', @source_object = 'titles', @source_owner = 'dbo', @type = 'table', @description = null, @column_tracking = 'true', @status = 'active', @pre_creation_cmd = 'drop', @creation_script = null, @article_resolver = null, @subset_filterclause = null
exec sp_addmergearticle @publication = 'SampleMergePublication', @article = 'titleauthor', @source_object = 'titleauthor', @source_owner = 'dbo',  @type = 'table', @description = null, @column_tracking = 'true', @status = 'active', @pre_creation_cmd = 'drop', @creation_script = null, @article_resolver = null, @subset_filterclause = null
exec sp_addmergearticle @publication = 'SampleMergePublication', @article = 'sales', @source_object = 'sales', @source_owner = 'dbo', @type = 'table', @description = null, @column_tracking = 'true', @status = 'active', @pre_creation_cmd = 'drop', @creation_script = null, @article_resolver = null, @subset_filterclause = null
exec sp_addmergearticle @publication = 'SampleMergePublication', @article = 'authors', @source_object = 'authors', @source_owner = 'dbo', @type = 'table', @description = null, @column_tracking = 'true', @status = 'active', @pre_creation_cmd = 'drop', @creation_script = null, @article_resolver = null, @subset_filterclause = null

-- Create publisher-side subscriptions 
exec sp_addmergesubscription @publication = 'SampleMergePublication', @subscriber = @@servername, @subscriber_db = 'pubs_replica', @subscription_type = 'pull', @subscriber_type = 'local', @subscription_priority = 0.0000000, @sync_type = 'automatic'
exec sp_addsubscription @publication = 'SampleTransactionalPublication', @subscriber = @@servername, @destination_db = 'pubs_replica', @sync_type = 'automatic', @subscription_type = 'pull', @update_mode = 'read only'


exec sp_addmergesubscription @publication = 'SampleMergePublication', @subscriber = @@servername, @subscriber_db = 'pubs_replica_html', @subscription_type = 'pull', @subscriber_type = 'local', @subscription_priority = 0.0000000, @sync_type = 'automatic'
exec sp_addsubscription @publication = 'SampleTransactionalPublication', @subscriber = @@servername, @destination_db = 'pubs_replica_html', @sync_type = 'automatic', @subscription_type = 'pull', @update_mode = 'read only'

go

-- Create the pubs_replica database
if not exists (select * from master..sysdatabases where name='pubs_replica')
	CREATE DATABASE pubs_replica on DEFAULT = 10
GO

use pubs_replica
GO

-- Create pull subscriptions 
exec sp_addmergepullsubscription @publication = 'SampleMergePublication', @publisher = @@servername, @publisher_db = 'pubs', @subscriber_type = 'local', @subscription_priority = 0.0000000, @sync_type = 'automatic', @description = 'Sample Merge publication .'
exec sp_addpullsubscription @publisher = @@servername, @publisher_db = 'pubs', @publication = 'SampleTransactionalPublication', @independent_agent = 'true', @subscription_type = 'pull', @description = 'Sample Transactional publication.', @update_mode = 'read only'
GO

-- Create the pubs_replica database
if not exists (select * from master..sysdatabases where name='pubs_replica_html')
	CREATE DATABASE pubs_replica_html on DEFAULT = 10
GO

use pubs_replica_html
GO

-- Create pull subscriptions 
exec sp_addmergepullsubscription @publication = 'SampleMergePublication', @publisher = @@servername, @publisher_db = 'pubs', @subscriber_type = 'local', @subscription_priority = 0.0000000, @sync_type = 'automatic', @description = 'Sample Merge publication .'
exec sp_addpullsubscription @publisher = @@servername, @publisher_db = 'pubs', @publication = 'SampleTransactionalPublication', @independent_agent = 'true', @subscription_type = 'pull', @description = 'Sample Transactional publication.', @update_mode = 'read only'
GO





