/******************************************************************************
* Microsoft SQLServer
*
* Microsoft Confidential.  Copyright 1993-1994 Microsoft Corporation.
*
* Component: Microsoft SQLServer SampleCustomResolver
*
* File: replcfac.cpp
*
* File Comments:Implementation of IClassFactory for CSampleCustomResolver
*
* Revision History:
*
*
******************************************************************************/
#include 	<windows.h>
#include 	<tchar.h>
#include 	<ole2.h>
#include 	<classfct.h>   

#define 	INITGUID
#include 	<sqlres.h>
#include 	<sqlresid.h>

/* Definition of CSampleCustomResolver */
#include 	<resolver.h> 

/* Change the following string if the name of the Resolver module is different from resolver.dll */
#define	szResolverDll					"resolver.dll"

/* Change the following description to be something specific to your application */
#define szResolverDescription 			"Microsoft SQLServer Sample Custom Resolver"

/* Change the following CLSID (use the guidgen utility to generate a new CLSID  and plug in appropriately) */
#define szClsidSampleCustomResolver 	"{CB4E5130-AB82-11d0-A8E0-00C04FC2DEB3}"
static const GUID CLSID_SampleCustomResolver = 
{ 0xcb4e5130, 0xab82, 0x11d0, { 0xa8, 0xe0, 0x0, 0xc0, 0x4f, 0xc2, 0xde, 0xb3 
} };

/* Do not change the following strings */
#define szArticleResolver				"SOFTWARE\\Microsoft\\MSSQLServer\\Replication\\ArticleResolver"
#define szBothThreading 			"Both"


LONG g_lObjectCount;
LONG g_lClassFactoryLocks;

/******************************************************************************
Function:		DllMain
Purpose:  		Main entry point for the DLL.
Returns			
	TRUE if DLL initialized correctly.
	FALSE if unable to load necessary libraries
*******************************************************************************/
extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
	{
	BOOL 	fRet = TRUE;
	HMODULE hModule = NULL;

	switch(dwReason)
		{
		case DLL_PROCESS_ATTACH:
	        hModule = GetModuleHandleA("");
	        if( hModule )
		        {
	    	    DisableThreadLibraryCalls(hModule);
	        	}
			break;
		default:
			break;
		}			
	return fRet;  
	}

/******************************************************************************
Function:		DllGetClassObject
Purpose:  		This function creates class factory instances for classes supported
Parameters:		VOID
				rclsid		- the desired class id.
	           	riid 		- the desired interface id.
             	ppv 		- placeholder for the obejct instance
Returns:  		
				S_OK 			- the call was successfull.
             	E_NOINTERFACE	- the desired interface is not supported.

Note:			Exported DLL initialization is run in context of running application
******************************************************************************/
STDAPI
DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID FAR* ppv)
	{
  	IClassFactory *	pClassFactory = NULL;
  	HRESULT 		hr = ResultFromScode(E_NOINTERFACE);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	/* Initialize return object */
	*ppv = NULL;

	/* Expose all class factories here */
	if (rclsid == CLSID_SampleCustomResolver) 
		pClassFactory = new CReplicationClassFactory(rclsid);
	else
		/* Oops, class not supported! */
		return ResultFromScode(CLASS_E_CLASSNOTAVAILABLE);
	
	if (pClassFactory != NULL) 
		{
		/* Ask for the desired interface */
		hr = pClassFactory->QueryInterface(riid, ppv);

		/* Release our instance */
		pClassFactory->Release();
  		}
	else
		return ResultFromScode(E_OUTOFMEMORY);

  	return hr;
	} 

/******************************************************************************
Function:		DllCanUnloadNow
Purpose:  		This function specifies if the DLL is no longer in use and should
				be unloaded.
Parameters:		VOID
Returns:  		
				S_OK 			- DLL can be unloaded now
				S_FALSE			- DLL cannot be unloaded now
    
Note:			Exported DLL initialization is run in context of running application
******************************************************************************/
STDAPI
DllCanUnloadNow(void)
	{
	return ResultFromScode(g_lObjectCount ? S_FALSE : S_OK);
	} 

/******************************************************************************
Function:		CReplicationClassFactory (Constructor)
Purpose:  		This function is the constructor for the CReplicationClassFactory class
Parameters:		
				rclsid		- the desired class id.
	
******************************************************************************/
CReplicationClassFactory::CReplicationClassFactory(REFCLSID rclsid)
	{
	/* Increment the global object count */
	InterlockedIncrement(&g_lObjectCount);

  	m_cRef = 1;
	m_clsid = rclsid;
	} 


/******************************************************************************
Function:		~CReplicationClassFactory (Destructor)
Purpose:  		This function is the desstructor for the CReplicationClassFactory class
Parameters:		VOID
******************************************************************************/
CReplicationClassFactory::~CReplicationClassFactory()
	{
	/* Decrement the global object count */
	InterlockedDecrement(&g_lObjectCount);
	}


/******************************************************************************
Function:		CReplicationClassFactory::AddRef
Purpose:  		This function increments the reference count associated with 
				the instance
Parameters:		VOID
Returns:   		The incremented ref count
******************************************************************************/
STDMETHODIMP_(ULONG) CReplicationClassFactory::
AddRef(VOID)
	{
	InterlockedIncrement((long *)&m_cRef);
  	return m_cRef;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::Release
Purpose:  		This function decrements the reference count associated with 
				the instance
Parameters:		VOID
Returns:   		The decremented ref count
******************************************************************************/
STDMETHODIMP_(ULONG) CReplicationClassFactory::
Release(VOID)
	{
	InterlockedDecrement((long *)&m_cRef);
	if (m_cRef == 0)
		{ 
		delete this; 
		return 0;
		} 
  	return m_cRef;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::QueryInterface
Purpose:  		This function implements the Query Interface for CReplicationClassFactory class
Parameters:		
				riid 	- interface id. of the desired interface.
	            ppv		- a pointer to the desired interface
Returns:   		
				S_OK 			- the call was successfull.
             	E_NOINTERFACE 	- the specified interface is not supported.
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
QueryInterface(REFIID riid, LPVOID FAR* ppv)
	{
  	HRESULT hr = ResultFromScode(E_NOINTERFACE);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

  	*ppv = NULL;

  	if ((riid == IID_IUnknown) || (riid == IID_IClassFactory)) 
  		{
		*ppv = (LPVOID)this;
		}

  	if (*ppv != NULL) 
  		{
		((LPUNKNOWN)*ppv)->AddRef();
		hr = NOERROR;
  		}

  	return hr;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::CreateInstance
Purpose:  		This function implements the CreateInstance for CReplicationClassFactory class
Parameters:		
				riid 	- interface id. of the desired interface.
	            ppv		- a pointer to the desired interface
Returns:   		
				S_OK 					- the call was successfull.
				CLASS_E_NOAGGREGATION	- implementation not aggregatable
				E_OUTOFMEMORY			- memory allocation failed
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
CreateInstance(IUnknown FAR* pUnkOuter, REFIID riid, LPVOID FAR *ppv)
	{
  	HRESULT hr = ResultFromScode(E_INVALIDARG);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	if (pUnkOuter != NULL)	
  		{
        /* The current implementation is not aggregatable.*/
	     return(CLASS_E_NOAGGREGATION);
    	}
    	
	IUnknown	  *pUnk;

	if (m_clsid == CLSID_SampleCustomResolver)
		{
		CSampleCustomResolver   *lpSampleCustomResolver;
 		lpSampleCustomResolver = new CSampleCustomResolver();
	    if (lpSampleCustomResolver == NULL)
		    {
			hr = E_OUTOFMEMORY;
			goto ErrExit;
    		}

		/* Successfully created standalone object, return requested interface. */
		pUnk = (IUnknown *)(lpSampleCustomResolver);
		hr = pUnk->QueryInterface(riid, ppv);
		pUnk->Release();
		}

ErrExit:							 
	return hr;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::LockServer
Purpose:  		This function implements LockServer for CReplicationClassFactory class
				LockSever keeps an open object application in memory. Keeping the 
				object application alive in memory allows instances of this class 
				to be created more quickly.
Parameters:		
				fLock 	- 	Specifies the lock count. If TRUE, it increments the 
							lock count; if FALSE, it decrements the lock count.

Returns:   		
				S_OK			-	The specified object was either locked 
									(fLock = TRUE) or unlocked from memory 
									(fLock = FALSE).
				E_FAIL			- 	Indicates an unspecified error.
				E_OUTOFMEMORY	-	Out of memory.
				E_UNEXPECTED	-	An unexpected error occurred.
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
LockServer(BOOL fLock)
	{
  	if (fLock)
  		InterlockedIncrement(&g_lClassFactoryLocks);
	else  		
  		InterlockedDecrement(&g_lClassFactoryLocks);

	return NOERROR;
	} 


/******************************************************************************
Function:		DllUnregisterServer
Purpose: 		Removes appropriate keys from the registry.
Returns:   		NONE
******************************************************************************/
STDAPI DllUnregisterServer(void)
	{
	HKEY	hk = NULL, hkSub = NULL;
	BOOL	fReturn = FALSE;

	CLSID 	clsidSampleCustomResolver = CLSID_SampleCustomResolver;

	/* Delete the keys under the custom resolver's CLSID */
	fReturn = ((RegOpenKeyExA(HKEY_CLASSES_ROOT, "CLSID", 0, KEY_EXECUTE | KEY_READ | KEY_WRITE, &hk) == ERROR_SUCCESS)
			&&(RegOpenKeyA(hk, szClsidSampleCustomResolver, &hkSub) == ERROR_SUCCESS)
			&&(RegDeleteKeyA(hkSub, (const char FAR *)"InprocServer32") == ERROR_SUCCESS)
			&&(RegCloseKey(hkSub) == ERROR_SUCCESS)
			&&(RegDeleteKeyA(hk, szClsidSampleCustomResolver) == ERROR_SUCCESS)
			&&(RegCloseKey(hk) == ERROR_SUCCESS)
			&&(RegOpenKeyExA(HKEY_LOCAL_MACHINE, szArticleResolver, 0, KEY_EXECUTE | KEY_READ | KEY_WRITE, &hk) == ERROR_SUCCESS)
			&&(RegDeleteValueA(hk, szResolverDescription) == ERROR_SUCCESS)
			&&(RegCloseKey(hk) == ERROR_SUCCESS));
	hk = hkSub = NULL;

	if (hkSub)
		RegCloseKey(hkSub), ERROR_SUCCESS;
	if (hk)
		RegCloseKey(hk), ERROR_SUCCESS;
	return (fReturn ? NOERROR : E_FAIL);
	}


/******************************************************************************
Function:		DllRegisterServer
Purpose: 		Adds appropriate keys to the registry. This allows the resolver
				to be enumerated via sp_enumcustomresolvers. 
Returns:   		NONE
******************************************************************************/
STDAPI DllRegisterServer(void)
	{
	HKEY		hk = NULL, hkSub = NULL, hkServerKey = NULL;
	char		szDllPath[_MAX_FNAME	+ 1];
	SHORT		cbDllPath;
	BOOL		fReturn = FALSE;


	if (!(cbDllPath = (SHORT) GetModuleFileNameA(GetModuleHandleA(szResolverDll), 
		szDllPath, _MAX_FNAME)))
		goto Finish;

	/* Make a clean start, ignore errors, (first time) */
	(void) DllUnregisterServer();

	CLSID 		clsidSampleCustomResolver = CLSID_SampleCustomResolver;
	
	/* Create the SampleCustomResolver key here */
	if (RegCreateKeyA(HKEY_LOCAL_MACHINE, szArticleResolver, &hkSub) != ERROR_SUCCESS)
		goto Finish;

	if (RegSetValueExA(hkSub, szResolverDescription, 0, REG_SZ, (const unsigned char *)szClsidSampleCustomResolver,
		strlen(szClsidSampleCustomResolver) + 1) != ERROR_SUCCESS)
		goto Finish;

	if (hkSub)
		RegCloseKey (hkSub);

	/* Create CLSID key */
	if (RegCreateKeyA(HKEY_CLASSES_ROOT, "CLSID", &hk) != ERROR_SUCCESS)
		goto Finish;

	/* Create CLSID\{CLASSID of SampleCustomResolver } key */
	if (RegCreateKeyA(hk, szClsidSampleCustomResolver, &hkSub) != ERROR_SUCCESS)
		goto Finish;

	/* Set CLSID\{CLASSID of SampleCustomResolver } key */
	if (RegSetValueExA(hkSub, NULL, 0, REG_SZ, (const unsigned char *)szResolverDescription, 
   		strlen(szResolverDescription) + 1) != ERROR_SUCCESS)
		goto Finish;
		
	/* Create CLSID\{CLASSID of SampleCustomResolver}\InProcServer key */
	if (RegCreateKeyA(hkSub, "InprocServer32", &hkServerKey) != ERROR_SUCCESS)
		goto Finish;

	/* Set CLSID\{CLASSID of SampleCustomResolver}\InProcServer keys */
	if (RegSetValueExA(hkServerKey, NULL, 0, REG_SZ, (const unsigned char *)szDllPath, 
		strlen(szDllPath) + 1) != ERROR_SUCCESS)
		goto Finish;

	if (RegSetValueExA(hkServerKey, "ThreadingModel", 0, REG_SZ, (const unsigned char *) szBothThreading,
		strlen(szBothThreading) + 1) != ERROR_SUCCESS)
		goto Finish;


	fReturn = TRUE;
Finish:
	/* Close the keys */
	if (hkServerKey)
		RegCloseKey (hkServerKey);

	if (hkSub)
		RegCloseKey (hkSub);

	if (hk)
		RegCloseKey (hk);

	return (fReturn ? NOERROR : E_FAIL);
	}


