/******************************************************************************
* Microsoft SQL Server
*
* Microsoft Confidential.  Copyright 1993-1994 Microsoft Corporation.
*
* Component: Microsoft SQL Server Replication Provider
*
* File: resolver.cpp
*
* File Comments:Implementation of ICustomResolver
*
* Revision History:
*
*
******************************************************************************/
#define 	INC_OLE2
#include 	<tchar.h>
#include 	<ole2.h>
#include 	<initguid.h>
#include 	<sqlres.h>
#include 	<sqlresid.h>

#include 	<resolver.h>
#include 	<sql.h>
#include 	<sqlext.h>

#define OSQL_SYSNAME                    132             /* same as MAX_SQLNAME, rounded to 32-bit alignment */
#define OSQL_SYSNAME_SET                128             /* max namelen we can set a property to. */
#define OSQL_MAXCOLLEN                  8004            /* same as SQL_MAXNAMELEN (max valid collen, rounded to 32-bit alignment) */
#define OSQL_MAXCOLLEN_SET              8000            /* max columnlen we can set a property to (past which, server can't handle; it's 'text'). */

#define CALLHRJ(FcnThatReturnsHRESULTS, label) \
	{if (FAILED(hr = FcnThatReturnsHRESULTS)) goto label;}

#ifdef DEBUG
#define ASSERT(condition, message) \
        if (!(condition) && (IDYES == MessageBox(NULL, message, _T("CustomResolver Assert Failed:  Select Yes to debug"), MB_YESNOCANCEL))) { \
        DebugBreak();   \
        }
#else
#define ASSERT(condition, message) 0L;
#endif

/* This is the date column that the resolver uses to do resolution */
LPTSTR SZDateColumn	= _T("ord_date");
#define DIFF(x, y) 				((x) - (y))
LONG 	CompareTimestamps(TIMESTAMP_STRUCT * ptimeOne, TIMESTAMP_STRUCT * ptimeTwo);

/*** Implementation of ISupportErrorInfo ***/
class CSupportErrorInfo : public ISupportErrorInfo
{
	CSampleCustomResolver * m_pParent; /* Controlling object */
public:
	CSupportErrorInfo(CSampleCustomResolver * pParent)
		{
		ASSERT(pParent != NULL, _T("Parent of me is NULL"));
		m_pParent = pParent; 		
		}
	~CSupportErrorInfo() {}

    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj)
    	{
		return m_pParent->QueryInterface(riid, ppvObj);
    	}
    STDMETHOD_(ULONG, AddRef)(void)
    	{
		return m_pParent->AddRef();
    	}
    STDMETHOD_(ULONG, Release)(void)
		{
		return m_pParent->Release();
		}
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid) 			
		{
		return S_OK;
		}
};


/******************************************************************************
Function:		CSampleCustomResolver::CSampleCustomResolver
******************************************************************************/
CSampleCustomResolver::CSampleCustomResolver()
	{
	/* Initialize the private variables of the CSampleCustomResolver class */
	m_cRef = 1;
	}


/******************************************************************************
Function:		~CSampleCustomResolver (Destructor)
Purpose:  		This function is the desstructor for the CSampleCustomResolver class
Parameters:		VOID
******************************************************************************/
CSampleCustomResolver::~CSampleCustomResolver()
	{
	}

/******************************************************************************
Function:		CSampleCustomResolver::AddRef
******************************************************************************/
STDMETHODIMP_(ULONG)	
CSampleCustomResolver::AddRef()
	{
	InterlockedIncrement((long *)&m_cRef);
  	return m_cRef;	
  	}


/******************************************************************************
Function:		CSampleCustomResolver::Release
******************************************************************************/
STDMETHODIMP_(ULONG)	
CSampleCustomResolver::Release()
	{
	InterlockedDecrement((long *)&m_cRef);
	if (m_cRef == 0)
		{ 
		delete this; 
		return 0;
		} 
  	return m_cRef;
	}

/******************************************************************************
Function:		CSampleCustomResolver::QueryInterface
******************************************************************************/
STDMETHODIMP 
CSampleCustomResolver::QueryInterface(REFIID riid, LPVOID FAR* ppv)
	{
	HRESULT		hr = S_OK;

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	*ppv = NULL;

    if ((riid == IID_IUnknown) || (riid == IID_ICustomResolver))
  		{
		*ppv = (LPVOID)this;
		}
    else	
	    hr = E_NOINTERFACE;

    if (*ppv != NULL) 
  		{
		((LPUNKNOWN)*ppv)->AddRef();
		hr = NOERROR;
  		}
			
    return(hr);
	}
/******************************************************************************
Function:		CSampleCustomResolver::GetHandledStates
Purpose:  		This method fills in a bitmap of which states this reconciler
				handles.
Parameters:		Output - A bitmap of states handled by the resolver. Use the 
				REPOLE_CHANGE_TYPE enum defined in sqlres.h and the predefined 
				combinations as a sample. This resolver handles all update 
				conflict states.
******************************************************************************/
STDMETHODIMP CSampleCustomResolver::GetHandledStates(DWORD	*pResolverBm)
	{
	*pResolverBm = REPOLEUpdateConflicts;
	return S_OK;
	}

/******************************************************************************
Function:		CSampleCustomResolver::Reconcile
Purpose:  		This method is invoked on a single data change. Based on the 
				set of states handled by the resolver ( i.e. whatever is returned
				by ::GetHandledStates(), this method is invoked for all those 
				handled states.
Parameters:		Interface pointer for a RowChange object.
******************************************************************************/
STDMETHODIMP CSampleCustomResolver::Reconcile(
	IReplRowChange *pRowChange,
    DWORD dwFlags,
    PVOID pvReserved)
	{
	HRESULT					hr = S_OK;
    REPOLE_CHANGE_TYPE 		ChangeType;
	REPOLE_PRIORITY_TYPE	PriorityWinner;
	REPOLE_COLSTATUS_TYPE 	ColumnStatus;
	DWORD 					dwNumColumns;
	DWORD					dwColid;
	BOOL					fLogConflict = FALSE;	// For merging case
	TCHAR					szTableName[OSQL_SYSNAME];
	TCHAR					szColumnName[OSQL_SYSNAME];
	TIMESTAMP_STRUCT    	SourceDate;
	TIMESTAMP_STRUCT    	DestDate;
	DWORD					cbColumnActual;
	LONG					lDataType;

	/*
	** Query the change type 
	*/
    CALLHRJ(pRowChange->GetChangeType(&ChangeType), Error);

	/* 
	** Find out the table name
	*/
	CALLHRJ(pRowChange->GetTableName(szTableName, OSQL_SYSNAME_SET), Error);

	switch (ChangeType)
		{
		case REPOLEChange_SubscriberUpdate_ConflictNoColTrack:
		case REPOLEChange_PublisherUpdate_ConflictNoColTrack:
			/* 
			** Find out how many columns so we can loop over them.
			*/
			CALLHRJ(pRowChange->GetNumColumns(&dwNumColumns), Error);

			/* Query the priority winner - source or destination ? */
			CALLHRJ(pRowChange->GetPriorityWinner(&PriorityWinner), Error);

			for (dwColid = 1; dwColid <= dwNumColumns; dwColid++)
				{
				/* 
				** Query the column's name
				*/
				CALLHRJ(pRowChange->GetColumnName(dwColid, szColumnName, OSQL_SYSNAME_SET), Error);

				/* If the column name is datecol get the source and dest column values */
				if (_tcscmp(szColumnName, SZDateColumn) == 0)
					{
					CALLHRJ(pRowChange->GetColumnDatatype(dwColid, &lDataType), Error);
					ASSERT(lDataType == SQL_C_TIMESTAMP, _T("The datecol must be of type timestamp"));

					CALLHRJ(pRowChange->GetSourceColumnValue(dwColid, &SourceDate, sizeof(TIMESTAMP_STRUCT), &cbColumnActual), Error);
					CALLHRJ(pRowChange->GetDestinationColumnValue(dwColid, &DestDate, sizeof(TIMESTAMP_STRUCT), &cbColumnActual), Error);

					/* 
					** Source's order date is larger that the dest's order date - source is winner 
					** If the times are equal resort to priority algorithm - priority winner chosen 
					** using GetPriorityWinner().
					*/
					if (CompareTimestamps(&SourceDate, &DestDate) > 0)
						PriorityWinner = REPOLEPriority_Source;
					else if (CompareTimestamps(&SourceDate, &DestDate) < 0)
						PriorityWinner = REPOLEPriority_Destination;
					break;					
					}			
				}
	

			/* 
			** If the source is the loser, ignore because we will handle this when the 
			** source replica is acting as the destination
			*/
			if (PriorityWinner != REPOLEPriority_Source)
				{
				break;	
				}

			// Log the destination's update as a conflict loser and propagate the source's update.
			CALLHRJ(pRowChange->LogConflict(FALSE, REPOLEConflict_UpdateConflict, TRUE), Error);
			CALLHRJ(pRowChange->CopyRowFromSource(), Error);
			break;

		case REPOLEChange_SubscriberUpdate_ConflictColTrack:
		case REPOLEChange_PublisherUpdate_ConflictColTrack:
			/* Query the priority winner - source or destination ? */
			CALLHRJ(pRowChange->GetPriorityWinner(&PriorityWinner), Error);

			/* 
			** Find out how many columns so we can loop over them.
			*/
			CALLHRJ(pRowChange->GetNumColumns(&dwNumColumns), Error);

			for (dwColid = 1; dwColid <= dwNumColumns; dwColid++)
				{
				/* 
				** Query the column's name
				*/
				CALLHRJ(pRowChange->GetColumnName(dwColid, szColumnName, OSQL_SYSNAME_SET), Error);

				/* If the column name is Date get the source and dest column values */
				if (_tcscmp(szColumnName, SZDateColumn) == 0)
					{
					CALLHRJ(pRowChange->GetColumnDatatype(dwColid, &lDataType), Error);
					ASSERT(lDataType == SQL_C_TIMESTAMP, _T("The datecol must be of type timestamp"));

					CALLHRJ(pRowChange->GetSourceColumnValue(dwColid, &SourceDate, sizeof(TIMESTAMP_STRUCT), &cbColumnActual), Error);
					CALLHRJ(pRowChange->GetDestinationColumnValue(dwColid, &DestDate, sizeof(TIMESTAMP_STRUCT), &cbColumnActual), Error);

					/* 
					** Source's order date is larger that the dest's order date - source is winner 
					** If the times are equal resort to priority algorithm - priority winner chosen 
					** using GetPriorityWinner().
					*/
					if (CompareTimestamps(&SourceDate, &DestDate) > 0)
						PriorityWinner = REPOLEPriority_Source;
					else if (CompareTimestamps(&SourceDate, &DestDate) < 0)
						PriorityWinner = REPOLEPriority_Destination;
					break;					
					}			
				}
	

			for (dwColid = 1; dwColid <= dwNumColumns; dwColid++)
				{
				/* 
				** Query the column's status  - was it updated, updated with / without conflict etc 
				*/
				CALLHRJ(pRowChange->GetColumnStatus((short)dwColid, &ColumnStatus), Error);

				/*
				** The column was not updated at the source, so just move to the next one
				*/
				if (ColumnStatus == REPOLEColumn_NotUpdated)
					continue;

				/*
				** The column was updated at the source, but no conflict, just copy the column
				*/
				if (ColumnStatus == REPOLEColumn_UpdatedNoConflict)
					{
					CALLHRJ(pRowChange->CopyColumnFromSource((short)dwColid), Error);
					continue;
					}
					
				/*
				** The column was updated at the source, but there is a conflict. Log the
				** losing side as conflict loser, if we haven't already done so.
				*/
				fLogConflict= TRUE;

				/*
				** The source is the winner - copy the column from the source 
				*/
				if (PriorityWinner != REPOLEPriority_Destination)
					{
					CALLHRJ(pRowChange->CopyColumnFromSource((short)dwColid), Error);
					}
				}	

			/* Log the conflict at the loosing side */
			if (fLogConflict)
				{	
				/*
				** The source lost the conflict - log it
				*/
				if (PriorityWinner != REPOLEPriority_Source)
					{
					CALLHRJ(pRowChange->LogConflict(TRUE, REPOLEConflict_UpdateConflict, TRUE), Error);
					}
				/*
				** The destination lost the conflict - log it
				*/
				else
					{
					CALLHRJ(pRowChange->LogConflict(FALSE, REPOLEConflict_UpdateConflict, TRUE), Error);
					}
				}
			/*
			** This actually updates the row according to the metadata set for the 
			** individual columns 
			*/
			CALLHRJ(pRowChange->UpdateRow(), Error);
			break;
		default:
			hr = E_UNEXPECTED;
			goto Error;	
			break;
		}

Error:		
	return (hr);		
	}


/**********************************************************************
Function:	CompareTimestamps
Purpose:	Function to obtain the difference between two time structures.
Parameters:
		[IN]	ptimeOne		- Pointer to first TIMESTAMP structure
		[IN]	ptimeTwo		- Pointer to second TIMESTAMP structure

Return:
		< 0	: If structure timeOne < structure timeTwo
		= 0	: If structure timeOne = structure timeTwo
		> 0	: If structure timeOne > structure timeTwo
**********************************************************************/
LONG CompareTimestamps(TIMESTAMP_STRUCT * ptimeOne, TIMESTAMP_STRUCT * ptimeTwo)
	{
	DWORD rc;
	if ((rc = DIFF(ptimeOne->year, ptimeTwo->year)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->month, ptimeTwo->month)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->day, ptimeTwo->day)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->hour, ptimeTwo->hour)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->minute, ptimeTwo->minute)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->second, ptimeTwo->second)) != 0)
		return rc;
	if ((rc = DIFF(ptimeOne->fraction, ptimeTwo->fraction)) != 0)
		return rc;
	
	return 0;

	}
			

