/******************************************************************************
* Microsoft SQL Server
*
* Microsoft Confidential.  Copyright 1993-1994 Microsoft Corporation.
*
* Component: Microsoft SQL Server Replication Stored Procedure Resolver
*
* File: resolver.cpp
*
* File Comments:Implementation of ICustomResolver
*
* Revision History:
*
*
******************************************************************************/
#define 	INC_OLE2
#include 	<tchar.h>
#include 	<ole2.h>
#include 	<initguid.h>
#include 	<sqlres.h>
#include 	<sqlresid.h>

#include 	<mergetxt.h>
#include 	<sql.h>
#include 	<sqlext.h>

#define OSQL_SYSNAME                    132             /* same as MAX_SQLNAME, rounded to 32-bit alignment */
#define OSQL_SYSNAME_SET                128             /* max namelen we can set a property to. */
#define OSQL_MAXSTRGUID                 40              /* String form of GUID, rounded to 32-bit alignment */

#define CALLHRJ(FcnThatReturnsHRESULTS, label) \
	{if (FAILED(hr = FcnThatReturnsHRESULTS)) goto label;}

#define ASSERT(condition, message) 0L;

extern LONG g_lObjectCount;



/******************************************************************************
Function:		CMergeTextResolver::CMergeTextResolver
******************************************************************************/
CMergeTextResolver::CMergeTextResolver()
	{
	/* Initialize the private variables of the CMergeTextResolver class */
	m_cRef 			= 1;

	/* Increment the global object count */
	InterlockedIncrement(&g_lObjectCount);
	}

/******************************************************************************
Function:		~CMergeTextResolver (Destructor)
Purpose:  		This function is the desstructor for the CMergeTextResolver class
Parameters:		VOID
******************************************************************************/
CMergeTextResolver::~CMergeTextResolver()
	{
	/* Decrement the global object count */
	InterlockedDecrement(&g_lObjectCount);

	}

/******************************************************************************
Function:		CMergeTextResolver::AddRef
******************************************************************************/
STDMETHODIMP_(ULONG)	
CMergeTextResolver::AddRef()
	{
	InterlockedIncrement((long *)&m_cRef);
  	return m_cRef;	
  	}


/******************************************************************************
Function:		CMergeTextResolver::Release
******************************************************************************/
STDMETHODIMP_(ULONG)	
CMergeTextResolver::Release()
	{
	InterlockedDecrement((long *)&m_cRef);
	if (m_cRef == 0)
		{ 
		delete this; 
		return 0;
		} 
  	return m_cRef;
	}

/******************************************************************************
Function:		CMergeTextResolver::QueryInterface
******************************************************************************/
STDMETHODIMP 
CMergeTextResolver::QueryInterface(REFIID riid, LPVOID FAR* ppv)
	{
	HRESULT		hr = S_OK;

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	*ppv = NULL;

    if ((riid == IID_IUnknown) || (riid == IID_ICustomResolver))
  		{
		*ppv = (LPVOID)this;
		}
    else	
	    hr = E_NOINTERFACE;

    if (*ppv != NULL) 
  		{
		((LPUNKNOWN)*ppv)->AddRef();
		hr = NOERROR;
  		}
			
    return(hr);
	}

/******************************************************************************
Function:		CMergeTextResolver::MergeColumn
Purpose:  		This method merges the data values for a single column.

Parameters:		Interface pointer for a RowChange object, column identifier, column datatype
******************************************************************************/
STDMETHODIMP CMergeTextResolver::MergeColumn(
	IReplRowChange 	*pRowChange,
	short			Colid,
	LONG			Datatype)
	{
	HRESULT					hr = S_OK;
	unsigned char			*pSrcBuffer = NULL;
	DWORD			cbSrcBuffer;
	unsigned char			*pDestBuffer = NULL;
	DWORD			cbDestBuffer;
	unsigned char			*pResultBuffer = NULL;
	DWORD			cbResultBuffer, i, cbCommon;
	DWORD			cbSeparator;

	// Get the source and destination versions of the column
    hr = pRowChange->GetDestinationColumnValue(Colid, (void *)pDestBuffer, 0, &cbDestBuffer);

	// If it is null, we can just copy the source data value
	if (cbDestBuffer == SQL_NULL_DATA)
		{
		CALLHRJ(pRowChange->CopyColumnFromSource(Colid), Error);

		// We haven't allocated anything yet, should be ok to return
		return hr;
		}
	else
		{
		// Allocate buffer
		pDestBuffer = (unsigned char *)malloc(cbDestBuffer);
		if (!pDestBuffer)
			{
			hr = E_OUTOFMEMORY;
			goto Error;
			}
		// Now really fetch the column
	    CALLHRJ(pRowChange->GetDestinationColumnValue(Colid, (void *)pDestBuffer, cbDestBuffer, &cbDestBuffer), Error);
		}

	// Now get the source version...
    hr = pRowChange->GetSourceColumnValue(Colid, (void *)pSrcBuffer, 0, &cbSrcBuffer);
	if (cbSrcBuffer == SQL_NULL_DATA)
		{
		// Can just leave current value at the destination...
		// Jump to error label to free allocated buffers.
		goto Error;
		}
	// Allocate buffer for source version
	pSrcBuffer = (unsigned char *)malloc(cbSrcBuffer);
	if (!pSrcBuffer)
		{
		hr = E_OUTOFMEMORY;
		goto Error;
		}
	// Now really fetch the column
    CALLHRJ(pRowChange->GetSourceColumnValue(Colid, (void *)pSrcBuffer, cbSrcBuffer, &cbSrcBuffer), Error);
	
	// Allocate buffer for result version of column
	pResultBuffer = (unsigned char *)malloc(cbSrcBuffer + cbDestBuffer + 4);
	if (!pResultBuffer)
		{
		hr = E_OUTOFMEMORY;
		goto Error;
		}

	// Now walk through source & dest. copying to result...
	for (i = 0; (i < cbSrcBuffer) && (i < cbDestBuffer); i++)
		{
		if (pSrcBuffer[i] != pDestBuffer[i])
			break;
		pResultBuffer[i] = pSrcBuffer[i];
		}
	// Remember how many bytes were in common.
	// Don't count it if only first byte of a two-byte char matched...
	if ((Datatype == SQL_WLONGVARCHAR) && ((i % 2) != 0))
		cbCommon = i - 1;
	else
		cbCommon = i;

	// Copy in the rest of the source buffer
	while (i < cbSrcBuffer)
		{
		pResultBuffer[i] = pSrcBuffer[i++];
		}
	// Put in a separator... 
	if (Datatype == SQL_WLONGVARCHAR)
		{
		cbSeparator = sizeof(TCHAR);
		memcpy(&pResultBuffer[i], (void *)_T('\n'), cbSeparator);
		}
	else
		{
		cbSeparator = 1;
		pResultBuffer[i] = '\n';
		}
	// Now copy the rest of the dest buffer
	for (i = 0; i < (cbDestBuffer - cbCommon); i++)
		{
		pResultBuffer[cbSrcBuffer + i + cbSeparator] = pDestBuffer[cbCommon + i];
		}
	cbResultBuffer = cbSrcBuffer + cbDestBuffer - cbCommon;
	
	// Now set the column
	CALLHRJ(pRowChange->SetColumn(Colid, pResultBuffer, cbResultBuffer), Error);

Error:
	if (pSrcBuffer)
			free(pSrcBuffer);
	if (pDestBuffer)
			free(pDestBuffer);
	if (pResultBuffer)
			free(pResultBuffer);
	
	return (hr);		
	}

	
/******************************************************************************
Function:		CMergeTextResolver::GetHandledStates
Purpose:  		This method fills in a bitmap of which states this reconciler
				handles.
Parameters:		Interface pointer for a RowChange object
******************************************************************************/
STDMETHODIMP CMergeTextResolver::GetHandledStates(DWORD	*pResolverBm)
	{
	
	*pResolverBm = (REPOLEChange_SubscriberUpdate_ConflictColTrack |	REPOLEChange_PublisherUpdate_ConflictColTrack);

	return S_OK;
	}
/******************************************************************************
Function:		CMergeTextResolver::Reconcile
Purpose:  		This method reconciles a single data change. Either an insert, 
	delete, or update to a base row.
Parameters:		Interface pointer for a RowChange object
******************************************************************************/
STDMETHODIMP CMergeTextResolver::Reconcile(
	IReplRowChange *pRowChange,
    DWORD dwFlags,
    PVOID pvReserved)
	{
	HRESULT					hr = S_OK;
    REPOLE_CHANGE_TYPE		ChangeType;
	REPOLE_PRIORITY_TYPE  	PriWinner;
	REPOLE_COLSTATUS_TYPE 	ColStatus;
	DWORD 					NumCols;
	short					Colid;
	BOOL					fLogConflict = FALSE;	// For merging case
	BOOL					fColsUpdated = FALSE;	// For merging case
	LONG					Datatype;
	

	/*
	** Query the change type 
	*/
    CALLHRJ(pRowChange->GetChangeType(&ChangeType), Error);

	/*
	** SP based resolver currently handles conflicts - might be too expensive to invoke this for all change types.
	*/
	switch (ChangeType)
		{
		case REPOLEChange_SubscriberUpdate_ConflictColTrack:
		case REPOLEChange_PublisherUpdate_ConflictColTrack:
			break;

		default:
			hr = E_UNEXPECTED;
			goto Error;	
			break;
		}

	// Find out who wins the conflict.
	CALLHRJ(pRowChange->GetPriorityWinner(&PriWinner), Error);

	// Find out how many columns so we can loop over them.
	CALLHRJ(pRowChange->GetNumColumns(&NumCols), Error);

	for (Colid = 1; Colid <= (short) NumCols; Colid++)
		{
		CALLHRJ(pRowChange->GetColumnStatus(Colid, &ColStatus), Error);
		// If not updated at source, get on to the next one
		if (ColStatus == REPOLEColumn_NotUpdated)
			continue;
		// If updated, but no conflict, just copy the column
		if (ColStatus == REPOLEColumn_UpdatedNoConflict)
			{
			CALLHRJ(pRowChange->CopyColumnFromSource(Colid), Error);
			fColsUpdated = TRUE;
			continue;
			}
		// If it is a text column, merge the values.  Otherwise,
		// treat it as a conflict.
		CALLHRJ(pRowChange->GetColumnDatatype(Colid, &Datatype), Error);

		if ((Datatype  == SQL_LONGVARCHAR) || 
			(Datatype  == SQL_WLONGVARCHAR))
			{
			// Get column values, merge unicode text values
			CALLHRJ(MergeColumn(pRowChange, Colid, Datatype), Error);
			fColsUpdated = TRUE;
			}
		else
			{
			fLogConflict= TRUE;
			if (PriWinner != REPOLEPriority_Destination)
				{
				CALLHRJ(pRowChange->CopyColumnFromSource(Colid), Error);
				fColsUpdated = TRUE;
				}
			}
		}	// For colid


	if (fLogConflict)
		{	
		if (PriWinner != REPOLEPriority_Source)
			{
			CALLHRJ(pRowChange->LogConflict(TRUE, REPOLEConflict_ColumnUpdateConflict, TRUE), Error);
			}
		else
			{
			CALLHRJ(pRowChange->LogConflict(TRUE, REPOLEConflict_ColumnUpdateConflict, TRUE), Error);
			}
		}
	if (fColsUpdated)
		{
		CALLHRJ(pRowChange->UpdateRow(), Error);
		}

	hr = NOERROR;

Error:
	return (hr);		
	}
			

void OleUtilSetErrorInfo(LPTSTR szSource, LPTSTR szDescription, LPCTSTR szHelpFile, DWORD dwHelpContext, LPIID pGUID)
	{
	LPCREATEERRORINFO 	pCreateErrorInfo = NULL;
	LPERRORINFO 		pErrorInfo = NULL;

	// Fill in the system (thread-specific) ErrorInfo object.
	// First see if there's already error info there.  If so, leave it there.
	if (S_OK == GetErrorInfo(0, &pErrorInfo) && pErrorInfo) 
		{
		SetErrorInfo(0, pErrorInfo);
		pErrorInfo->Release();
		return;
		}

	HRESULT hr = CreateErrorInfo(&pCreateErrorInfo);
	if (SUCCEEDED(hr))
		{
#ifdef UNICODE
		pCreateErrorInfo->SetSource(szSource);
		pCreateErrorInfo->SetDescription(szDescription);
#else
		pCreateErrorInfo->SetSource(ATOW_STR(szSource));
		pCreateErrorInfo->SetDescription(ATOW_STR(szDescription));
#endif //UNICODE
		
		// pCreateErrorInfo->SetHelpFile(const_cast<LPOLESTR>(wzHelpFile));
		// pCreateErrorInfo->SetHelpContext(dwHelpContext);
		// pCreateErrorInfo->SetGUID(pIID ? *pIID : IID_NULL);
	
		hr = pCreateErrorInfo->QueryInterface(IID_IErrorInfo, (LPVOID FAR*) &pErrorInfo);
		if (SUCCEEDED(hr))
			{
			SetErrorInfo(0, pErrorInfo);
			pErrorInfo->Release();
			}
		}		
	if (pCreateErrorInfo)		
		pCreateErrorInfo->Release();
	}

LONG g_lObjectCount;
LONG g_lClassFactoryLocks;

HINSTANCE g_hInstModule = 0;
HINSTANCE g_hInstResource = 0;

/******************************************************************************
Function:		DllMain
Purpose:  		Main entry point for the DLL.
Returns			
	TRUE if DLL initialized correctly.
	FALSE if unable to load necessary libraries
*******************************************************************************/
extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
	{
	BOOL 	fRet = TRUE;
	HMODULE hModule = NULL;

	switch(dwReason)
		{
		case DLL_PROCESS_ATTACH:
	        hModule = GetModuleHandleA("");
	        if( hModule )
		        {
	    	    DisableThreadLibraryCalls(hModule);
	        	}
			break;
		default:
			break;
		}			
	return fRet;  
	}

/******************************************************************************
Function:		DllGetClassObject
Purpose:  		This function creates class factory instances for classes supported
Parameters:		VOID
				rclsid		- the desired class id.
	           	riid 		- the desired interface id.
             	ppv 		- placeholder for the obejct instance
Returns:  		
				S_OK 			- the call was successfull.
             	E_NOINTERFACE	- the desired interface is not supported.

Note:			Exported DLL initialization is run in context of running application
******************************************************************************/
STDAPI
DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID FAR* ppv)
	{
  	IClassFactory *	pClassFactory = NULL;
  	HRESULT 		hr = ResultFromScode(E_NOINTERFACE);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	/* Initialize return object */
	*ppv = NULL;

	/* Expose all class factories here */
	if (rclsid == CLSID_MergeTextResolver) 
		pClassFactory = new CReplicationClassFactory(rclsid);
	else
		/* Oops, class not supported! */
		return ResultFromScode(CLASS_E_CLASSNOTAVAILABLE);
	
	if (pClassFactory != NULL) 
		{
		/* Ask for the desired interface */
		hr = pClassFactory->QueryInterface(riid, ppv);

		/* Release our instance */
		pClassFactory->Release();
  		}
	else
		return ResultFromScode(E_OUTOFMEMORY);

  	return hr;
	} 

/******************************************************************************
Function:		DllCanUnloadNow
Purpose:  		This function specifies if the DLL is no longer in use and should
				be unloaded.
Parameters:		VOID
Returns:  		
				S_OK 			- DLL can be unloaded now
				S_FALSE			- DLL cannot be unloaded now
    
Note:			Exported DLL initialization is run in context of running application
******************************************************************************/
STDAPI
DllCanUnloadNow(void)
	{
	return S_OK;
	ResultFromScode(g_lObjectCount ? S_FALSE : S_OK);
	} 

/******************************************************************************
Function:		CReplicationClassFactory (Constructor)
Purpose:  		This function is the constructor for the CReplicationClassFactory class
Parameters:		
				rclsid		- the desired class id.
	
******************************************************************************/
CReplicationClassFactory::CReplicationClassFactory(REFCLSID rclsid)
	{
	/* Increment the global object count */
	InterlockedIncrement(&g_lObjectCount);

  	m_cRef = 1;
	m_clsid = rclsid;
	} 


/******************************************************************************
Function:		~CReplicationClassFactory (Destructor)
Purpose:  		This function is the desstructor for the CReplicationClassFactory class
Parameters:		VOID
******************************************************************************/
CReplicationClassFactory::~CReplicationClassFactory()
	{
	/* Decrement the global object count */
	InterlockedDecrement(&g_lObjectCount);
	}


/******************************************************************************
Function:		CReplicationClassFactory::AddRef
Purpose:  		This function increments the reference count associated with 
				the instance
Parameters:		VOID
Returns:   		The incremented ref count
******************************************************************************/
STDMETHODIMP_(ULONG) CReplicationClassFactory::
AddRef(VOID)
	{
	InterlockedIncrement((long *)&m_cRef);
  	return m_cRef;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::Release
Purpose:  		This function decrements the reference count associated with 
				the instance
Parameters:		VOID
Returns:   		The decremented ref count
******************************************************************************/
STDMETHODIMP_(ULONG) CReplicationClassFactory::
Release(VOID)
	{
	InterlockedDecrement((long *)&m_cRef);
	if (m_cRef == 0)
		{ 
		delete this; 
		return 0;
		} 
  	return m_cRef;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::QueryInterface
Purpose:  		This function implements the Query Interface for CReplicationClassFactory class
Parameters:		
				riid 	- interface id. of the desired interface.
	            ppv		- a pointer to the desired interface
Returns:   		
				S_OK 			- the call was successfull.
             	E_NOINTERFACE 	- the specified interface is not supported.
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
QueryInterface(REFIID riid, LPVOID FAR* ppv)
	{
  	HRESULT hr = ResultFromScode(E_NOINTERFACE);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

  	*ppv = NULL;

  	if ((riid == IID_IUnknown) || (riid == IID_IClassFactory)) 
  		{
		*ppv = (LPVOID)this;
		}

  	if (*ppv != NULL) 
  		{
		((LPUNKNOWN)*ppv)->AddRef();
		hr = NOERROR;
  		}

  	return hr;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::CreateInstance
Purpose:  		This function implements the CreateInstance for CReplicationClassFactory class
Parameters:		
				riid 	- interface id. of the desired interface.
	            ppv		- a pointer to the desired interface
Returns:   		
				S_OK 					- the call was successfull.
				CLASS_E_NOAGGREGATION	- implementation not aggregatable
				E_OUTOFMEMORY			- memory allocation failed
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
CreateInstance(IUnknown FAR* pUnkOuter, REFIID riid, LPVOID FAR *ppv)
	{
  	HRESULT hr = ResultFromScode(E_INVALIDARG);

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	if (pUnkOuter != NULL)	
  		{
        /* The current implementation is not aggregatable.*/
	     return(CLASS_E_NOAGGREGATION);
    	}
    	
	IUnknown	  *pUnk;

	if (m_clsid == CLSID_MergeTextResolver)
		{
		CMergeTextResolver   *lpMergeTextResolver;
 		lpMergeTextResolver = new CMergeTextResolver();
	    if (lpMergeTextResolver == NULL)
		    {
			hr = E_OUTOFMEMORY;
			goto ErrExit;
    		}

		/* Successfully created standalone object, return requested interface. */
		pUnk = (IUnknown *)(lpMergeTextResolver);
		hr = pUnk->QueryInterface(riid, ppv);
		pUnk->Release();
		}

ErrExit:							 
	return hr;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::LockServer
Purpose:  		This function implements LockServer for CReplicationClassFactory class
				LockSever keeps an open object application in memory. Keeping the 
				object application alive in memory allows instances of this class 
				to be created more quickly.
Parameters:		
				fLock 	- 	Specifies the lock count. If TRUE, it increments the 
							lock count; if FALSE, it decrements the lock count.

Returns:   		
				S_OK			-	The specified object was either locked 
									(fLock = TRUE) or unlocked from memory 
									(fLock = FALSE).
				E_FAIL			- 	Indicates an unspecified error.
				E_OUTOFMEMORY	-	Out of memory.
				E_UNEXPECTED	-	An unexpected error occurred.
******************************************************************************/
STDMETHODIMP CReplicationClassFactory::
LockServer(BOOL fLock)
	{
  	if (fLock)
  		InterlockedIncrement(&g_lClassFactoryLocks);
	else  		
  		InterlockedDecrement(&g_lClassFactoryLocks);

	return NOERROR;
	} 

/******************************************************************************
Function:		CReplicationClassFactory::ClassInit
Purpose:  		DLL linkage time Class Initialization.  Called on process attach.
				Typically class-wide initialization things like ::RegisterClass
				are done here.  But, since this object has no "class date" it
				does not need to do anything here.
Parameters:		VOID
Returns:   		TRUE/FALSE
******************************************************************************/
BOOL
CReplicationClassFactory::ClassInit()
	{
    return(TRUE);
	}

/******************************************************************************
Function:		CReplicationClassFactory::ClassDestroy
Purpose:  		DLL process detach time Class destruction. As above, a useful 
				convention. Things like ::UnregisterClass go here.
Parameters:		VOID
Returns:   		TRUE/FALSE
******************************************************************************/
BOOL
CReplicationClassFactory::ClassDestroy()
	{
    return(TRUE);
	}


/******************************************************************************
Function:		DllUnregisterServer
Purpose: 		Removes appropriate keys from the registry.
Returns:   		NONE
******************************************************************************/
STDAPI DllUnregisterServer(void)
	{
	HKEY	hk = NULL, hkSub = NULL;
	BOOL	fReturn = FALSE;

	CLSID 	clsidMergeTextResolver = CLSID_MergeTextResolver;

	/* Delete the keys under the custom resolver's CLSID */
	fReturn = ((RegOpenKeyExA(HKEY_CLASSES_ROOT, "CLSID", 0, KEY_EXECUTE | KEY_READ | KEY_WRITE, &hk) == ERROR_SUCCESS)
			&&(RegOpenKeyA(hk, szClsidMergeTextResolver, &hkSub) == ERROR_SUCCESS)
			&&(RegDeleteKeyA(hkSub, (const char FAR *)"InprocServer32") == ERROR_SUCCESS)
			&&(RegCloseKey(hkSub) == ERROR_SUCCESS)
			&&(RegDeleteKeyA(hk, szClsidMergeTextResolver) == ERROR_SUCCESS)
			&&(RegCloseKey(hk) == ERROR_SUCCESS)
			&&(RegOpenKeyExA(HKEY_LOCAL_MACHINE, szArticleResolver, 0, KEY_EXECUTE | KEY_READ | KEY_WRITE, &hk) == ERROR_SUCCESS)
			&&(RegDeleteValueA(hk, szResolverDescription) == ERROR_SUCCESS)
			&&(RegCloseKey(hk) == ERROR_SUCCESS));
	hk = hkSub = NULL;

	if (hkSub)
		RegCloseKey(hkSub), ERROR_SUCCESS;
	if (hk)
		RegCloseKey(hk), ERROR_SUCCESS;
	return (fReturn ? NOERROR : E_FAIL);
	}


/******************************************************************************
Function:		DllRegisterServer
Purpose: 		Adds appropriate keys to the registry.
Returns:   		NONE
******************************************************************************/
STDAPI DllRegisterServer(void)
	{
	HKEY		hk = NULL, hkSub = NULL, hkServerKey = NULL;
	char		szDllPath[_MAX_FNAME	+ 1];
	SHORT		cbDllPath;
	BOOL		fReturn = FALSE;


	if (!(cbDllPath = (SHORT) GetModuleFileNameA(GetModuleHandleA(szResolverDll), 
		szDllPath, _MAX_FNAME)))
		goto Finish;

	/* Make a clean start, ignore errors, (first time) */
	(void) DllUnregisterServer();

	CLSID 		clsidMergeTextResolver = CLSID_MergeTextResolver;
	
	/* Create the MergeTextResolver key here */
	if (RegCreateKeyA(HKEY_LOCAL_MACHINE, szArticleResolver, &hkSub) != ERROR_SUCCESS)
		goto Finish;

	if (RegSetValueExA(hkSub, szResolverDescription, 0, REG_SZ, (const unsigned char *)szClsidMergeTextResolver,
		strlen(szClsidMergeTextResolver) + 1) != ERROR_SUCCESS)
		goto Finish;

	if (hkSub)
		RegCloseKey (hkSub);

	/* Create CLSID key */
	if (RegCreateKeyA(HKEY_CLASSES_ROOT, "CLSID", &hk) != ERROR_SUCCESS)
		goto Finish;

	/* Create CLSID\{CLASSID of MergeTextResolver } key */
	if (RegCreateKeyA(hk, szClsidMergeTextResolver, &hkSub) != ERROR_SUCCESS)
		goto Finish;

	/* Set CLSID\{CLASSID of MergeTextResolver } key */
	if (RegSetValueExA(hkSub, NULL, 0, REG_SZ, (const unsigned char *)szResolverDescription, 
   		strlen(szResolverDescription) + 1) != ERROR_SUCCESS)
		goto Finish;
		
	/* Create CLSID\{CLASSID of MergeTextResolver}\InProcServer key */
	if (RegCreateKeyA(hkSub, "InprocServer32", &hkServerKey) != ERROR_SUCCESS)
		goto Finish;

	/* Set CLSID\{CLASSID of MergeTextResolver}\InProcServer keys */
	if (RegSetValueExA(hkServerKey, NULL, 0, REG_SZ, (const unsigned char *)szDllPath, 
		strlen(szDllPath) + 1) != ERROR_SUCCESS)
		goto Finish;

	if (RegSetValueExA(hkServerKey, "ThreadingModel", 0, REG_SZ, (const unsigned char *) szBothThreading,
		strlen(szBothThreading) + 1) != ERROR_SUCCESS)
		goto Finish;


	fReturn = TRUE;
Finish:
	/* Close the keys */
	if (hkServerKey)
		RegCloseKey (hkServerKey);

	if (hkSub)
		RegCloseKey (hkSub);

	if (hk)
		RegCloseKey (hk);

	return (fReturn ? NOERROR : E_FAIL);
	}


