//****************************************************************************
//*	File: odbcconn.h  ( ODBC Connection Class )
//****************************************************************************
#ifndef _ODBCCONN_INCL_
#define _ODBCCONN_INCL_

#include <sqlres.h>
#include <sql.h>
#include <sqlext.h>


#define ODBCCONN_E_RETRY       MAKE_SCODE(SEVERITY_SUCCESS,FACILITY_ITF,0x1001)
#define ODBCCONN_E_CANCEL      MAKE_SCODE(SEVERITY_SUCCESS,FACILITY_ITF,0x1002)

#define DEFAULT_LOGIN_TIMEOUT		60
#define DEFAULT_QUERY_TIMEOUT		60
#define MAX_MESSAGE_LEN				1024	
#define MAX_FORMAT_LEN				256
#define MAX_TEXT					256
#define MAX_NAME					129
#define MAX_ERRORS					2
#define	MAX_ERR_LEN					256
#define MAX_SOURCENAME  			100		// max length of error source name (dependent on table MSrepl_errors)


#define RC_NOTSUCCESSFUL(rc) ((rc)>>1)	// ODBC not success check macro
#define RC_SUCCESSFUL(rc) (!(rc)>>1)	// ODBC success check macro

#define SRV_VERSION_7X     		1
#define SRV_VERSION_6X     		2
#define SRV_VERSION_6PLUS  		3
#define SRV_VERSION_NONE   		SRV_VERSION_7X
#define SQL_REPL_SERVER_NAME	37283			// Undocument ODBC connection option


// ODBC errors

#define ROLE_CONNECTION_FAILURE     1001
#define ROLE_OPERATION_ABORTED      1002
#define ROLE_CONNECTION_TIMEOUT     1003
#define ROLE_DEADLOCK               1004
#define ROLE_TABLE_LOCKED           1005
#define ROLE_CANNOT_LOCK_TABLE      1006
#define ROLE_ODBC_ERROR             1007
#define ROLE_NOALLOCENV             1008

// Security Type
#define SECURITY_MODE_STANDARD			0
#define SECURITY_MODE_INTEGRATED		1
#define SECURITY_MODE_NTAUTHENTICATION	2
#define SECURITY_MODE_MIN				0
#define SECURITY_MODE_MAX				2

// Data Source Type values
#define SQL_SERVER					0
#define ODBC_DATA_SOURCE			1
#define JET_DATABASE                2

// SQL type to map to a C type
SWORD GetCTypeFromSqlType(SWORD fSqlType);

// Error source type
typedef enum {
	REPL_SOURCE_NULL		= 0,
	REPL_SOURCE_COMMAND		= 1,
	REPL_SOURCE_AGENT		= 2,
	REPL_SOURCE_OS			= 3,
	REPL_SOURCE_ODBC		= 4,
	REPL_SOURCE_SQLSERVER	= 5,
	REPL_SOURCE_NETLIB		= 6,
	REPL_SOURCE_SQLDMO		= 7
} REPL_SOURCE_TYPE;

typedef UNALIGNED struct odbc_error
	{
	TCHAR				szSqlState[SQL_SQLSTATE_SIZE+1];
	TCHAR 				szErrMsg[SQL_MAX_MESSAGE_LENGTH+1]; 
	SWORD				wErrMsgLen;	  
	SDWORD				NativeError;
	TCHAR				szNativeError[MAX_ERR_LEN+1];
	REPL_SOURCE_TYPE 	SourceType;
	TCHAR				szSourceName[MAX_SOURCENAME+1];
	} ODBC_ERROR;

class CODBCConnection 
{
private:
    BOOL 			IsRetryableError( ODBC_ERROR* pError, DWORD* pdwReplOleErrCode );
	BOOL			m_fNoErrorAssert;
public:
	HENV			m_henv;
	HDBC 			m_hdbc;			   
	HSTMT 			m_hstmt;
	TCHAR 			m_szOutcome[MAX_MESSAGE_LEN];
	DWORD			m_cbLen;

	BOOL 			Concat(TCHAR * szMessage);
	VOID			LoadMessage(DWORD wID, ...);
	LONG 			ConstructOutcome();

	INT				m_Type;			// O SQL Server 1 DSN
	TCHAR			m_szName[MAX_NAME];
	TCHAR 			m_szApplication[MAX_NAME*2];
	TCHAR			m_szServerRole[MAX_NAME]; // role of server, e.g. "Publisher", "Distributor", "Subscriber"
	TCHAR 			m_szHost[MAX_NAME];
	TCHAR 			m_szConnectName[MAX_NAME];
	TCHAR 			m_szServerName[MAX_NAME];
	TCHAR 			m_szRemServerName[MAX_NAME];
	TCHAR 			m_szNetwork[MAX_NAME];
	TCHAR 			m_szAddress[MAX_NAME];
	TCHAR 			m_szDatabase[MAX_NAME];
    TCHAR           m_szDatabasePath[MAX_PATH];  // Path to database if JET MDB
	TCHAR 			m_szLoginName[MAX_NAME];
	TCHAR 			m_szPassword[MAX_NAME];
	TCHAR 			m_szUserName[MAX_NAME];
	INT 			m_LoginTimeout;
	INT 			m_QueryTimeout;
	ULONG 			m_PacketSize;
	TCHAR			m_szDbmsName[MAX_TEXT];
	INT				m_ServerMemory;
	DWORD			m_SecurityMode; /* 0 standard, 1 integrated , -1 null*/
	INT				m_SkipFailureLevel; /* 0 stop on all failures, 1 skip only well-known failures, 2 skip on all failures (2 not implemented yet) */

	// ODBC
	
	TCHAR 			m_szDbmsVer[MAX_TEXT];
	TCHAR			m_szDriverName[MAX_TEXT];
	TCHAR           m_szDriverVer[MAX_TEXT];
	INT 			m_ApiConformance;
	INT 			m_SqlConformance;
	INT 			m_TxnCapable;
	TCHAR 			m_ReadOnly[2];
	TCHAR 			m_IdentifierQuote[2];
	TCHAR 			m_szSpclChrs[64];         
	INT 			m_NonNullableCols;
	INT 			m_MaxTableNameLen;
	INT 			m_MaxColNameLen;
	TCHAR 			m_NeedLongDataLen[2];
	INT 			m_MaxColsInTable;
	INT 			m_MaxColsInIndex;
	INT 			m_MaxCharLiteralLen;
	INT 			m_MaxStatementLen;
	INT 			m_MaxRowSize;
	INT 			m_OwnerUsage;
	BOOL 			m_fAnsiCodepage;

    DWORD           m_SQLServerVersion;
    DWORD           m_dwReplOleErrorCode;

	// Data Status
	BOOL			m_bData;
	BOOL			m_fCommandPrepared;

	// ODBC Error Information
	ODBC_ERROR		m_Error[MAX_ERRORS];
	INT				m_NumErrors;

	// ODBC last command cause execution or prepare failure
	TCHAR *			m_szErrorCommand;

	
	TCHAR			m_szMsg[MAX_MESSAGE_LEN];

	// Return Status Information
   	HRESULT			m_hr;

	CODBCConnection();
	~CODBCConnection();

	HRESULT Connect(IConnectionInfo * pConnectionInfo);
	HRESULT Disconnect();
	HRESULT GetError();
	VOID GetErrorCodeText(SDWORD *pNativeErrorCode, TCHAR *szNativeErrorText, int BuffLen);
	VOID SetNoErrorAssertFlag(BOOL fAssertMode);
	VOID ClearErrors();
	VOID GetName(TCHAR *szName);	// get name of the connection
	VOID GetDatabaseName(TCHAR *szDatabaseName);
	VOID GetServerName(TCHAR *szSrvName, TCHAR *szRemSrvName);
	HRESULT ExecuteDirect(TCHAR * szBuffer);
	HRESULT CloseCursor();
	HRESULT Prepare(TCHAR * szBuffer);
	HRESULT Execute();
	HRESULT Fetch();
	HRESULT MoreResults();
	HRESULT GetNextResultSet();
	HRESULT ChangeDatabase(TCHAR * szDatabase);
	HRESULT BeginTran();
	HRESULT Commit();
	HRESULT Rollback();

	HRESULT GetColumnData(int iCol, int ColType, TCHAR *pData, SDWORD cbMax, SDWORD *pcbActual); 
	HRESULT GetColumnCount(LONG * plColumnCount);
	HRESULT GetColumnName(int iCol, LPTSTR szColumnName, int cbBufferLength);
	HRESULT GetColumnDatatype(int iCol, LONG * plDataType);
	HRESULT GetColumnLength(int iCol, LONG * plColumnLength);
	HRESULT GetColumnUpdatable(int iCol, LONG * plColumnUpdatable, LONG *plColumnIdentity);
	HRESULT GetColumnScale(int iCol, LONG * plColumnScale);
	HRESULT GetColumnPrecision(int iCol, LONG * plColumnPrecision);
	HRESULT GetDisplayLength(int iCol, LONG * plDisplayLength);
	HRESULT BindParameter(int iCol, int ParamType, int CType, int SqlType, UDWORD cbColDef, 
			SWORD ibScale, LPVOID pvData, SDWORD cbMax, SDWORD *pcbActual);
	HRESULT GetData(int iCol, int CType, LPVOID pvData, SDWORD cbMax, SDWORD *pcbActual);
	BOOL FMoreData() { return m_bData; }


	inline	TCHAR *GetErrorCommand(){ return m_szErrorCommand;}
	inline	ODBC_ERROR GetReplError(INT i) { return m_Error[i];}
	inline	ODBC_ERROR *GetReplError() { return m_Error;}

	inline	INT GetNumErrors() { return m_NumErrors;}
	inline	VOID SetNumErrors(INT NumErrors) { m_NumErrors = NumErrors;}

};

#endif
