/******************************************************************************
* Microsoft SQL Server
*
* Microsoft Confidential.  Copyright 1993-1994 Microsoft Corporation.
*
* Component: Microsoft SQL Server Replication Stored Procedure Resolver
*
* File: resolver.cpp
*
* File Comments:Implementation of ICustomResolver
*
* Revision History:
*
*
******************************************************************************/
#define 	INC_OLE2
#include 	<tchar.h>
#include 	<ole2.h>
#include 	<initguid.h>
#include 	<sqlres.h>
#include 	<sqlresid.h>

#include 	<odbcconn.h>
#include 	<pubspres.h>
#include 	<sql.h>
#include 	<sqlext.h>

#define OSQL_SYSNAME                    132             /* same as MAX_SQLNAME, rounded to 32-bit alignment */
#define OSQL_SYSNAME_SET                128             /* max namelen we can set a property to. */
#define OSQL_MAXSTRGUID                 40              /* String form of GUID, rounded to 32-bit alignment */


#define CALLHRJ(FcnThatReturnsHRESULTS, label) \
	{if (FAILED(hr = FcnThatReturnsHRESULTS)) goto label;}

#ifdef DEBUG
#define Assert(condition, message) \
        if (!(condition) && (IDYES == MessageBox(NULL, message, _T("CustomResolver Assert Failed:  Select Yes to debug"), MB_YESNOCANCEL))) { \
        DebugBreak();   \
        }
#else
#define Assert(condition, message) 0L;
#endif

extern LONG g_lObjectCount;

LPTSTR SzCustomResolverProc = _T("exec %s @tableowner = [%s], @tablename = [%s], @rowguid = [%s], @subscriber = [%s], @subscriber_db = [%s], @subscriber_security_mode = %d, @subscriber_login = \'%s\', @subscriber_password = \'%s\'");

void 	OleUtilSetErrorInfo(LPTSTR szSource, LPTSTR szDescription, LPCTSTR szHelpFile = NULL, DWORD dwHelpContext = 0, LPIID pGUID = NULL);

/*** Implementation of ISupportErrorInfo ***/
class CSupportErrorInfo : public ISupportErrorInfo
{
	CPubSpResolver * m_pParent; /* Controlling object */
public:
	CSupportErrorInfo(CPubSpResolver * pParent)
		{
		//Assert(pParent != NULL, "Parent must not be NULL");
		m_pParent = pParent; 		
		}
	~CSupportErrorInfo() {}

    STDMETHOD(QueryInterface)(REFIID riid, LPVOID FAR* ppvObj)
    	{
		return m_pParent->QueryInterface(riid, ppvObj);
    	}
    STDMETHOD_(ULONG, AddRef)(void)
    	{
		return m_pParent->AddRef();
    	}
    STDMETHOD_(ULONG, Release)(void)
		{
		return m_pParent->Release();
		}
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid) 			
		{
		return S_OK;
		}
};



/******************************************************************************
Function:		CPubSpResolver::CPubSpResolver
******************************************************************************/
CPubSpResolver::CPubSpResolver()
	{
	/* Initialize the private variables of the CStoredProcedureResolver class */
	m_cRef 						= 1;
	m_pSupportErrorInfo 		= NULL;
	m_fSourceConnectionExists 	= FALSE;
	m_fDestConnectionExists 	= FALSE;
	m_fInitialized 				= InitInstance();

	/* Increment the global object count */
	InterlockedIncrement(&g_lObjectCount);
	}

/******************************************************************************
Function:		~CPubSpResolver (Destructor)
Purpose:  		This function is the desstructor for the CPubSpResolver class
Parameters:		VOID
******************************************************************************/
CPubSpResolver::~CPubSpResolver()
	{
	/* Decrement the global object count */
	InterlockedDecrement(&g_lObjectCount);

	/* Uninitialize the resolver */
	ExitInstance();
	}

/******************************************************************************
Function:		CPubSpResolver::AddRef
******************************************************************************/
STDMETHODIMP_(ULONG)	
CPubSpResolver::AddRef()
	{
	InterlockedIncrement((long *)&m_cRef);
  	return m_cRef;	
  	}


/******************************************************************************
Function:		CStoredProcedureResolver::Release
******************************************************************************/
STDMETHODIMP_(ULONG)	
CPubSpResolver::Release()
	{
	InterlockedDecrement((long *)&m_cRef);
	if (m_cRef == 0)
		{ 
		delete this; 
		return 0;
		} 
  	return m_cRef;
	}

/******************************************************************************
Function:		CPubSpResolver::QueryInterface
******************************************************************************/
STDMETHODIMP 
CPubSpResolver::QueryInterface(REFIID riid, LPVOID FAR* ppv)
	{
	HRESULT		hr = S_OK;

	/* Check args */
	if (!ppv)
		return ResultFromScode(E_INVALIDARG);

	*ppv = NULL;

    if ((riid == IID_IUnknown) || (riid == IID_ICustomResolver))
  		{
		*ppv = (LPVOID)this;
		}
    else if (riid == IID_ISupportErrorInfo)
		{
		*ppv = (LPVOID)m_pSupportErrorInfo;
  		}
    else	
	    hr = E_NOINTERFACE;

    if (*ppv != NULL) 
  		{
		((LPUNKNOWN)*ppv)->AddRef();
		hr = NOERROR;
  		}
			
    return(hr);
	}

/******************************************************************************
Function:		InitInstance
Purpose:  		This function initializes the partular instance of the sp resolver
Parameters:		VOID
******************************************************************************/
BOOL CPubSpResolver::InitInstance()
	{
	/* Initialize the embedded interface pointers */
	if (!m_pSupportErrorInfo)
		{
		m_pSupportErrorInfo = new CSupportErrorInfo(this);
		if (m_pSupportErrorInfo == NULL)
			{
			return FALSE;
			}
		}
	return TRUE;
	}
	
	
/******************************************************************************
Function:		ExitInstance
Purpose:  		This function deinitializes the partular instance of the 
				sp resolver
Parameters:		VOID
******************************************************************************/
BOOL CPubSpResolver::ExitInstance()
	{
	/* DeInitialize the embedded interface pointers */
	if (m_pSupportErrorInfo)
		{
		delete m_pSupportErrorInfo;
		m_pSupportErrorInfo = NULL;
		}
	if (m_fSourceConnectionExists || m_fDestConnectionExists)		
		m_ReplConnection.Disconnect();
		
	return TRUE;
	}


	
/******************************************************************************
Function:		CPubSpResolver::GetHandledStates
Purpose:  		This method fills in a bitmap of which states this reconciler
				handles.
Parameters:		Interface pointer for a RowChange object
******************************************************************************/
STDMETHODIMP CPubSpResolver::GetHandledStates(DWORD	*pResolverBm)
	{
	if (!m_fInitialized)
		return E_UNEXPECTED;
	
	*pResolverBm = REPOLEUpdateConflicts;
	return S_OK;
	}

/******************************************************************************
Function:		CPubSpResolver::Reconcile
Purpose:  		This method reconciles a single data change. Either an insert, 
	delete, or update to a base row.
Parameters:		Interface pointer for a RowChange object
******************************************************************************/
STDMETHODIMP CPubSpResolver::Reconcile(
	IReplRowChange *pRowChange,
    DWORD dwFlags,
    PVOID pvReserved)
	{
	HRESULT					hr = E_UNEXPECTED;
    REPOLE_CHANGE_TYPE 		ChangeType;
	DWORD 					dwNumColumns;
	DWORD					dwColid;
	TCHAR					szCommand[2048];
	TCHAR					szOwnerName[OSQL_SYSNAME];
	TCHAR					szTableName[OSQL_SYSNAME];
	TCHAR					szResolverProcedureName[2 * OSQL_SYSNAME + 2]; // should be owner qualified
	TCHAR					szRowGuid[OSQL_MAXSTRGUID];
	LPOLESTR 				pGuidStrAlloc;
	GUID					guidRow;
	LPVOID					pvBuffer = NULL;
	LONG					cbBuffer = 0;
	IConnectionInfo * 		pSourceConnectionInfo;
	IConnectionInfo * 		pDestConnectionInfo;
	BOOL					fSourceIsPublisher, fDestIsPublisher;
	BOOL					fAlreadyGotNull = FALSE;
	
	TCHAR					szSourceServer[OSQL_SYSNAME];
	TCHAR					szSourceDatabase[OSQL_SYSNAME];	
	TCHAR					szSourceLogin[OSQL_SYSNAME];	
	TCHAR					szSourcePassword[OSQL_SYSNAME];	
	REPOLE_SECURITY_TYPE	SourceSecurityMode;
	
	TCHAR					szDestServer[OSQL_SYSNAME];
	TCHAR					szDestDatabase[OSQL_SYSNAME];	
	TCHAR					szDestLogin[OSQL_SYSNAME];	
	TCHAR					szDestPassword[OSQL_SYSNAME];	
	REPOLE_SECURITY_TYPE	DestSecurityMode;
	
	LONG 					lColumnLength, lColumnSQLDataType, lColumnCDataType;
	BOOL					fUpdatable, fIdentity;

	if (!m_fInitialized)
		return E_UNEXPECTED;

	/*
	** Query the change type 
	*/
    CALLHRJ(pRowChange->GetChangeType(&ChangeType), Error);

	/*
	** SP based resolver currently handles conflicts - might be too expensive to invoke this for all change types.
	*/
	switch (ChangeType)
		{
		case REPOLEChange_SubscriberUpdate_ConflictNoColTrack:
		case REPOLEChange_PublisherUpdate_ConflictNoColTrack:
		case REPOLEChange_SubscriberUpdate_ConflictColTrack:
		case REPOLEChange_PublisherUpdate_ConflictColTrack:
			break;

		default:
			hr = E_UNEXPECTED;
			goto Error;	
			break;
		}


	/* 
	** Find out the resolver procedure name
	*/
	CALLHRJ(pRowChange->GetResolverProcedureName(szResolverProcedureName, OSQL_SYSNAME_SET), Error);
	
	/* 
	** Find out the table and owner name
	*/
	CALLHRJ(pRowChange->GetTableOwnerName(szOwnerName,  OSQL_SYSNAME_SET), Error);
	CALLHRJ(pRowChange->GetTableName(szTableName, OSQL_SYSNAME_SET), Error);

	/*
	** Get the row identifier and convert the RowGuid to a string.
	*/
	CALLHRJ(pRowChange->GetRowIdentifier(&guidRow), Error);
	hr = StringFromIID(guidRow, &pGuidStrAlloc);
	if FAILED(hr)
		return hr;
	/* Copy the RowGuid without the { and } brace */		
	_tcsncpy(szRowGuid, pGuidStrAlloc + 1, OSQL_MAXSTRGUID);
	szRowGuid[36] = _T('\0');
	CoTaskMemFree(pGuidStrAlloc);
	
	/* 
	** Get the source connection information
	*/
	fSourceIsPublisher = FALSE;
	CALLHRJ(pRowChange->GetSourceConnectionInfo(&pSourceConnectionInfo, &fSourceIsPublisher), Error);
	/* 
	** Get the dest connection information
	*/
	fDestIsPublisher = FALSE;
	CALLHRJ(pRowChange->GetDestinationConnectionInfo(&pDestConnectionInfo, &fDestIsPublisher), Error);


	/*
	** Always establish a connection to the publisher since this is the publisher side sp resolver
	*/
	if (fSourceIsPublisher)
		{
		pDestConnectionInfo->GetServerName(szDestServer, OSQL_SYSNAME);
		pDestConnectionInfo->GetDatabase(szDestDatabase, OSQL_SYSNAME);
		pDestConnectionInfo->GetSecurityMode(&DestSecurityMode);
		pDestConnectionInfo->GetLogin(szDestLogin, OSQL_SYSNAME);
		pDestConnectionInfo->GetPassword(szDestPassword, OSQL_SYSNAME);
		_stprintf(szCommand, SzCustomResolverProc, szResolverProcedureName, szOwnerName, szTableName, szRowGuid,
				szDestServer, szDestDatabase, DestSecurityMode, szDestLogin, szDestPassword);
		/* Establish a connection if it does not already exist */				
		if (!m_fSourceConnectionExists)
			{
			if (FAILED(hr = m_ReplConnection.Connect(pSourceConnectionInfo)))
				goto Error;
			m_fSourceConnectionExists = TRUE;
			}		
		}
	else		
		{
		Assert(fDestIsPublisher, _T("Dest is Publisher"));
		pSourceConnectionInfo->GetServerName(szSourceServer, OSQL_SYSNAME);
		pSourceConnectionInfo->GetDatabase(szSourceDatabase, OSQL_SYSNAME);
		pSourceConnectionInfo->GetSecurityMode(&SourceSecurityMode);
		pSourceConnectionInfo->GetLogin(szSourceLogin, OSQL_SYSNAME);
		pSourceConnectionInfo->GetPassword(szSourcePassword, OSQL_SYSNAME);
		_stprintf(szCommand, SzCustomResolverProc, szResolverProcedureName, szOwnerName, szTableName, szRowGuid,
				szSourceServer, szSourceDatabase, SourceSecurityMode, szSourceLogin, szSourcePassword);

		/* Establish a connection if it does not already exist */				
		if (!m_fDestConnectionExists)	
			{
			if (FAILED(hr = m_ReplConnection.Connect(pDestConnectionInfo)))
				goto Error;
			m_fDestConnectionExists = TRUE;
			}
		}
		

	CALLHRJ(m_ReplConnection.ExecuteDirect(szCommand), Error);

	CALLHRJ(m_ReplConnection.Fetch(), Error);

	/* There should be one result set that returns the resolved data for the row */
	if(!m_ReplConnection.FMoreData())
		{
		goto Error;
		}

	/* 
	** Find out how many columns so we can loop over them.
	*/
	CALLHRJ(pRowChange->GetNumColumns(&dwNumColumns), Error);

	// Get all columns of the user table
	for (dwColid = 1; dwColid <= dwNumColumns; dwColid++)
		{
		/* Get the column information out of the result set */

		CALLHRJ(m_ReplConnection.GetColumnDatatype(dwColid, &lColumnSQLDataType), Error);
		lColumnCDataType = GetCTypeFromSqlType((short)lColumnSQLDataType);
		CALLHRJ(m_ReplConnection.GetColumnLength(dwColid, &lColumnLength), Error);
		CALLHRJ(m_ReplConnection.GetColumnUpdatable(dwColid, (LONG *)&fUpdatable, (LONG *)&fIdentity), Error);

		fAlreadyGotNull = FALSE;
			
		// What is the length?
		if (lColumnSQLDataType == SQL_LONGVARCHAR ||
			lColumnSQLDataType == SQL_WLONGVARCHAR ||
			lColumnSQLDataType == SQL_LONGVARBINARY)
			{
			CALLHRJ(m_ReplConnection.GetData(dwColid, lColumnCDataType, &cbBuffer, 0, &cbBuffer), Error);
			if (cbBuffer == SQL_NULL_DATA)
				{
				//  Use a valid cbBuffergth for allocation, etc.
				cbBuffer = 255 * 2 ;
				fAlreadyGotNull = TRUE;
				}
			}
		else
			{
			cbBuffer = lColumnLength;
			}
				
		// Bump up length to allow null terminator for character and wide character columns
		if (lColumnCDataType == SQL_C_CHAR )
			{
			cbBuffer += 1;
			}

		if (lColumnCDataType == SQL_C_WCHAR )
			{
			cbBuffer += 2;
			}

		// We have to allocate a new buffer.
		pvBuffer = malloc(cbBuffer);
		if (!pvBuffer)
			{
			hr = E_OUTOFMEMORY;
			goto Error;
			}

		if (fAlreadyGotNull)
			{
			cbBuffer = SQL_NULL_DATA;
			}
		else
			{
			// Get the data
			CALLHRJ(m_ReplConnection.GetData(dwColid, lColumnCDataType, pvBuffer,
				 cbBuffer, &cbBuffer), Error);
			}

		CALLHRJ(pRowChange->SetColumn(dwColid, pvBuffer, cbBuffer), Error);

		if (pvBuffer)
			free(pvBuffer);
		}


	CALLHRJ(pRowChange->UpdateRow(), Error);

	hr = NOERROR;

Error:
	m_ReplConnection.CloseCursor();
	if (pSourceConnectionInfo)
		pSourceConnectionInfo->Release();
	if (pDestConnectionInfo)
		pDestConnectionInfo->Release();
	m_ReplConnection.ConstructOutcome();
	OleUtilSetErrorInfo(_T("Microsoft SQLServer Publisher Stored Procedure Resolver"), m_ReplConnection.m_szOutcome);
	return (hr);		
	}
			

void OleUtilSetErrorInfo(LPTSTR szSource, LPTSTR szDescription, LPCTSTR szHelpFile, DWORD dwHelpContext, LPIID pGUID)
	{
	LPCREATEERRORINFO 	pCreateErrorInfo = NULL;
	LPERRORINFO 		pErrorInfo = NULL;

	// Fill in the system (thread-specific) ErrorInfo object.
	// First see if there's already error info there.  If so, leave it there.
	if (GetErrorInfo(0, &pErrorInfo) && pErrorInfo) 
		{
		SetErrorInfo(0, pErrorInfo);
		pErrorInfo->Release();
		return;
		}

	HRESULT hr = CreateErrorInfo(&pCreateErrorInfo);
	if (SUCCEEDED(hr))
		{
#ifdef UNICODE
		pCreateErrorInfo->SetSource(szSource);
		pCreateErrorInfo->SetDescription(szDescription);
#else
		pCreateErrorInfo->SetSource(ATOW_STR(szSource));
		pCreateErrorInfo->SetDescription(ATOW_STR(szDescription));
#endif //UNICODE
		
		// pCreateErrorInfo->SetHelpFile(const_cast<LPOLESTR>(wzHelpFile));
		// pCreateErrorInfo->SetHelpContext(dwHelpContext);
		// pCreateErrorInfo->SetGUID(pIID ? *pIID : IID_NULL);
	
		hr = pCreateErrorInfo->QueryInterface(IID_IErrorInfo, (LPVOID FAR*) &pErrorInfo);
		if (SUCCEEDED(hr))
			{
			SetErrorInfo(0, pErrorInfo);
			pErrorInfo->Release();
			}
		}		
	if (pCreateErrorInfo)		
		pCreateErrorInfo->Release();
	}

