set nocount on 
go


if object_id('conflict_losers') is not null
   drop table conflict_losers
go


/* This is the table that will store the copies of the conflicting row */
create table conflict_losers
(
        value varchar(20),
    authority tinyint,
      guidcol uniqueidentifier,
  merge_stamp datetime,
     notified bit
)
go


if object_id('resolver_log') is not null
   drop table resolver_log
go


/* This is the table that will log the conflict calls */

create table resolver_log
( 
       log_id int identity,
      guidcol uniqueidentifier,
  merge_stamp datetime,
resolver_info varchar(255)

)
go

if object_id('sp_authority_resolver') is not null
   drop procedure sp_authority_resolver
go

create procedure sp_authority_resolver
           @tablename sysname,          /* Assuming it is owned by dbo for now */  
             @rowguid uniqueidentifier, 
              @publisher sysname,          /* source server */ 
           @publisher_db sysname,          /* source db */ 
@publisher_security_mode int = NULL,  
        @publisher_login sysname = NULL,
     @publisher_password sysname = NULL
as

/* *********************************************************************** */
/* This procedure is called by replmerg.exe when a conflict is encountered */
/* in the merge conflict. It retrieves the 'authority' value from the      */
/* table at publisher and subscriber, and the row with the highest         */
/* authority will 'win' and be returned to replmerg.exe to be applied to   */
/* both servers.  If both rows have the same authority, the source table   */
/* will win.                                                               */
/* *********************************************************************** */

set nocount on 

declare @stmt varchar(1000), @stamp datetime
select @stamp = getdate()


insert into resolver_log (guidcol,merge_stamp,resolver_info) 
values (@rowguid,@stamp,'Resolver invoked')

/* No hetero query support for now */ 
if @publisher <> @@servername 
begin
   raiserror('Source and dest tables must be in the current server',22,127)
   return(1)
end 

/* This table will store both copies of the conflicting row */ 
create table #resolver_data 
(      origin char(1),
        value varchar(20),
    authority tinyint,
      guidcol uniqueidentifier
)

/* Get the publisher row */
select @stmt = 
'select ''s'', value, authority, guidcol from ' + @publisher_db + '.dbo.' + @tablename +
' where guidcol = ''' + convert(nchar(36),@rowguid) + ''''

-- print @stmt

/* Load it into the temp table */
insert into #resolver_data (origin,value,authority,guidcol) 
exec (@stmt)

/* Get the dest row */
select @stmt = 
'select ''d'', value, authority, guidcol from '  + @tablename +
' where guidcol = ''' + convert(nchar(36),@rowguid) + ''''

-- print @stmt
/* Load it into the temp table */
insert into #resolver_data (origin,value,authority,guidcol) 
exec (@stmt)

/* Load the authorities into the log table */
insert into resolver_log (guidcol,merge_stamp,resolver_info) 
select @rowguid,@stamp,'Sub Authority='+convert(varchar(6),authority) 
from #resolver_data where origin = 's'

insert into resolver_log (guidcol,merge_stamp,resolver_info) 
select @rowguid,@stamp,'Pub Authority='+convert(varchar(6),authority) 
from #resolver_data where origin = 'd'


if (select authority from #resolver_data where origin = 's') >= 
   (select authority from #resolver_data where origin = 'd')
begin 

   /* Sub table wins - log the losing row in the Pub db */ 
   select @stmt = 
   'insert into  conflict_losers 
   (value,authority,guidcol,merge_stamp,notified)
   select value,authority,guidcol,getdate(),0
   from #resolver_data 
   where origin = ''d'''

   exec (@stmt)

   insert into resolver_log values (@rowguid,@stamp,'Sub row won')


   /* Return the Src row to be applied to both the source and the dest */
   select value, authority, guidcol 
   from #resolver_data
   where origin = 's'

   return(0)
end    
else
begin 
 
   /* Dest table wins - log the losing row in the src db */ 
   select @stmt = 
   'insert into ' + @publisher_db + '..conflict_losers 
   (value,authority,guidcol,merge_stamp,notified)
   select value,authority,guidcol,getdate(),0
   from #resolver_data 
   where origin = ''s'''

   exec (@stmt)

   insert into resolver_log values (@rowguid,@stamp,'Pub row won')


   /* Return the Dest row to be applied to both the source and the dest */
   select value, authority, guidcol 
   from #resolver_data
   where origin = 'd'

   return(0)
end  
go

if object_id('add_data') is not null
   drop procedure add_data
go
set nocount on
go


create procedure add_data
@rowcount int = null
as 

/* ********************************************************************** */
/* This procedure loads the specified number of rows, all have authority  */
/* of zero. The row is updated later with a new value and authority and   */
/* it is involved in a conflict, the authority of the row will be used by */
/* the resolver sp to determine which update succeeds.                    */
/* ********************************************************************** */

set nocount on

if @rowcount = 0
begin
   raiserror('Error: No rowcount value was specified',22,127) with log
   return(1)
end

declare @counter int, @value varchar(20)
select @counter = 0

begin tran

while @counter <= @rowcount - 1
begin 
   select @counter = @counter + 1
   select @value = db_name() + ' Row# ' + convert(varchar(8),@counter)
   insert into testdata (value,authority) values (@value,0)
   if @@error <> 0 
      goto inserror
end
goto done

inserror:
rollback tran
raiserror('Error: Insert Failed',22,127) with log
return(1)

done:
print convert(varchar(6),@counter) + ' rows were successfully installed'
commit tran 
return(0)
go


