#define INITGUID
#define DBINITCONSTANTS
#include <direct.h>
#include <stdio.h>
#include <repldist.h>

//
// Function Prototypes
//
HRESULT DoSnapShot();
HRESULT DoLogReader();
HRESULT GetDistributionDataSource (
	IDBInitialize**	ppIDBInitialize,
	WCHAR			*szAppName,
	DISTTYPEENUM	distType);
HRESULT RetrieveError (
	REFIID ErrorIID, 
	IUnknown *	pUnkErrorInterface, 
	CHAR * szErrorString, 
	DISTERRORDESC ** ppDistErrorDesc, 
	INT * pNumErrors ) ;
void PrintError(
	CHAR * szErrorMessage, 
	DISTERRORDESC * pDistErrorDesc, 
	INT cDistErrorDescs ) ;

//
// Global variable declarations
//
DISTCOMMANDDESC aCommand[60];
char pszCmdBuf[60][255];

//
// Constant declarations
//
const char szDir[] = ":\\mssql7\\repldata\\unc\\samppub\\";
const char szScriptCmd[] = "%ssamptab.sch";
const char szBCPCmd[] = "sync -t\"SampleTable1\" -d\"%ssamptab.bcp\" -u";
const char szCmdUpdate[] = "update SampleTable1 set c2 = 'Number %d' where c1 = %d";
const WCHAR wszSnapShot[] = L"SampleSnapShotAgent";
const WCHAR wszLogreader[] = L"SampleLogReaderAgent";

#define  MAX_MESSAGE_LEN			1024	// Max. Length of an error message string
INT NumCmds = 30;

//
// Main module
//		Calls the routines to do Snapshot and Logreader work
//
void main()
{
	HRESULT hr = NOERROR;

	printf ("Sample Publisher Agent Startup.\n");

	// Initialize OLE
	hr = OleInitialize(NULL);
	if (FAILED(hr))
	{
		printf("\n Unable to initialize OLE.\n") ;
		goto CLEANUP ;
	}
	
	hr = DoSnapShot();
	if (FAILED(hr))
	{
		printf("\n SnapShot Failed\n") ;
		goto CLEANUP ;
	}

	hr = DoLogReader();
	if (FAILED(hr))
	{
		printf("\n LogReader Failed\n") ;
		goto CLEANUP ;
	}

CLEANUP :

	// Uninitialize OLE.
	OleUninitialize();

	return;
}

//
// This routine performs the Snapshot
//
HRESULT DoSnapShot()
{
	HRESULT				hr = NOERROR;
	IDBInitialize*		pIDBInitialize = NULL ;
	IDBCreateSession*	pIDBCS = NULL;
	IDistributionStore* pIDistStore = NULL;
	IDistributionLog*	pIDistLog = NULL;
	IUnknown *			pErrorInterface ;
	IID					ErrorIID ;	
	DistByteArray		XactId;
	DistByteArray		XactSeqno;
	INT					NumTransactions = 0; 
	INT					NumCommands = 0;
	INT					NumErrors = 0;
	INT					i = 0;
	CHAR				szMessage[256];
	CHAR				szErrorString[MAX_MESSAGE_LEN] ;
	CHAR				szWrkDir[_MAX_PATH];
	DISTERRORDESC *		pDistErrorDesc = NULL ;

	// Get the current working dir
	sprintf(szWrkDir, "%c", (char)(_getdrive() + 'A' - 1) );
	strcat(szWrkDir, szDir);
	
	// Initialize the distribution data source
	hr = GetDistributionDataSource (&pIDBInitialize, (WCHAR *)wszSnapShot, DISTTYPE_SNAPSHOT);
	if (FAILED(hr))
	{
		printf("\n Unable to connect to distribution data source.\n") ;
		goto CLEANUP ;
	}
	
	// Request the IDBCreateSession interface
	hr = pIDBInitialize->QueryInterface(IID_IDBCreateSession, (void**) &pIDBCS);
	if (FAILED(hr))
	{
		printf("\n CreateSession interface not supported.\n") ;
		goto CLEANUP ;
	}

	// Create a session returning a pointer to its IDistributionLog interface
	hr = pIDBCS->CreateSession(NULL, IID_IDistributionLog, (LPUNKNOWN*) &pIDistLog);
	if (FAILED(hr))
	{
		printf("\n Unable to create new session on data source.\n") ;
		goto CLEANUP ;
	}

	// Get the IDistributionStore interface
	hr = pIDistLog->QueryInterface(IID_IDistributionStore, (void**) &pIDistStore);
	if (FAILED(hr))
	{
		printf("\n Distribution Store interface not supported") ;
		goto CLEANUP ;
	}

	// Log a history message
	hr = pIDistLog->AddLog(DISTLOG_STARTUP, "Sample Publisher Snapshot Agent Startup", NULL, 
								NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	// Get the last transaction id and sequence Number from the SQL Distributor
	hr = pIDistStore->GetLastTransaction((BYTE *)&XactId, (BYTE *)&XactSeqno);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionStore ;
		goto ERRORLOOKUP ;
	}
	NumTransactions++;
	XactId++;
	XactSeqno++;

	//
	// Prepare the commands for Snapshot
	//

	// Set working dir - NOTE : The article id should always be ZERO
	// for this COMMAND
	NumCommands++;
	aCommand[i].PublicationId = 1;
	aCommand[i].ArticleId = 0;
	aCommand[i].CommandId = NumCommands;
	aCommand[i].CommandType = DISTCMDTYPE_SQL_WORKINGDIR;
	aCommand[i].fPartialCommand = FALSE;
	aCommand[i].pXactId = (BYTE *)&XactId;
	aCommand[i].pXactSeqno = (BYTE *)&XactSeqno;
	aCommand[i].szOriginator = NULL;
	aCommand[i].szOriginatorDB = NULL;
	aCommand[i].szCommand = (LPSTR)szWrkDir;
	i++;

	// Execute Script - table schema
	NumCommands++;
	aCommand[i].PublicationId = 1;
	aCommand[i].ArticleId = 1;
	aCommand[i].CommandId = NumCommands;
	aCommand[i].CommandType = DISTCMDTYPE_SCRIPT;
	aCommand[i].fPartialCommand = FALSE;
	aCommand[i].pXactId = (BYTE *)&XactId;
	aCommand[i].pXactSeqno = (BYTE *)&XactSeqno;
	aCommand[i].szOriginator = NULL;
	aCommand[i].szOriginatorDB = NULL;

	sprintf(pszCmdBuf[i], szScriptCmd, szWrkDir);
	aCommand[i].szCommand = pszCmdBuf[i];
	i++;

	// Import Data - char BCP
	NumCommands++;
	aCommand[i].PublicationId = 1;
	aCommand[i].ArticleId = 1;
	aCommand[i].CommandId = NumCommands;
	aCommand[i].CommandType = DISTCMDTYPE_CHAR_BCP;
	aCommand[i].fPartialCommand = FALSE;
	aCommand[i].pXactId = (BYTE *)&XactId;
	aCommand[i].pXactSeqno = (BYTE *)&XactSeqno;
	aCommand[i].szOriginator = NULL;
	aCommand[i].szOriginatorDB = NULL;
	
	sprintf(pszCmdBuf[i], szBCPCmd, szWrkDir);
	aCommand[i].szCommand = pszCmdBuf[i];
	i++;

	// Add the transaction commands
	hr = pIDistStore->AddTransactionCommands(i, aCommand);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionStore ;
		goto ERRORLOOKUP ;
	}

	// Log a history messages
	sprintf (szMessage, "Added %d Transaction(s) consisting of %d Commands(s).", 
					NumTransactions, NumCommands);
	printf ("%s\n", szMessage);
	hr = pIDistLog->AddLog(DISTLOG_INPROGRESS, szMessage, (BYTE *)&XactSeqno, 
						NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	sprintf (szMessage, "Sample Publisher Snapshot Agent Succeeded.");
	printf ("%s\n", szMessage);
	hr = pIDistLog->AddLog(DISTLOG_SUCCESS, szMessage, (BYTE *)&XactSeqno, 
						NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))			
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	goto CLEANUP ;
	
ERRORLOOKUP :
	// Pointer to interface causing the error
	if (IID_IDistributionLog == ErrorIID)
		pErrorInterface = pIDistLog ;
	else
		pErrorInterface = pIDistStore ;
	
	hr = RetrieveError(ErrorIID, pErrorInterface, szErrorString, &pDistErrorDesc, &NumErrors) ;
	if (!FAILED(hr))
	{
		PrintError(szErrorString, pDistErrorDesc, NumErrors) ;
		
		// Log the error message
		if (pIDistLog && IID_IDistributionStore == ErrorIID)	
			pIDistLog->AddLog(DISTLOG_FAILURE, szErrorString, NULL, 
				NumTransactions, NumCommands, NumErrors, pDistErrorDesc );
	}
	
CLEANUP :
	if (pIDistLog)
		pIDistLog->Release();
	if (pIDistStore)
		pIDistStore->Release();
	if (pIDBCS)
		pIDBCS->Release();
	if (pIDBInitialize)
		pIDBInitialize->Release();
	if (pDistErrorDesc)
		delete [] pDistErrorDesc ;

	return hr;
}

//
// This routine performs the LogReader
//
HRESULT DoLogReader()
{
	HRESULT				hr = NOERROR;
	IDBInitialize*		pIDBInitialize = NULL ;
	IDBCreateSession*	pIDBCS = NULL;
	IDistributionStore* pIDistStore = NULL;
	IDistributionLog*	pIDistLog = NULL;
	IUnknown *			pErrorInterface ;
	IID					ErrorIID ;	
	DistByteArray		XactId;
	DistByteArray		XactSeqno;
	INT					NumTransactions = 0; 
	INT					NumCommands = 0;
	INT					NumErrors = 0;
	INT					i = 0;
	CHAR				szMessage[256];
	CHAR				szErrorString[MAX_MESSAGE_LEN] ;
	DISTERRORDESC *		pDistErrorDesc = NULL ;
	
	// Initialize the distribution data source
	hr = GetDistributionDataSource (&pIDBInitialize, (WCHAR *)wszLogreader, DISTTYPE_LOGREADER);
	if (FAILED(hr))
	{
		printf("\n Unable to connect to distribution data source.\n") ;
		goto CLEANUP ;
	}
	
	// Request the IDBCreateSession interface
	hr = pIDBInitialize->QueryInterface(IID_IDBCreateSession, (void**) &pIDBCS);
	if (FAILED(hr))
	{
		printf("\n CreateSession interface not supported.\n") ;
		goto CLEANUP ;
	}

	// Create a session returning a pointer to its IDistributionLog interface
	hr = pIDBCS->CreateSession(NULL, IID_IDistributionLog, (LPUNKNOWN*) &pIDistLog);
	if (FAILED(hr))
	{
		printf("\n Unable to create new session on data source.\n") ;
		goto CLEANUP ;
	}

	// Get the IDistributionStore interface
	hr = pIDistLog->QueryInterface(IID_IDistributionStore, (void**) &pIDistStore);
	if (FAILED(hr))
	{
		printf("\n Distribution Store interface not supported") ;
		goto CLEANUP ;
	}

	// Log a history message
	hr = pIDistLog->AddLog(DISTLOG_STARTUP, "Sample Publisher Logreader Agent Startup", NULL, 
								NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	// Get the last transaction id and sequence Number from the SQL Distributor
	hr = pIDistStore->GetLastTransaction((BYTE *)&XactId, (BYTE *)&XactSeqno);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionStore ;
		goto ERRORLOOKUP ;
	}
	
	//
	// As Logreader Agent
	//
	NumTransactions++;
	XactId++;
	XactSeqno++;

	for (i = 0, NumCommands=0; i < NumCmds; i++)
	{
		NumCommands++;
		aCommand[i].PublicationId = 1;
		aCommand[i].ArticleId = 1;
		aCommand[i].CommandId = NumCommands;
		aCommand[i].CommandType = DISTCMDTYPE_SQL;
		aCommand[i].fPartialCommand = FALSE;

		sprintf(pszCmdBuf[i], szCmdUpdate, NumCommands+1, NumCommands+1);
		aCommand[i].szCommand = pszCmdBuf[i];

		aCommand[i].pXactId = (BYTE *)&XactId;
		aCommand[i].pXactSeqno = (BYTE *)&XactSeqno;
		aCommand[i].szOriginator = NULL;
		aCommand[i].szOriginatorDB = NULL;
	}
	
	// Add the transaction commands
	hr = pIDistStore->AddTransactionCommands(i, aCommand);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionStore ;
		goto ERRORLOOKUP ;
	}

	// Log a history messages
	sprintf (szMessage, "Added %d Transaction(s) consisting of %d Commands(s).", 
					NumTransactions, NumCommands);
	printf ("%s\n", szMessage);
	hr = pIDistLog->AddLog(DISTLOG_INPROGRESS, szMessage, (BYTE *)&XactSeqno, 
						NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	sprintf (szMessage, "Sample Publisher Logreader Agent Succeeded.");
	printf ("%s\n", szMessage);
	hr = pIDistLog->AddLog(DISTLOG_SUCCESS, szMessage, (BYTE *)&XactSeqno, 
						NumTransactions, NumCommands, NumErrors, NULL);
	if (FAILED(hr))			
	{
		ErrorIID = IID_IDistributionLog ;
		goto ERRORLOOKUP ;
	}

	goto CLEANUP ;


ERRORLOOKUP :
	// Pointer to interface causing the error
	if (IID_IDistributionLog == ErrorIID)
		pErrorInterface = pIDistLog ;
	else
		pErrorInterface = pIDistStore ;
	
	hr = RetrieveError(ErrorIID, pErrorInterface, szErrorString, &pDistErrorDesc, &NumErrors) ;
	if (!FAILED(hr))
	{
		PrintError(szErrorString, pDistErrorDesc, NumErrors) ;
		
		// Log the error message
		if (pIDistLog && IID_IDistributionStore == ErrorIID)	
			pIDistLog->AddLog(DISTLOG_FAILURE, szErrorString, NULL, 
				NumTransactions, NumCommands, NumErrors, pDistErrorDesc );
	}
	
CLEANUP :
	if (pIDistLog)
		pIDistLog->Release();
	if (pIDistStore)
		pIDistStore->Release();
	if (pIDBCS)
		pIDBCS->Release();
	if (pIDBInitialize)
		pIDBInitialize->Release();
	if (pDistErrorDesc)
		delete [] pDistErrorDesc ;

	return hr;
}

//
// Connect to a Distribution Data Source and get a pointer to its IDBInitialize interface
//
HRESULT GetDistributionDataSource (
	IDBInitialize**	ppIDBInitialize,
	WCHAR			*szAppName,
	DISTTYPEENUM	distType)
{
	IDBInitialize*	pIDBInit = NULL;
	IDBProperties*	pIDBProperties = NULL;
	DBPROP			rgProperties[12];
	DBPROPSET		PropertySet;
	HRESULT			hr;
	INT				NumErrors ;
	DISTERRORDESC * pDistErrorDesc ;
	CHAR			szErrorString[MAX_MESSAGE_LEN] ;
	
	// Instantiate a data source object for the SQL Publishing provider.
	hr = CoCreateInstance(CLSID_SQLDistribution, NULL, CLSCTX_INPROC_SERVER, IID_IDBInitialize,
		(void**) &pIDBInit);

	// Initialize the VARIANTs and the options in rgProperties.
	for (ULONG i = 0; i < 12; i++) {
	   VariantInit(&rgProperties[i].vValue);
	   rgProperties[i].dwOptions = DBPROPOPTIONS_REQUIRED;
	   if (i == 6 || i == 7)
		   rgProperties[i].vValue.vt = VT_I2;
	   if (i == 10)
		   rgProperties[i].vValue.vt = VT_I1;
	   if (i == 11)
		   rgProperties[i].vValue.vt = VT_I4;
	   else
		   rgProperties[i].vValue.vt = VT_BSTR;
	};

	rgProperties[0].dwPropertyID   = DBPROP_INIT_LOCATION;
	rgProperties[0].vValue.bstrVal =
	   SysAllocStringLen(OLESTR(""), wcslen(OLESTR("")));

	rgProperties[1].dwPropertyID   = DBPROP_INIT_DATASOURCE;
	rgProperties[1].vValue.bstrVal =
	   SysAllocStringLen(OLESTR("distribution"), wcslen(OLESTR("distribution")));

	rgProperties[2].dwPropertyID   = DBPROP_AUTH_PASSWORD;
	rgProperties[2].vValue.bstrVal =
	   SysAllocStringLen(OLESTR(""), wcslen(OLESTR("")));

	rgProperties[3].dwPropertyID   = DBPROP_AUTH_USERID;
	rgProperties[3].vValue.bstrVal =
	   SysAllocStringLen(OLESTR("sa"), wcslen(OLESTR("sa")));

	rgProperties[4].dwPropertyID   = DBPROP_INIT_PUBLISHER_NAME;
	rgProperties[4].vValue.bstrVal =
	   SysAllocStringLen(OLESTR("SamplePublisher"), wcslen(OLESTR("SamplePublisher")));

	rgProperties[5].dwPropertyID   = DBPROP_INIT_PUBLISHER_DATASOURCE;
	rgProperties[5].vValue.bstrVal =
	   SysAllocStringLen(OLESTR("SampleDatabase"), wcslen(OLESTR("SampleDatabase")));

	rgProperties[6].dwPropertyID   = DBPROP_INIT_XACTID_SIZE;
	rgProperties[6].vValue.bVal = sizeof(DistByteArray);

	rgProperties[7].dwPropertyID   = DBPROP_INIT_XACT_SEQNO_SIZE;
	rgProperties[7].vValue.bVal = sizeof(DistByteArray);

	rgProperties[8].dwPropertyID   = DBPROP_PUBLICATION_NAME;
	rgProperties[8].vValue.bstrVal =
	   SysAllocStringLen(OLESTR("SamplePublication"), wcslen(OLESTR("SamplePublication")));

	rgProperties[9].dwPropertyID   = DBPROP_APPLICATION_NAME;
	rgProperties[9].vValue.bstrVal = SysAllocStringLen(szAppName, wcslen(szAppName));

	rgProperties[10].dwPropertyID  = DBPROP_APPLICATION_TYPE;
	rgProperties[10].vValue.bVal   = distType;

	rgProperties[11].dwPropertyID  = DBPROP_INIT_TIMEOUT ;
	rgProperties[11].vValue.lVal   = 5 ;

	// Create the initialization structure.
	PropertySet.rgProperties    = rgProperties;
	PropertySet.cProperties     = 12;
	PropertySet.guidPropertySet = DBPROPSET_DBINIT;

	// Set the initialization properties.
	pIDBInit->QueryInterface(IID_IDBProperties, (void**) &pIDBProperties);
	pIDBProperties->SetProperties(1, &PropertySet);
	pIDBProperties->Release();

	// Initialize the SQL Distributor object.
	hr = pIDBInit->Initialize();

	if( FAILED(hr) )
	{
		RetrieveError(IID_IDBInitialize, pIDBInit, szErrorString, &pDistErrorDesc, &NumErrors);

		PrintError(szErrorString, pDistErrorDesc, NumErrors) ;
		if (pDistErrorDesc)
			delete [] pDistErrorDesc ;
	
		return hr ;
	}

	*ppIDBInitialize = pIDBInit;

	return S_OK ;
}


//
// Retrieve OLE DB errors using IErrorInfo and IErrorRecords interfaces, and return an 
// error text and its associated DISTERRORDESC error descriptors
//
HRESULT RetrieveError (
			REFIID				ErrorIID,	// IN | IID of the interface that caused the error
			IUnknown *			pUnkErrorInterface, // IN | Pointer to interface that caused the error
			CHAR *				szErrorString,	// OUT | The actual provider-supplied error string
			DISTERRORDESC **	ppDistErrorDesc, // OUT | An array of DISTERRORDESC structs returned by the provider
			INT *				pNumErrors // OUT | # of entries in the DISTERRORDESC array
)
{
	// Handle the error using the error lookup service and the custom error object
	ISupportErrorInfo	*	pISupportErrorInfo = NULL ;
	IErrorInfo			*	pIErrorInfo = NULL ;
	IErrorInfo			*	pErrorInfoRec = NULL ;
	IErrorRecords		*	pIErrorRecords = NULL ;
	IGetCustomErrorInfo	*	pICustomErrorInfo = NULL ;
	DISTERRORDESC		*	pDistErrorDesc ;
	HRESULT					hr ;
	ULONG					cErrorRecords, nLoopVar ;
	BSTR					bstrErrorDescription = NULL, bstrErrorSource = NULL ;
					
	if (IID_IDistributionLog != ErrorIID && 
			IID_IDistributionStore != ErrorIID &&
				IID_IDBInitialize != ErrorIID)
	{
		hr = E_FAIL ;
		goto HANDLEERROR ;
	}

	// Get a pointer to the ISupportErrorInfo interface
	hr = pUnkErrorInterface->QueryInterface(IID_ISupportErrorInfo, (void **)&pISupportErrorInfo) ;
	if (FAILED(hr))
		goto HANDLEERROR ;

	// Does the called interface support error objects ?
	hr = pISupportErrorInfo->InterfaceSupportsErrorInfo(ErrorIID) ;
	if (S_OK != hr)
		goto HANDLEERROR ;

	hr = GetErrorInfo(0L, &pIErrorInfo) ;
	if (S_OK != hr)
		goto HANDLEERROR ;

	pIErrorInfo->QueryInterface(IID_IErrorRecords, (LPVOID FAR *)&pIErrorRecords) ;
	pIErrorRecords->GetRecordCount(&cErrorRecords) ;
			
	pDistErrorDesc = new DISTERRORDESC[sizeof(DISTERRORDESC) * cErrorRecords] ;
	*pNumErrors = 0 ;
			
	for (nLoopVar = 0 ; nLoopVar < cErrorRecords ; nLoopVar++)
	{
		// Get a pointer to the IErrorInfo interface of an error record
		pIErrorRecords->GetErrorInfo(nLoopVar, GetUserDefaultLCID(), &pErrorInfoRec) ;

		// Get the description and the source of the error in the error record, 
		// as given by the provider
		pErrorInfoRec->GetDescription(&bstrErrorDescription) ;	
		pErrorInfoRec->GetSource(&bstrErrorSource) ;

		// Convert Unicode Source and Description strings to ANSI
		wcstombs(szErrorString, bstrErrorDescription, MAX_MESSAGE_LEN) ;
						
		// Get the custom error object stored in the error record, if any
		hr = pIErrorRecords->GetCustomErrorObject(nLoopVar, IID_IGetCustomErrorInfo, (LPUNKNOWN *)&pICustomErrorInfo) ;
		if (SUCCEEDED(hr) && pICustomErrorInfo)
		{
			pICustomErrorInfo->GetCustomErrorInfo(&pDistErrorDesc[*pNumErrors]) ;
			if (!(*pDistErrorDesc[*pNumErrors].szSourceName) && bstrErrorSource)
				wcstombs(pDistErrorDesc[*pNumErrors].szSourceName, bstrErrorSource, MAX_MESSAGE_LEN) ;

				(*pNumErrors)++;
		}

		SysFreeString(bstrErrorDescription) ;
		SysFreeString(bstrErrorSource) ;

		pErrorInfoRec->Release() ;
	}
				
	pIErrorRecords->Release() ;
	pIErrorInfo->Release() ;

	*ppDistErrorDesc = pDistErrorDesc ;

	return S_OK ;

HANDLEERROR:
	*ppDistErrorDesc = NULL ;
	*pNumErrors = 0 ;

	return (hr);
}

//
// Print an error message and its associated descriptors
//
void PrintError( CHAR * szErrorMessage, DISTERRORDESC * pDistErrorDesc, INT cDistErrorDescs )
{
	INT nCount = 0 ;
	
	if (!cDistErrorDescs)
		return ;

	printf ("Sample Publisher Agent Failed.\n");
	printf("\n # of errors : %d", cDistErrorDescs) ;
	printf("\n Provider Error Message : %s\n", szErrorMessage) ;

	for (; nCount < cDistErrorDescs ; nCount ++)
	{
		printf("\n Error Descriptor # %d :", nCount+1) ;
		printf("\n Source Type		= %d", pDistErrorDesc[nCount].SourceType) ;
		printf("\n Source Name		= %s", pDistErrorDesc[nCount].szSourceName) ;
		printf("\n Native ErrorCode	= %s", pDistErrorDesc[nCount].szErrorCode) ;
		printf("\n Actual Error		= %s\n", pDistErrorDesc[nCount].bstrErrorText) ;
	}
}
